/* Generated by Cython 0.29.17 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "name": "octobot.strategy_optimizer.strategy_optimizer",
        "sources": [
            "octobot/strategy_optimizer/strategy_optimizer.py"
        ]
    },
    "module_name": "octobot.strategy_optimizer.strategy_optimizer"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_17"
#define CYTHON_HEX_VERSION 0x001D11F0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
#if PY_VERSION_HEX >= 0x030800A4 && PY_VERSION_HEX < 0x030800B2
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, 0, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : (Py_INCREF(func), func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__octobot__strategy_optimizer__strategy_optimizer
#define __PYX_HAVE_API__octobot__strategy_optimizer__strategy_optimizer
/* Early includes */
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "octobot/strategy_optimizer/strategy_optimizer.py",
  "octobot/strategy_optimizer/strategy_optimizer.pxd",
  "stringsource",
};

/*--- Type declarations ---*/
struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer;
struct __pyx_opt_args_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_find_optimal_configuration;

/* "octobot/strategy_optimizer/strategy_optimizer.pxd":38
 *     cdef public list run_results
 * 
 *     cpdef void find_optimal_configuration(self, list TAs=*, list time_frames=*, list risks=*)             # <<<<<<<<<<<<<<
 *     cpdef void print_report(self)
 *     cpdef int get_overall_progress(self)
 */
struct __pyx_opt_args_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_find_optimal_configuration {
  int __pyx_n;
  PyObject *TAs;
  PyObject *time_frames;
  PyObject *risks;
};

/* "octobot/strategy_optimizer/strategy_optimizer.pxd":18
 * #  License along with this library.
 * 
 * cdef class StrategyOptimizer:             # <<<<<<<<<<<<<<
 *     cdef object logger
 *     cdef dict config
 */
struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer {
  PyObject_HEAD
  struct __pyx_vtabstruct_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_vtab;
  PyObject *logger;
  PyObject *config;
  PyObject *tentacles_setup_config;
  PyObject *sorted_results_by_time_frame;
  PyObject *sorted_results_through_all_time_frame;
  PyObject *current_test_suite;
  PyObject *errors;
  int run_id;
  int total_nb_runs;
  int is_properly_initialized;
  PyObject *trading_mode;
  PyObject *strategy_class;
  PyObject *all_time_frames;
  PyObject *all_TAs;
  PyObject *risks;
  int is_computing;
  PyObject *run_results;
};



/* "octobot/strategy_optimizer/strategy_optimizer.py":47
 * 
 * 
 * class StrategyOptimizer:             # <<<<<<<<<<<<<<
 *     """
 *     StrategyOptimizer is a tool that performs backtesting with different configurations
 */

struct __pyx_vtabstruct_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer {
  void (*find_optimal_configuration)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, int __pyx_skip_dispatch, struct __pyx_opt_args_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_find_optimal_configuration *__pyx_optional_args);
  void (*print_report)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, int __pyx_skip_dispatch);
  int (*get_overall_progress)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, int __pyx_skip_dispatch);
  int (*is_in_progress)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, int __pyx_skip_dispatch);
  int (*get_current_test_suite_progress)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, int __pyx_skip_dispatch);
  PyObject *(*get_report)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, int __pyx_skip_dispatch);
  PyObject *(*get_errors_description)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, int __pyx_skip_dispatch);
  void (*_run_test_suite)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, PyObject *, PyObject *);
  void (*_adapt_tentacles_config)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, PyObject *);
  void (*_find_optimal_configuration_using_results)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *);
  PyObject *(*_get_all_TA)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *);
};
static struct __pyx_vtabstruct_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_vtabptr_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectFormatSimple.proto */
#if CYTHON_COMPILING_IN_PYPY
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#elif PY_MAJOR_VERSION < 3
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyString_CheckExact(s)) ? PyUnicode_FromEncodedObject(s, NULL, "strict") :\
        PyObject_Format(s, f))
#elif CYTHON_USE_TYPE_SLOTS
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyLong_CheckExact(s)) ? PyLong_Type.tp_str(s) :\
        likely(PyFloat_CheckExact(s)) ? PyFloat_Type.tp_str(s) :\
        PyObject_Format(s, f))
#else
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#endif

/* IncludeStringH.proto */
#include <string.h>

/* JoinPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_SubtractObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceSubtract(op1, op2) : PyNumber_Subtract(op1, op2))
#endif

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* BuildPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, char* chars, int clength,
                                                int prepend_sign, char padding_char);

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto */
#define __Pyx_unpack_tuple2(tuple, value1, value2, is_tuple, has_known_size, decref_tuple)\
    (likely(is_tuple || PyTuple_Check(tuple)) ?\
        (likely(has_known_size || PyTuple_GET_SIZE(tuple) == 2) ?\
            __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple) :\
            (__Pyx_UnpackTupleError(tuple, 2), -1)) :\
        __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple))
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;  // used by FusedFunction for copying defaults
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_Check(obj)  (__Pyx_TypeCheck(obj, __pyx_CyFunctionType))
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* PyObjectFormat.proto */
#if CYTHON_USE_UNICODE_WRITER
static PyObject* __Pyx_PyObject_Format(PyObject* s, PyObject* f);
#else
#define __Pyx_PyObject_Format(s, f) PyObject_Format(s, f)
#endif

/* PyUnicode_Substring.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Substring(
            PyObject* text, Py_ssize_t start, Py_ssize_t stop);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* py_dict_pop.proto */
static CYTHON_INLINE PyObject *__Pyx_PyDict_Pop(PyObject *d, PyObject *key, PyObject *default_value);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod2.proto */
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* CallUnboundCMethod1.proto */
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* append.proto */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchInspect.proto */
static PyObject* __Pyx_patch_inspect(PyObject* module);

/* PatchAsyncIO.proto */
static PyObject* __Pyx_patch_asyncio(PyObject* module);

/* ClassMethod.proto */
#include "descrobject.h"
static CYTHON_UNUSED PyObject* __Pyx_Method_ClassMethod(PyObject *method);

/* GetNameInClass.proto */
#define __Pyx_GetNameInClass(var, nmspace, name)  (var) = __Pyx__GetNameInClass(nmspace, name)
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static void __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_find_optimal_configuration(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_find_optimal_configuration *__pyx_optional_args); /* proto*/
static void __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer__run_test_suite(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_config, PyObject *__pyx_v_evaluators); /* proto*/
static void __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer__adapt_tentacles_config(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_activated_evaluators); /* proto*/
static void __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer__find_optimal_configuration_using_results(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto*/
static void __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_print_report(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_overall_progress(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_is_in_progress(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_current_test_suite_progress(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_report(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_errors_description(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer__get_all_TA(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto*/

/* Module declarations from 'octobot.strategy_optimizer.strategy_optimizer' */
static PyTypeObject *__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer = 0;
static PyObject *__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer___pyx_unpickle_StrategyOptimizer__set_state(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "octobot.strategy_optimizer.strategy_optimizer"
extern int __pyx_module_is_main_octobot__strategy_optimizer__strategy_optimizer;
int __pyx_module_is_main_octobot__strategy_optimizer__strategy_optimizer = 0;

/* Implementation of 'octobot.strategy_optimizer.strategy_optimizer' */
static PyObject *__pyx_builtin_staticmethod;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_print;
static PyObject *__pyx_builtin_RuntimeError;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_sorted;
static PyObject *__pyx_builtin_round;
static const char __pyx_k_[] = "/";
static const char __pyx_k_f[] = "f";
static const char __pyx_k_i[] = "i";
static const char __pyx_k_TA[] = "TA";
static const char __pyx_k__4[] = " *** ";
static const char __pyx_k__5[] = ": ";
static const char __pyx_k__6[] = "";
static const char __pyx_k__7[] = ", ";
static const char __pyx_k__8[] = " ...";
static const char __pyx_k_TAs[] = "TAs";
static const char __pyx_k_cls[] = "cls";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_pop[] = "pop";
static const char __pyx_k_pow[] = "pow";
static const char __pyx_k_run[] = "run";
static const char __pyx_k_Mode[] = "Mode";
static const char __pyx_k_RANK[] = "RANK";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_elem[] = "elem";
static const char __pyx_k_info[] = "info";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_math[] = "math";
static const char __pyx_k_mean[] = "mean";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_risk[] = ", risk: ";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_ERROR[] = "ERROR";
static const char __pyx_k_debug[] = "debug";
static const char __pyx_k_error[] = "error";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_print[] = "print";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_risks[] = "risks";
static const char __pyx_k_round[] = "round";
static const char __pyx_k_state[] = "state";
static const char __pyx_k_union[] = "union";
static const char __pyx_k_using[] = " using ";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_CONFIG[] = "CONFIG";
static const char __pyx_k_Result[] = " => Result: ";
static const char __pyx_k_TRADES[] = "TRADES";
static const char __pyx_k_append[] = "append";
static const char __pyx_k_config[] = "config";
static const char __pyx_k_dict_2[] = "_dict";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_result[] = "result";
static const char __pyx_k_risk_2[] = "risk";
static const char __pyx_k_risk_s[] = " risk(s).";
static const char __pyx_k_sorted[] = "sorted";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_values[] = "values";
static const char __pyx_k_asyncio[] = "asyncio";
static const char __pyx_k_inspect[] = "inspect";
static const char __pyx_k_logging[] = "logging";
static const char __pyx_k_results[] = "results";
static const char __pyx_k_reverse[] = "reverse";
static const char __pyx_k_deepcopy[] = "deepcopy";
static const char __pyx_k_get_name[] = "get_name";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_enumerate[] = "enumerate";
static const char __pyx_k_eval_conf[] = "eval_conf";
static const char __pyx_k_evaluator[] = "evaluator";
static const char __pyx_k_processed[] = " processed";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_Strategies[] = "Strategies";
static const char __pyx_k_evaluators[] = "evaluators";
static const char __pyx_k_exceptions[] = "exceptions";
static const char __pyx_k_get_logger[] = "get_logger";
static const char __pyx_k_get_report[] = "get_report";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_time_frame[] = "time_frame";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_TAEvaluator[] = "TAEvaluator";
static const char __pyx_k_dict_shaped[] = "dict_shaped";
static const char __pyx_k_sorting_key[] = "_sorting_key";
static const char __pyx_k_time_frames[] = ", time frames :";
static const char __pyx_k_to_use_conf[] = "to_use_conf";
static const char __pyx_k_RuntimeError[] = "RuntimeError";
static const char __pyx_k_all_elements[] = "all_elements";
static const char __pyx_k_current_elem[] = "current_elem";
static const char __pyx_k_print_report[] = "print_report";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_staticmethod[] = "staticmethod";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_trading_mode[] = " trading mode, ";
static const char __pyx_k_use_setstate[] = "use_setstate";
static const char __pyx_k_asyncio_tasks[] = "asyncio.tasks";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_strategy_name[] = "strategy_name";
static const char __pyx_k_time_frames_2[] = "time_frames";
static const char __pyx_k_trades_counts[] = "trades_counts";
static const char __pyx_k_CONFIG_TRADING[] = "CONFIG_TRADING";
static const char __pyx_k_is_in_progress[] = "is_in_progress";
static const char __pyx_k_min_time_frame[] = "min_time_frame";
static const char __pyx_k_run_test_suite[] = "run_test_suite";
static const char __pyx_k_strategy_using[] = " strategy using ";
static const char __pyx_k_trading_mode_2[] = " trading mode *** ";
static const char __pyx_k_TestSuiteResult[] = "TestSuiteResult";
static const char __pyx_k_default_element[] = "default_element";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_time_frames_and[] = " time frames and ";
static const char __pyx_k_TRADES_IN_RESULT[] = "TRADES_IN_RESULT";
static const char __pyx_k_current_progress[] = "current_progress";
static const char __pyx_k_StrategyEvaluator[] = "StrategyEvaluator";
static const char __pyx_k_StrategyOptimizer[] = "StrategyOptimizer";
static const char __pyx_k_StrategyTestSuite[] = "StrategyTestSuite";
static const char __pyx_k_elem_conf_history[] = "elem_conf_history";
static const char __pyx_k_get_average_score[] = "get_average_score";
static const char __pyx_k_get_result_string[] = "get_result_string";
static const char __pyx_k_octobot_constants[] = "octobot.constants";
static const char __pyx_k_tentacles_Trading[] = "tentacles.Trading";
static const char __pyx_k_CONFIG_TRADER_RISK[] = "CONFIG_TRADER_RISK";
static const char __pyx_k_asyncio_coroutines[] = "asyncio.coroutines";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_get_config_summary[] = "get_config_summary";
static const char __pyx_k_get_sorted_results[] = "_get_sorted_results";
static const char __pyx_k_AbstractTradingMode[] = "AbstractTradingMode";
static const char __pyx_k_Run_with_evaluators[] = " Run with: evaluators: ";
static const char __pyx_k_tentacles_Evaluator[] = "tentacles.Evaluator";
static const char __pyx_k_time_frame_rank_sum[] = " (time frame rank sum: ";
static const char __pyx_k_average_trades_count[] = ") average trades count: ";
static const char __pyx_k_finished_computation[] = " finished computation.";
static const char __pyx_k_get_filtered_results[] = "_get_filtered_results";
static const char __pyx_k_get_overall_progress[] = "get_overall_progress";
static const char __pyx_k_get_activated_element[] = "_get_activated_element";
static const char __pyx_k_get_class_from_string[] = "get_class_from_string";
static const char __pyx_k_get_test_suite_result[] = "get_test_suite_result";
static const char __pyx_k_octobot_trading_modes[] = "octobot_trading.modes";
static const char __pyx_k_technical_evaluator_s[] = " technical evaluator(s), ";
static const char __pyx_k_Logging_level_restored[] = "Logging level restored.";
static const char __pyx_k_average_trades_count_2[] = " average trades count: ";
static const char __pyx_k_current_forced_element[] = "current_forced_element";
static const char __pyx_k_get_errors_description[] = "get_errors_description";
static const char __pyx_k_tentacles_setup_config[] = "tentacles_setup_config";
static const char __pyx_k_get_global_logger_level[] = "get_global_logger_level";
static const char __pyx_k_nb_elements_to_consider[] = "nb_elements_to_consider";
static const char __pyx_k_set_global_logger_level[] = "set_global_logger_level";
static const char __pyx_k_CONFIG_FORCED_TIME_FRAME[] = "CONFIG_FORCED_TIME_FRAME";
static const char __pyx_k_convert_result_into_dict[] = "convert_result_into_dict";
static const char __pyx_k_create_evaluator_classes[] = "create_evaluator_classes";
static const char __pyx_k_get_required_time_frames[] = "get_required_time_frames";
static const char __pyx_k_get_tentacles_activation[] = "get_tentacles_activation";
static const char __pyx_k_init_trading_mode_config[] = "init_trading_mode_config";
static const char __pyx_k_initialize_with_strategy[] = "initialize_with_strategy";
static const char __pyx_k_additional_elements_count[] = "additional_elements_count";
static const char __pyx_k_octobot_commons_data_util[] = "octobot_commons.data_util";
static const char __pyx_k_octobot_trading_api_modes[] = "octobot_trading.api.modes";
static const char __pyx_k_octobot_trading_constants[] = "octobot_trading.constants";
static const char __pyx_k_StrategyOptimizer_get_name[] = "StrategyOptimizer.get_name";
static const char __pyx_k_find_optimal_configuration[] = "find_optimal_configuration";
static const char __pyx_k_get_activated_trading_mode[] = "get_activated_trading_mode";
static const char __pyx_k_minimum_time_frame_ranking[] = " minimum time frame ranking*** ";
static const char __pyx_k_Top_rankings_per_time_frame[] = " *** Top rankings per time frame *** ";
static const char __pyx_k_evaluator_parent_inspection[] = "evaluator_parent_inspection";
static const char __pyx_k_StrategyOptimizer_get_report[] = "StrategyOptimizer.get_report";
static const char __pyx_k_octobot_evaluators_constants[] = "octobot_evaluators.constants";
static const char __pyx_k_is_relevant_evaluation_config[] = "_is_relevant_evaluation_config";
static const char __pyx_k_Overall_best_configuration_for[] = " *** Overall best configuration for ";
static const char __pyx_k_StrategyOptimizer_print_report[] = "StrategyOptimizer.print_report";
static const char __pyx_k_in_installed_strategies_Please[] = " in installed strategies. Please make sure to enter the name of the class, ex: FullMixedStrategiesEvaluator";
static const char __pyx_k_is_already_computing_processed[] = " is already computing: processed ";
static const char __pyx_k_pyx_unpickle_StrategyOptimizer[] = "__pyx_unpickle_StrategyOptimizer";
static const char __pyx_k_trading_mode_parent_inspection[] = "trading_mode_parent_inspection";
static const char __pyx_k_StrategyOptimizer__find_optimal[] = "StrategyOptimizer._find_optimal_configuration_using_results.<locals>._sorting_key";
static const char __pyx_k_StrategyOptimizer__get_filtered[] = "StrategyOptimizer._get_filtered_results";
static const char __pyx_k_Top_rankings_through_all_time_f[] = " *** Top rankings through all time frames *** ";
static const char __pyx_k_get_current_test_suite_progress[] = "get_current_test_suite_progress";
static const char __pyx_k_get_sorted_results_locals_lambd[] = "_get_sorted_results.<locals>.<lambda>";
static const char __pyx_k_octobot_commons_logging_logging[] = "octobot_commons.logging.logging_util";
static const char __pyx_k_octobot_evaluators_evaluator_TA[] = "octobot_evaluators.evaluator.TA_evaluator";
static const char __pyx_k_octobot_strategy_optimizer_test[] = "octobot.strategy_optimizer.test_suite_result";
static const char __pyx_k_update_activation_configuration[] = "update_activation_configuration";
static const char __pyx_k_Full_execution_sorted_results_Mi[] = "Full execution sorted results: Minimum time frames are defining the range of the run since it finishes at the end of the first data, aka minimum time frame. Therefore all different time frames are different price actions and can't be compared independently.";
static const char __pyx_k_Impossible_to_find_a_strategy_ma[] = "Impossible to find a strategy matching class name: ";
static const char __pyx_k_Incompatible_checksums_s_vs_0x2d[] = "Incompatible checksums (%s vs 0x2d2eb06 = (all_TAs, all_time_frames, config, current_test_suite, errors, is_computing, is_properly_initialized, logger, risks, run_id, run_results, sorted_results_by_time_frame, sorted_results_through_all_time_frame, strategy_class, tentacles_setup_config, total_nb_runs, trading_mode))";
static const char __pyx_k_OPTIMIZER_FORCE_ASYNCIO_DEBUG_OP[] = "OPTIMIZER_FORCE_ASYNCIO_DEBUG_OPTION";
static const char __pyx_k_Setting_logging_level_to_logging[] = "Setting logging level to logging.ERROR to limit messages.";
static const char __pyx_k_StrategyOptimizer___reduce_cytho[] = "StrategyOptimizer.__reduce_cython__";
static const char __pyx_k_StrategyOptimizer___setstate_cyt[] = "StrategyOptimizer.__setstate_cython__";
static const char __pyx_k_StrategyOptimizer__get_activated[] = "StrategyOptimizer._get_activated_element";
static const char __pyx_k_StrategyOptimizer__get_sorted_re[] = "StrategyOptimizer._get_sorted_results";
static const char __pyx_k_StrategyOptimizer__is_relevant_e[] = "StrategyOptimizer._is_relevant_evaluation_config";
static const char __pyx_k_StrategyOptimizer_find_optimal_c[] = "StrategyOptimizer.find_optimal_configuration";
static const char __pyx_k_StrategyOptimizer_get_current_te[] = "StrategyOptimizer.get_current_test_suite_progress";
static const char __pyx_k_StrategyOptimizer_get_errors_des[] = "StrategyOptimizer.get_errors_description";
static const char __pyx_k_StrategyOptimizer_get_overall_pr[] = "StrategyOptimizer.get_overall_progress";
static const char __pyx_k_StrategyOptimizer_is_in_progress[] = "StrategyOptimizer.is_in_progress";
static const char __pyx_k_Trying_to_find_an_optimized_conf[] = "Trying to find an optimized configuration for ";
static const char __pyx_k_octobot_commons_tentacles_manage[] = "octobot_commons.tentacles_management.class_inspector";
static const char __pyx_k_octobot_evaluators_api_evaluator[] = "octobot_evaluators.api.evaluators";
static const char __pyx_k_octobot_evaluators_evaluator_str[] = "octobot_evaluators.evaluator.strategy_evaluator";
static const char __pyx_k_octobot_strategy_optimizer_strat[] = "octobot/strategy_optimizer/strategy_optimizer.py";
static const char __pyx_k_octobot_tentacles_manager_api_co[] = "octobot_tentacles_manager.api.configurator";
static const char __pyx_k_octobot_strategy_optimizer_strat_2[] = "octobot.strategy_optimizer.strategy_optimizer";
static const char __pyx_k_octobot_strategy_optimizer_strat_3[] = "octobot.strategy_optimizer.strategy_test_suite";
static PyObject *__pyx_kp_u_;
static PyObject *__pyx_n_s_AbstractTradingMode;
static PyObject *__pyx_n_s_CONFIG;
static PyObject *__pyx_n_s_CONFIG_FORCED_TIME_FRAME;
static PyObject *__pyx_n_s_CONFIG_TRADER_RISK;
static PyObject *__pyx_n_s_CONFIG_TRADING;
static PyObject *__pyx_n_s_ERROR;
static PyObject *__pyx_kp_u_Full_execution_sorted_results_Mi;
static PyObject *__pyx_kp_u_Impossible_to_find_a_strategy_ma;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x2d;
static PyObject *__pyx_kp_u_Logging_level_restored;
static PyObject *__pyx_n_s_Mode;
static PyObject *__pyx_n_s_OPTIMIZER_FORCE_ASYNCIO_DEBUG_OP;
static PyObject *__pyx_kp_u_Overall_best_configuration_for;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_RANK;
static PyObject *__pyx_kp_u_Result;
static PyObject *__pyx_kp_u_Run_with_evaluators;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_kp_u_Setting_logging_level_to_logging;
static PyObject *__pyx_n_s_Strategies;
static PyObject *__pyx_n_s_StrategyEvaluator;
static PyObject *__pyx_n_s_StrategyOptimizer;
static PyObject *__pyx_n_s_StrategyOptimizer___reduce_cytho;
static PyObject *__pyx_n_s_StrategyOptimizer___setstate_cyt;
static PyObject *__pyx_n_s_StrategyOptimizer__find_optimal;
static PyObject *__pyx_n_s_StrategyOptimizer__get_activated;
static PyObject *__pyx_n_s_StrategyOptimizer__get_filtered;
static PyObject *__pyx_n_s_StrategyOptimizer__get_sorted_re;
static PyObject *__pyx_n_s_StrategyOptimizer__is_relevant_e;
static PyObject *__pyx_n_s_StrategyOptimizer_find_optimal_c;
static PyObject *__pyx_n_s_StrategyOptimizer_get_current_te;
static PyObject *__pyx_n_s_StrategyOptimizer_get_errors_des;
static PyObject *__pyx_n_s_StrategyOptimizer_get_name;
static PyObject *__pyx_n_s_StrategyOptimizer_get_overall_pr;
static PyObject *__pyx_n_s_StrategyOptimizer_get_report;
static PyObject *__pyx_n_s_StrategyOptimizer_is_in_progress;
static PyObject *__pyx_n_s_StrategyOptimizer_print_report;
static PyObject *__pyx_n_s_StrategyTestSuite;
static PyObject *__pyx_n_s_TA;
static PyObject *__pyx_n_s_TAEvaluator;
static PyObject *__pyx_n_s_TAs;
static PyObject *__pyx_n_s_TRADES;
static PyObject *__pyx_n_s_TRADES_IN_RESULT;
static PyObject *__pyx_n_s_TestSuiteResult;
static PyObject *__pyx_kp_u_Top_rankings_per_time_frame;
static PyObject *__pyx_kp_u_Top_rankings_through_all_time_f;
static PyObject *__pyx_kp_u_Trying_to_find_an_optimized_conf;
static PyObject *__pyx_kp_u__4;
static PyObject *__pyx_kp_u__5;
static PyObject *__pyx_kp_u__6;
static PyObject *__pyx_kp_u__7;
static PyObject *__pyx_kp_u__8;
static PyObject *__pyx_n_s_additional_elements_count;
static PyObject *__pyx_n_s_all_elements;
static PyObject *__pyx_n_s_append;
static PyObject *__pyx_n_s_asyncio;
static PyObject *__pyx_n_s_asyncio_coroutines;
static PyObject *__pyx_n_s_asyncio_tasks;
static PyObject *__pyx_kp_u_average_trades_count;
static PyObject *__pyx_kp_u_average_trades_count_2;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_cls;
static PyObject *__pyx_n_s_config;
static PyObject *__pyx_n_s_convert_result_into_dict;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_n_s_create_evaluator_classes;
static PyObject *__pyx_n_s_current_elem;
static PyObject *__pyx_n_s_current_forced_element;
static PyObject *__pyx_n_s_current_progress;
static PyObject *__pyx_n_s_debug;
static PyObject *__pyx_n_s_deepcopy;
static PyObject *__pyx_n_s_default_element;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_dict_2;
static PyObject *__pyx_n_s_dict_shaped;
static PyObject *__pyx_n_s_elem;
static PyObject *__pyx_n_s_elem_conf_history;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_error;
static PyObject *__pyx_n_s_eval_conf;
static PyObject *__pyx_n_s_evaluator;
static PyObject *__pyx_n_s_evaluator_parent_inspection;
static PyObject *__pyx_n_s_evaluators;
static PyObject *__pyx_n_s_exceptions;
static PyObject *__pyx_n_u_f;
static PyObject *__pyx_n_s_find_optimal_configuration;
static PyObject *__pyx_kp_u_finished_computation;
static PyObject *__pyx_n_s_get_activated_element;
static PyObject *__pyx_n_s_get_activated_trading_mode;
static PyObject *__pyx_n_s_get_average_score;
static PyObject *__pyx_n_s_get_class_from_string;
static PyObject *__pyx_n_s_get_config_summary;
static PyObject *__pyx_n_s_get_current_test_suite_progress;
static PyObject *__pyx_n_s_get_errors_description;
static PyObject *__pyx_n_s_get_filtered_results;
static PyObject *__pyx_n_s_get_global_logger_level;
static PyObject *__pyx_n_s_get_logger;
static PyObject *__pyx_n_s_get_name;
static PyObject *__pyx_n_s_get_overall_progress;
static PyObject *__pyx_n_s_get_report;
static PyObject *__pyx_n_s_get_required_time_frames;
static PyObject *__pyx_n_s_get_result_string;
static PyObject *__pyx_n_s_get_sorted_results;
static PyObject *__pyx_n_s_get_sorted_results_locals_lambd;
static PyObject *__pyx_n_s_get_tentacles_activation;
static PyObject *__pyx_n_s_get_test_suite_result;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_kp_u_in_installed_strategies_Please;
static PyObject *__pyx_n_s_info;
static PyObject *__pyx_n_s_init_trading_mode_config;
static PyObject *__pyx_n_s_initialize_with_strategy;
static PyObject *__pyx_n_s_inspect;
static PyObject *__pyx_kp_u_is_already_computing_processed;
static PyObject *__pyx_n_s_is_in_progress;
static PyObject *__pyx_n_s_is_relevant_evaluation_config;
static PyObject *__pyx_n_s_items;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_logging;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_math;
static PyObject *__pyx_n_s_mean;
static PyObject *__pyx_n_s_min_time_frame;
static PyObject *__pyx_kp_u_minimum_time_frame_ranking;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_nb_elements_to_consider;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_octobot_commons_data_util;
static PyObject *__pyx_n_s_octobot_commons_logging_logging;
static PyObject *__pyx_n_s_octobot_commons_tentacles_manage;
static PyObject *__pyx_n_s_octobot_constants;
static PyObject *__pyx_n_s_octobot_evaluators_api_evaluator;
static PyObject *__pyx_n_s_octobot_evaluators_constants;
static PyObject *__pyx_n_s_octobot_evaluators_evaluator_TA;
static PyObject *__pyx_n_s_octobot_evaluators_evaluator_str;
static PyObject *__pyx_kp_s_octobot_strategy_optimizer_strat;
static PyObject *__pyx_n_s_octobot_strategy_optimizer_strat_2;
static PyObject *__pyx_n_s_octobot_strategy_optimizer_strat_3;
static PyObject *__pyx_n_s_octobot_strategy_optimizer_test;
static PyObject *__pyx_n_s_octobot_tentacles_manager_api_co;
static PyObject *__pyx_n_s_octobot_trading_api_modes;
static PyObject *__pyx_n_s_octobot_trading_constants;
static PyObject *__pyx_n_s_octobot_trading_modes;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_pop;
static PyObject *__pyx_n_s_pow;
static PyObject *__pyx_n_s_print;
static PyObject *__pyx_n_s_print_report;
static PyObject *__pyx_kp_u_processed;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_StrategyOptimizer;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_result;
static PyObject *__pyx_n_s_results;
static PyObject *__pyx_n_s_reverse;
static PyObject *__pyx_kp_u_risk;
static PyObject *__pyx_n_s_risk_2;
static PyObject *__pyx_kp_u_risk_s;
static PyObject *__pyx_n_s_risks;
static PyObject *__pyx_n_s_round;
static PyObject *__pyx_n_s_run;
static PyObject *__pyx_n_s_run_test_suite;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_n_s_set_global_logger_level;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_sorted;
static PyObject *__pyx_n_s_sorting_key;
static PyObject *__pyx_n_s_state;
static PyObject *__pyx_n_s_staticmethod;
static PyObject *__pyx_n_s_strategy_name;
static PyObject *__pyx_kp_u_strategy_using;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_kp_u_technical_evaluator_s;
static PyObject *__pyx_n_s_tentacles_Evaluator;
static PyObject *__pyx_n_s_tentacles_Trading;
static PyObject *__pyx_n_s_tentacles_setup_config;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_time_frame;
static PyObject *__pyx_kp_u_time_frame_rank_sum;
static PyObject *__pyx_kp_u_time_frames;
static PyObject *__pyx_n_s_time_frames_2;
static PyObject *__pyx_kp_u_time_frames_and;
static PyObject *__pyx_n_s_to_use_conf;
static PyObject *__pyx_n_s_trades_counts;
static PyObject *__pyx_kp_u_trading_mode;
static PyObject *__pyx_kp_u_trading_mode_2;
static PyObject *__pyx_n_s_trading_mode_parent_inspection;
static PyObject *__pyx_n_s_union;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_update_activation_configuration;
static PyObject *__pyx_n_s_use_setstate;
static PyObject *__pyx_kp_u_using;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_values;
static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer___init__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_config, PyObject *__pyx_v_tentacles_setup_config, PyObject *__pyx_v_strategy_name); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_2find_optimal_configuration(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_TAs, PyObject *__pyx_v_time_frames, PyObject *__pyx_v_risks); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_41_find_optimal_configuration_using_results__sorting_key(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_elem); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_4print_report(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_6get_overall_progress(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_8is_in_progress(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_10get_current_test_suite_progress(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12get_report(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14get_errors_description(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_16get_name(PyTypeObject *__pyx_v_cls); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_18_get_activated_element(PyObject *__pyx_v_all_elements, PyObject *__pyx_v_current_forced_element, PyObject *__pyx_v_nb_elements_to_consider, PyObject *__pyx_v_elem_conf_history, PyObject *__pyx_v_default_element, PyObject *__pyx_v_dict_shaped); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_20_get_filtered_results(PyObject *__pyx_v_results, PyObject *__pyx_v_time_frame); /* proto */
static PyObject *__pyx_lambda_funcdef_lambda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_result); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_22_get_sorted_results(PyObject *__pyx_v_results, PyObject *__pyx_v_time_frame); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_24_is_relevant_evaluation_config(PyObject *__pyx_v_evaluator); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23is_properly_initialized___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23is_properly_initialized_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode_4__del__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class_4__del__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames_4__del__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs_4__del__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks_4__del__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12is_computing___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12is_computing_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results_4__del__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_26__reduce_cython__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_28__setstate_cython__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer___pyx_unpickle_StrategyOptimizer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop = {0, &__pyx_n_s_pop, 0, 0, 0};
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_5;
static PyObject *__pyx_int_47377158;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_codeobj__3;
static PyObject *__pyx_codeobj__10;
static PyObject *__pyx_codeobj__12;
static PyObject *__pyx_codeobj__14;
static PyObject *__pyx_codeobj__16;
static PyObject *__pyx_codeobj__18;
static PyObject *__pyx_codeobj__20;
static PyObject *__pyx_codeobj__22;
static PyObject *__pyx_codeobj__24;
static PyObject *__pyx_codeobj__26;
static PyObject *__pyx_codeobj__28;
static PyObject *__pyx_codeobj__30;
static PyObject *__pyx_codeobj__32;
static PyObject *__pyx_codeobj__34;
static PyObject *__pyx_codeobj__36;
static PyObject *__pyx_codeobj__38;
/* Late includes */

/* "octobot/strategy_optimizer/strategy_optimizer.py":51
 *     StrategyOptimizer is a tool that performs backtesting with different configurations
 *     """
 *     def __init__(self, config, tentacles_setup_config, strategy_name):             # <<<<<<<<<<<<<<
 *         self.is_properly_initialized = False
 *         self.logger = get_logger(self.get_name())
 */

/* Python wrapper */
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_config = 0;
  PyObject *__pyx_v_tentacles_setup_config = 0;
  PyObject *__pyx_v_strategy_name = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_config,&__pyx_n_s_tentacles_setup_config,&__pyx_n_s_strategy_name,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_config)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tentacles_setup_config)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 1); __PYX_ERR(0, 51, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_strategy_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 2); __PYX_ERR(0, 51, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 51, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_config = values[0];
    __pyx_v_tentacles_setup_config = values[1];
    __pyx_v_strategy_name = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 51, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer___init__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self), __pyx_v_config, __pyx_v_tentacles_setup_config, __pyx_v_strategy_name);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer___init__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_config, PyObject *__pyx_v_tentacles_setup_config, PyObject *__pyx_v_strategy_name) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_UCS4 __pyx_t_12;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":52
 *     """
 *     def __init__(self, config, tentacles_setup_config, strategy_name):
 *         self.is_properly_initialized = False             # <<<<<<<<<<<<<<
 *         self.logger = get_logger(self.get_name())
 *         self.config = config
 */
  __pyx_v_self->is_properly_initialized = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":53
 *     def __init__(self, config, tentacles_setup_config, strategy_name):
 *         self.is_properly_initialized = False
 *         self.logger = get_logger(self.get_name())             # <<<<<<<<<<<<<<
 *         self.config = config
 *         self.tentacles_setup_config = deepcopy(tentacles_setup_config)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_get_logger); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_name); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->logger);
  __Pyx_DECREF(__pyx_v_self->logger);
  __pyx_v_self->logger = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":54
 *         self.is_properly_initialized = False
 *         self.logger = get_logger(self.get_name())
 *         self.config = config             # <<<<<<<<<<<<<<
 *         self.tentacles_setup_config = deepcopy(tentacles_setup_config)
 *         init_trading_mode_config(self.config)
 */
  if (!(likely(PyDict_CheckExact(__pyx_v_config))||((__pyx_v_config) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_v_config)->tp_name), 0))) __PYX_ERR(0, 54, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_config;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->config);
  __Pyx_DECREF(__pyx_v_self->config);
  __pyx_v_self->config = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":55
 *         self.logger = get_logger(self.get_name())
 *         self.config = config
 *         self.tentacles_setup_config = deepcopy(tentacles_setup_config)             # <<<<<<<<<<<<<<
 *         init_trading_mode_config(self.config)
 *         create_evaluator_classes(self.config)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_deepcopy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_tentacles_setup_config) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_tentacles_setup_config);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->tentacles_setup_config);
  __Pyx_DECREF(__pyx_v_self->tentacles_setup_config);
  __pyx_v_self->tentacles_setup_config = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":56
 *         self.config = config
 *         self.tentacles_setup_config = deepcopy(tentacles_setup_config)
 *         init_trading_mode_config(self.config)             # <<<<<<<<<<<<<<
 *         create_evaluator_classes(self.config)
 *         self.trading_mode = get_activated_trading_mode(self.config, tentacles_setup_config)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_init_trading_mode_config); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_self->config) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_self->config);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":57
 *         self.tentacles_setup_config = deepcopy(tentacles_setup_config)
 *         init_trading_mode_config(self.config)
 *         create_evaluator_classes(self.config)             # <<<<<<<<<<<<<<
 *         self.trading_mode = get_activated_trading_mode(self.config, tentacles_setup_config)
 *         self.strategy_class = get_class_from_string(strategy_name, StrategyEvaluator,
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_create_evaluator_classes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_self->config) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_self->config);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":58
 *         init_trading_mode_config(self.config)
 *         create_evaluator_classes(self.config)
 *         self.trading_mode = get_activated_trading_mode(self.config, tentacles_setup_config)             # <<<<<<<<<<<<<<
 *         self.strategy_class = get_class_from_string(strategy_name, StrategyEvaluator,
 *                                                     Strategies, evaluator_parent_inspection)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_get_activated_trading_mode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_self->config, __pyx_v_tentacles_setup_config};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 58, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_self->config, __pyx_v_tentacles_setup_config};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 58, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 58, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_self->config);
    __Pyx_GIVEREF(__pyx_v_self->config);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_6, __pyx_v_self->config);
    __Pyx_INCREF(__pyx_v_tentacles_setup_config);
    __Pyx_GIVEREF(__pyx_v_tentacles_setup_config);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_6, __pyx_v_tentacles_setup_config);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 58, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->trading_mode);
  __Pyx_DECREF(__pyx_v_self->trading_mode);
  __pyx_v_self->trading_mode = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":59
 *         create_evaluator_classes(self.config)
 *         self.trading_mode = get_activated_trading_mode(self.config, tentacles_setup_config)
 *         self.strategy_class = get_class_from_string(strategy_name, StrategyEvaluator,             # <<<<<<<<<<<<<<
 *                                                     Strategies, evaluator_parent_inspection)
 *         self.run_results = []
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_get_class_from_string); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_StrategyEvaluator); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":60
 *         self.trading_mode = get_activated_trading_mode(self.config, tentacles_setup_config)
 *         self.strategy_class = get_class_from_string(strategy_name, StrategyEvaluator,
 *                                                     Strategies, evaluator_parent_inspection)             # <<<<<<<<<<<<<<
 *         self.run_results = []
 *         self.sorted_results_by_time_frame = {}
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_Strategies); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_evaluator_parent_inspection); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_7 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_7, __pyx_v_strategy_name, __pyx_t_4, __pyx_t_3, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 4+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 59, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_7, __pyx_v_strategy_name, __pyx_t_4, __pyx_t_3, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 4+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 59, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(4+__pyx_t_6); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 59, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(__pyx_v_strategy_name);
    __Pyx_GIVEREF(__pyx_v_strategy_name);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_6, __pyx_v_strategy_name);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_6, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 3+__pyx_t_6, __pyx_t_5);
    __pyx_t_4 = 0;
    __pyx_t_3 = 0;
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 59, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":59
 *         create_evaluator_classes(self.config)
 *         self.trading_mode = get_activated_trading_mode(self.config, tentacles_setup_config)
 *         self.strategy_class = get_class_from_string(strategy_name, StrategyEvaluator,             # <<<<<<<<<<<<<<
 *                                                     Strategies, evaluator_parent_inspection)
 *         self.run_results = []
 */
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->strategy_class);
  __Pyx_DECREF(__pyx_v_self->strategy_class);
  __pyx_v_self->strategy_class = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":61
 *         self.strategy_class = get_class_from_string(strategy_name, StrategyEvaluator,
 *                                                     Strategies, evaluator_parent_inspection)
 *         self.run_results = []             # <<<<<<<<<<<<<<
 *         self.sorted_results_by_time_frame = {}
 *         self.sorted_results_through_all_time_frame = {}
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->run_results);
  __Pyx_DECREF(__pyx_v_self->run_results);
  __pyx_v_self->run_results = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":62
 *                                                     Strategies, evaluator_parent_inspection)
 *         self.run_results = []
 *         self.sorted_results_by_time_frame = {}             # <<<<<<<<<<<<<<
 *         self.sorted_results_through_all_time_frame = {}
 *         self.all_time_frames = []
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->sorted_results_by_time_frame);
  __Pyx_DECREF(__pyx_v_self->sorted_results_by_time_frame);
  __pyx_v_self->sorted_results_by_time_frame = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":63
 *         self.run_results = []
 *         self.sorted_results_by_time_frame = {}
 *         self.sorted_results_through_all_time_frame = {}             # <<<<<<<<<<<<<<
 *         self.all_time_frames = []
 *         self.all_TAs = []
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->sorted_results_through_all_time_frame);
  __Pyx_DECREF(__pyx_v_self->sorted_results_through_all_time_frame);
  __pyx_v_self->sorted_results_through_all_time_frame = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":64
 *         self.sorted_results_by_time_frame = {}
 *         self.sorted_results_through_all_time_frame = {}
 *         self.all_time_frames = []             # <<<<<<<<<<<<<<
 *         self.all_TAs = []
 *         self.risks = []
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->all_time_frames);
  __Pyx_DECREF(__pyx_v_self->all_time_frames);
  __pyx_v_self->all_time_frames = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":65
 *         self.sorted_results_through_all_time_frame = {}
 *         self.all_time_frames = []
 *         self.all_TAs = []             # <<<<<<<<<<<<<<
 *         self.risks = []
 *         self.current_test_suite = None
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->all_TAs);
  __Pyx_DECREF(__pyx_v_self->all_TAs);
  __pyx_v_self->all_TAs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":66
 *         self.all_time_frames = []
 *         self.all_TAs = []
 *         self.risks = []             # <<<<<<<<<<<<<<
 *         self.current_test_suite = None
 *         self.errors = set()
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->risks);
  __Pyx_DECREF(__pyx_v_self->risks);
  __pyx_v_self->risks = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":67
 *         self.all_TAs = []
 *         self.risks = []
 *         self.current_test_suite = None             # <<<<<<<<<<<<<<
 *         self.errors = set()
 * 
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->current_test_suite);
  __Pyx_DECREF(__pyx_v_self->current_test_suite);
  __pyx_v_self->current_test_suite = Py_None;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":68
 *         self.risks = []
 *         self.current_test_suite = None
 *         self.errors = set()             # <<<<<<<<<<<<<<
 * 
 *         self.is_computing = False
 */
  __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->errors);
  __Pyx_DECREF(__pyx_v_self->errors);
  __pyx_v_self->errors = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":70
 *         self.errors = set()
 * 
 *         self.is_computing = False             # <<<<<<<<<<<<<<
 *         self.run_id = 0
 *         self.total_nb_runs = 0
 */
  __pyx_v_self->is_computing = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":71
 * 
 *         self.is_computing = False
 *         self.run_id = 0             # <<<<<<<<<<<<<<
 *         self.total_nb_runs = 0
 * 
 */
  __pyx_v_self->run_id = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":72
 *         self.is_computing = False
 *         self.run_id = 0
 *         self.total_nb_runs = 0             # <<<<<<<<<<<<<<
 * 
 *         if not self.strategy_class:
 */
  __pyx_v_self->total_nb_runs = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":74
 *         self.total_nb_runs = 0
 * 
 *         if not self.strategy_class:             # <<<<<<<<<<<<<<
 *             self.logger.error(f"Impossible to find a strategy matching class name: {strategy_name} in installed "
 *                               f"strategies. Please make sure to enter the name of the class, "
 */
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_v_self->strategy_class); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 74, __pyx_L1_error)
  __pyx_t_10 = ((!__pyx_t_9) != 0);
  if (__pyx_t_10) {

    /* "octobot/strategy_optimizer/strategy_optimizer.py":75
 * 
 *         if not self.strategy_class:
 *             self.logger.error(f"Impossible to find a strategy matching class name: {strategy_name} in installed "             # <<<<<<<<<<<<<<
 *                               f"strategies. Please make sure to enter the name of the class, "
 *                               f"ex: FullMixedStrategiesEvaluator")
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_11 = 0;
    __pyx_t_12 = 127;
    __Pyx_INCREF(__pyx_kp_u_Impossible_to_find_a_strategy_ma);
    __pyx_t_11 += 51;
    __Pyx_GIVEREF(__pyx_kp_u_Impossible_to_find_a_strategy_ma);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_kp_u_Impossible_to_find_a_strategy_ma);
    __pyx_t_5 = __Pyx_PyObject_FormatSimple(__pyx_v_strategy_name, __pyx_empty_unicode); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_12 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) > __pyx_t_12) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) : __pyx_t_12;
    __pyx_t_11 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_5);
    __pyx_t_5 = 0;
    __Pyx_INCREF(__pyx_kp_u_in_installed_strategies_Please);
    __pyx_t_11 += 107;
    __Pyx_GIVEREF(__pyx_kp_u_in_installed_strategies_Please);
    PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_kp_u_in_installed_strategies_Please);
    __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_8, 3, __pyx_t_11, __pyx_t_12); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_8, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":74
 *         self.total_nb_runs = 0
 * 
 *         if not self.strategy_class:             # <<<<<<<<<<<<<<
 *             self.logger.error(f"Impossible to find a strategy matching class name: {strategy_name} in installed "
 *                               f"strategies. Please make sure to enter the name of the class, "
 */
    goto __pyx_L3;
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":79
 *                               f"ex: FullMixedStrategiesEvaluator")
 *         else:
 *             self.is_properly_initialized = True             # <<<<<<<<<<<<<<
 * 
 *     def find_optimal_configuration(self, TAs=None, time_frames=None, risks=None):
 */
  /*else*/ {
    __pyx_v_self->is_properly_initialized = 1;
  }
  __pyx_L3:;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":51
 *     StrategyOptimizer is a tool that performs backtesting with different configurations
 *     """
 *     def __init__(self, config, tentacles_setup_config, strategy_name):             # <<<<<<<<<<<<<<
 *         self.is_properly_initialized = False
 *         self.logger = get_logger(self.get_name())
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":81
 *             self.is_properly_initialized = True
 * 
 *     def find_optimal_configuration(self, TAs=None, time_frames=None, risks=None):             # <<<<<<<<<<<<<<
 *         if not self.is_computing:
 * 
 */

static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_3find_optimal_configuration(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static void __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_find_optimal_configuration(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_find_optimal_configuration *__pyx_optional_args) {
  PyObject *__pyx_v_TAs = ((PyObject*)Py_None);
  PyObject *__pyx_v_time_frames = ((PyObject*)Py_None);
  PyObject *__pyx_v_risks = ((PyObject*)Py_None);
  PyObject *__pyx_v_previous_log_level = NULL;
  PyObject *__pyx_v_nb_TAs = NULL;
  PyObject *__pyx_v_nb_TFs = NULL;
  PyObject *__pyx_v_risk = NULL;
  PyObject *__pyx_v_eval_conf_history = NULL;
  PyObject *__pyx_v_evaluator_conf_iteration = NULL;
  PyObject *__pyx_v_current_forced_evaluator = NULL;
  PyObject *__pyx_v_nb_evaluators = NULL;
  CYTHON_UNUSED PyObject *__pyx_v__ = NULL;
  PyObject *__pyx_v_activated_evaluators = NULL;
  PyObject *__pyx_v_time_frames_conf_history = NULL;
  PyObject *__pyx_v_time_frame_conf_iteration = NULL;
  PyObject *__pyx_v_current_forced_time_frame = NULL;
  PyObject *__pyx_v_nb_time_frames = NULL;
  PyObject *__pyx_v_activated_time_frames = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_UCS4 __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  Py_ssize_t __pyx_t_11;
  PyObject *(*__pyx_t_12)(PyObject *);
  Py_ssize_t __pyx_t_13;
  PyObject *(*__pyx_t_14)(PyObject *);
  Py_ssize_t __pyx_t_15;
  PyObject *(*__pyx_t_16)(PyObject *);
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  int __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  PyObject *(*__pyx_t_22)(PyObject *);
  Py_ssize_t __pyx_t_23;
  PyObject *(*__pyx_t_24)(PyObject *);
  Py_ssize_t __pyx_t_25;
  PyObject *(*__pyx_t_26)(PyObject *);
  PyObject *__pyx_t_27 = NULL;
  PyObject *__pyx_t_28 = NULL;
  Py_ssize_t __pyx_t_29;
  int __pyx_t_30;
  char const *__pyx_t_31;
  PyObject *__pyx_t_32 = NULL;
  PyObject *__pyx_t_33 = NULL;
  PyObject *__pyx_t_34 = NULL;
  PyObject *__pyx_t_35 = NULL;
  PyObject *__pyx_t_36 = NULL;
  PyObject *__pyx_t_37 = NULL;
  __Pyx_RefNannySetupContext("find_optimal_configuration", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_TAs = __pyx_optional_args->TAs;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_time_frames = __pyx_optional_args->time_frames;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_risks = __pyx_optional_args->risks;
        }
      }
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_find_optimal_configuration); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_3find_optimal_configuration)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_TAs, __pyx_v_time_frames, __pyx_v_risks};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_TAs, __pyx_v_time_frames, __pyx_v_risks};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 81, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_TAs);
          __Pyx_GIVEREF(__pyx_v_TAs);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_TAs);
          __Pyx_INCREF(__pyx_v_time_frames);
          __Pyx_GIVEREF(__pyx_v_time_frames);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_time_frames);
          __Pyx_INCREF(__pyx_v_risks);
          __Pyx_GIVEREF(__pyx_v_risks);
          PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_risks);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":82
 * 
 *     def find_optimal_configuration(self, TAs=None, time_frames=None, risks=None):
 *         if not self.is_computing:             # <<<<<<<<<<<<<<
 * 
 *             # set is_computing to True to prevent any simultaneous start
 */
  __pyx_t_7 = ((!(__pyx_v_self->is_computing != 0)) != 0);
  if (likely(__pyx_t_7)) {

    /* "octobot/strategy_optimizer/strategy_optimizer.py":85
 * 
 *             # set is_computing to True to prevent any simultaneous start
 *             self.is_computing = True             # <<<<<<<<<<<<<<
 * 
 *             self.errors = set()
 */
    __pyx_v_self->is_computing = 1;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":87
 *             self.is_computing = True
 * 
 *             self.errors = set()             # <<<<<<<<<<<<<<
 *             self.run_results = []
 *             self.sorted_results_by_time_frame = {}
 */
    __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 87, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->errors);
    __Pyx_DECREF(__pyx_v_self->errors);
    __pyx_v_self->errors = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":88
 * 
 *             self.errors = set()
 *             self.run_results = []             # <<<<<<<<<<<<<<
 *             self.sorted_results_by_time_frame = {}
 *             self.sorted_results_through_all_time_frame = {}
 */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 88, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->run_results);
    __Pyx_DECREF(__pyx_v_self->run_results);
    __pyx_v_self->run_results = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":89
 *             self.errors = set()
 *             self.run_results = []
 *             self.sorted_results_by_time_frame = {}             # <<<<<<<<<<<<<<
 *             self.sorted_results_through_all_time_frame = {}
 * 
 */
    __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 89, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->sorted_results_by_time_frame);
    __Pyx_DECREF(__pyx_v_self->sorted_results_by_time_frame);
    __pyx_v_self->sorted_results_by_time_frame = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":90
 *             self.run_results = []
 *             self.sorted_results_by_time_frame = {}
 *             self.sorted_results_through_all_time_frame = {}             # <<<<<<<<<<<<<<
 * 
 *             previous_log_level = get_global_logger_level()
 */
    __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 90, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->sorted_results_through_all_time_frame);
    __Pyx_DECREF(__pyx_v_self->sorted_results_through_all_time_frame);
    __pyx_v_self->sorted_results_through_all_time_frame = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":92
 *             self.sorted_results_through_all_time_frame = {}
 * 
 *             previous_log_level = get_global_logger_level()             # <<<<<<<<<<<<<<
 * 
 *             try:
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_get_global_logger_level); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 92, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 92, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_previous_log_level = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":94
 *             previous_log_level = get_global_logger_level()
 * 
 *             try:             # <<<<<<<<<<<<<<
 *                 self.all_TAs = self._get_all_TA() if TAs is None else TAs
 *                 nb_TAs = len(self.all_TAs)
 */
    /*try:*/ {

      /* "octobot/strategy_optimizer/strategy_optimizer.py":95
 * 
 *             try:
 *                 self.all_TAs = self._get_all_TA() if TAs is None else TAs             # <<<<<<<<<<<<<<
 *                 nb_TAs = len(self.all_TAs)
 * 
 */
      __pyx_t_7 = (__pyx_v_TAs == ((PyObject*)Py_None));
      if ((__pyx_t_7 != 0)) {
        __pyx_t_2 = ((struct __pyx_vtabstruct_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self->__pyx_vtab)->_get_all_TA(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 95, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_1 = __pyx_t_2;
        __pyx_t_2 = 0;
      } else {
        __Pyx_INCREF(__pyx_v_TAs);
        __pyx_t_1 = __pyx_v_TAs;
      }
      __Pyx_GIVEREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_v_self->all_TAs);
      __Pyx_DECREF(__pyx_v_self->all_TAs);
      __pyx_v_self->all_TAs = ((PyObject*)__pyx_t_1);
      __pyx_t_1 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":96
 *             try:
 *                 self.all_TAs = self._get_all_TA() if TAs is None else TAs
 *                 nb_TAs = len(self.all_TAs)             # <<<<<<<<<<<<<<
 * 
 *                 self.all_time_frames = self.strategy_class.get_required_time_frames(self.config) \
 */
      __pyx_t_1 = __pyx_v_self->all_TAs;
      __Pyx_INCREF(__pyx_t_1);
      if (unlikely(__pyx_t_1 == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 96, __pyx_L5_error)
      }
      __pyx_t_8 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 96, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 96, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_nb_TAs = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":99
 * 
 *                 self.all_time_frames = self.strategy_class.get_required_time_frames(self.config) \
 *                     if time_frames is None else time_frames             # <<<<<<<<<<<<<<
 *                 nb_TFs = len(self.all_time_frames)
 * 
 */
      __pyx_t_7 = (__pyx_v_time_frames == ((PyObject*)Py_None));
      if ((__pyx_t_7 != 0)) {

        /* "octobot/strategy_optimizer/strategy_optimizer.py":98
 *                 nb_TAs = len(self.all_TAs)
 * 
 *                 self.all_time_frames = self.strategy_class.get_required_time_frames(self.config) \             # <<<<<<<<<<<<<<
 *                     if time_frames is None else time_frames
 *                 nb_TFs = len(self.all_time_frames)
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->strategy_class, __pyx_n_s_get_required_time_frames); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 98, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_6 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_6, __pyx_v_self->config) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_self->config);
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 98, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 98, __pyx_L5_error)
        __pyx_t_1 = __pyx_t_2;
        __pyx_t_2 = 0;
      } else {

        /* "octobot/strategy_optimizer/strategy_optimizer.py":99
 * 
 *                 self.all_time_frames = self.strategy_class.get_required_time_frames(self.config) \
 *                     if time_frames is None else time_frames             # <<<<<<<<<<<<<<
 *                 nb_TFs = len(self.all_time_frames)
 * 
 */
        __Pyx_INCREF(__pyx_v_time_frames);
        __pyx_t_1 = __pyx_v_time_frames;
      }

      /* "octobot/strategy_optimizer/strategy_optimizer.py":98
 *                 nb_TAs = len(self.all_TAs)
 * 
 *                 self.all_time_frames = self.strategy_class.get_required_time_frames(self.config) \             # <<<<<<<<<<<<<<
 *                     if time_frames is None else time_frames
 *                 nb_TFs = len(self.all_time_frames)
 */
      __Pyx_GIVEREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_v_self->all_time_frames);
      __Pyx_DECREF(__pyx_v_self->all_time_frames);
      __pyx_v_self->all_time_frames = ((PyObject*)__pyx_t_1);
      __pyx_t_1 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":100
 *                 self.all_time_frames = self.strategy_class.get_required_time_frames(self.config) \
 *                     if time_frames is None else time_frames
 *                 nb_TFs = len(self.all_time_frames)             # <<<<<<<<<<<<<<
 * 
 *                 self.risks = [1] if risks is None else risks
 */
      __pyx_t_1 = __pyx_v_self->all_time_frames;
      __Pyx_INCREF(__pyx_t_1);
      if (unlikely(__pyx_t_1 == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 100, __pyx_L5_error)
      }
      __pyx_t_8 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 100, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_nb_TFs = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":102
 *                 nb_TFs = len(self.all_time_frames)
 * 
 *                 self.risks = [1] if risks is None else risks             # <<<<<<<<<<<<<<
 * 
 *                 self.logger.info(f"Trying to find an optimized configuration for {self.strategy_class.get_name()} "
 */
      __pyx_t_7 = (__pyx_v_risks == ((PyObject*)Py_None));
      if ((__pyx_t_7 != 0)) {
        __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 102, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_INCREF(__pyx_int_1);
        __Pyx_GIVEREF(__pyx_int_1);
        PyList_SET_ITEM(__pyx_t_2, 0, __pyx_int_1);
        __pyx_t_1 = __pyx_t_2;
        __pyx_t_2 = 0;
      } else {
        __Pyx_INCREF(__pyx_v_risks);
        __pyx_t_1 = __pyx_v_risks;
      }
      __Pyx_GIVEREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_v_self->risks);
      __Pyx_DECREF(__pyx_v_self->risks);
      __pyx_v_self->risks = ((PyObject*)__pyx_t_1);
      __pyx_t_1 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":104
 *                 self.risks = [1] if risks is None else risks
 * 
 *                 self.logger.info(f"Trying to find an optimized configuration for {self.strategy_class.get_name()} "             # <<<<<<<<<<<<<<
 *                                  f"strategy using {self.trading_mode.get_name()} trading mode, {self.all_TAs} "
 *                                  f"technical evaluator(s), {self.all_time_frames} time frames and {self.risks} "
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_info); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 104, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = PyTuple_New(11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 104, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_8 = 0;
      __pyx_t_9 = 127;
      __Pyx_INCREF(__pyx_kp_u_Trying_to_find_an_optimized_conf);
      __pyx_t_8 += 46;
      __Pyx_GIVEREF(__pyx_kp_u_Trying_to_find_an_optimized_conf);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_kp_u_Trying_to_find_an_optimized_conf);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->strategy_class, __pyx_n_s_get_name); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 104, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_10 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_6 = (__pyx_t_10) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_10) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 104, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_t_6, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 104, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_9 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) > __pyx_t_9) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) : __pyx_t_9;
      __pyx_t_8 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
      __pyx_t_4 = 0;
      __Pyx_INCREF(__pyx_kp_u_strategy_using);
      __pyx_t_8 += 16;
      __Pyx_GIVEREF(__pyx_kp_u_strategy_using);
      PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_kp_u_strategy_using);

      /* "octobot/strategy_optimizer/strategy_optimizer.py":105
 * 
 *                 self.logger.info(f"Trying to find an optimized configuration for {self.strategy_class.get_name()} "
 *                                  f"strategy using {self.trading_mode.get_name()} trading mode, {self.all_TAs} "             # <<<<<<<<<<<<<<
 *                                  f"technical evaluator(s), {self.all_time_frames} time frames and {self.risks} "
 *                                  f"risk(s).")
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->trading_mode, __pyx_n_s_get_name); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 105, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_10 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_4 = (__pyx_t_10) ? __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_10) : __Pyx_PyObject_CallNoArg(__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 105, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_FormatSimple(__pyx_t_4, __pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 105, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_9 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) > __pyx_t_9) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) : __pyx_t_9;
      __pyx_t_8 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_t_6);
      __pyx_t_6 = 0;
      __Pyx_INCREF(__pyx_kp_u_trading_mode);
      __pyx_t_8 += 15;
      __Pyx_GIVEREF(__pyx_kp_u_trading_mode);
      PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_kp_u_trading_mode);
      __pyx_t_6 = __Pyx_PyObject_FormatSimple(__pyx_v_self->all_TAs, __pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 105, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) > __pyx_t_9) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) : __pyx_t_9;
      __pyx_t_8 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_t_6);
      __pyx_t_6 = 0;
      __Pyx_INCREF(__pyx_kp_u_technical_evaluator_s);
      __pyx_t_8 += 25;
      __Pyx_GIVEREF(__pyx_kp_u_technical_evaluator_s);
      PyTuple_SET_ITEM(__pyx_t_3, 6, __pyx_kp_u_technical_evaluator_s);

      /* "octobot/strategy_optimizer/strategy_optimizer.py":106
 *                 self.logger.info(f"Trying to find an optimized configuration for {self.strategy_class.get_name()} "
 *                                  f"strategy using {self.trading_mode.get_name()} trading mode, {self.all_TAs} "
 *                                  f"technical evaluator(s), {self.all_time_frames} time frames and {self.risks} "             # <<<<<<<<<<<<<<
 *                                  f"risk(s).")
 * 
 */
      __pyx_t_6 = __Pyx_PyObject_FormatSimple(__pyx_v_self->all_time_frames, __pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 106, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) > __pyx_t_9) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) : __pyx_t_9;
      __pyx_t_8 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_3, 7, __pyx_t_6);
      __pyx_t_6 = 0;
      __Pyx_INCREF(__pyx_kp_u_time_frames_and);
      __pyx_t_8 += 17;
      __Pyx_GIVEREF(__pyx_kp_u_time_frames_and);
      PyTuple_SET_ITEM(__pyx_t_3, 8, __pyx_kp_u_time_frames_and);
      __pyx_t_6 = __Pyx_PyObject_FormatSimple(__pyx_v_self->risks, __pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 106, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) > __pyx_t_9) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) : __pyx_t_9;
      __pyx_t_8 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_3, 9, __pyx_t_6);
      __pyx_t_6 = 0;
      __Pyx_INCREF(__pyx_kp_u_risk_s);
      __pyx_t_8 += 9;
      __Pyx_GIVEREF(__pyx_kp_u_risk_s);
      PyTuple_SET_ITEM(__pyx_t_3, 10, __pyx_kp_u_risk_s);

      /* "octobot/strategy_optimizer/strategy_optimizer.py":104
 *                 self.risks = [1] if risks is None else risks
 * 
 *                 self.logger.info(f"Trying to find an optimized configuration for {self.strategy_class.get_name()} "             # <<<<<<<<<<<<<<
 *                                  f"strategy using {self.trading_mode.get_name()} trading mode, {self.all_TAs} "
 *                                  f"technical evaluator(s), {self.all_time_frames} time frames and {self.risks} "
 */
      __pyx_t_6 = __Pyx_PyUnicode_Join(__pyx_t_3, 11, __pyx_t_8, __pyx_t_9); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 104, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 104, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":109
 *                                  f"risk(s).")
 * 
 *                 self.total_nb_runs = int(len(self.risks) * ((math.pow(2, nb_TFs) - 1) * (math.pow(2, nb_TAs) - 1)))             # <<<<<<<<<<<<<<
 * 
 *                 self.logger.info("Setting logging level to logging.ERROR to limit messages.")
 */
      __pyx_t_1 = __pyx_v_self->risks;
      __Pyx_INCREF(__pyx_t_1);
      if (unlikely(__pyx_t_1 == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 109, __pyx_L5_error)
      }
      __pyx_t_8 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 109, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 109, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_math); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 109, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_pow); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 109, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_int_2, __pyx_v_nb_TFs};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 109, __pyx_L5_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_int_2, __pyx_v_nb_TFs};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 109, __pyx_L5_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 109, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_4);
        if (__pyx_t_6) {
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
        }
        __Pyx_INCREF(__pyx_int_2);
        __Pyx_GIVEREF(__pyx_int_2);
        PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_int_2);
        __Pyx_INCREF(__pyx_v_nb_TFs);
        __Pyx_GIVEREF(__pyx_v_nb_TFs);
        PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_v_nb_TFs);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 109, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyInt_SubtractObjC(__pyx_t_2, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 109, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_math); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 109, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_pow); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 109, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_int_2, __pyx_v_nb_TAs};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 109, __pyx_L5_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_int_2, __pyx_v_nb_TAs};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 109, __pyx_L5_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_10 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 109, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_INCREF(__pyx_int_2);
        __Pyx_GIVEREF(__pyx_int_2);
        PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_5, __pyx_int_2);
        __Pyx_INCREF(__pyx_v_nb_TAs);
        __Pyx_GIVEREF(__pyx_v_nb_TAs);
        PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_5, __pyx_v_nb_TAs);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_10, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 109, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyInt_SubtractObjC(__pyx_t_2, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 109, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = PyNumber_Multiply(__pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 109, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = PyNumber_Multiply(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 109, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyNumber_Int(__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 109, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 109, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_v_self->total_nb_runs = __pyx_t_5;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":111
 *                 self.total_nb_runs = int(len(self.risks) * ((math.pow(2, nb_TFs) - 1) * (math.pow(2, nb_TAs) - 1)))
 * 
 *                 self.logger.info("Setting logging level to logging.ERROR to limit messages.")             # <<<<<<<<<<<<<<
 *                 set_global_logger_level(logging.ERROR)
 * 
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_info); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 111, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_2 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_1, __pyx_kp_u_Setting_logging_level_to_logging) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_kp_u_Setting_logging_level_to_logging);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 111, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":112
 * 
 *                 self.logger.info("Setting logging level to logging.ERROR to limit messages.")
 *                 set_global_logger_level(logging.ERROR)             # <<<<<<<<<<<<<<
 * 
 *                 self.run_id = 1
 */
      __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_set_global_logger_level); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 112, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_logging); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 112, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_ERROR); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 112, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_2 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_1, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_3);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 112, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":114
 *                 set_global_logger_level(logging.ERROR)
 * 
 *                 self.run_id = 1             # <<<<<<<<<<<<<<
 *                 # test with several risks
 *                 for risk in self.risks:
 */
      __pyx_v_self->run_id = 1;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":116
 *                 self.run_id = 1
 *                 # test with several risks
 *                 for risk in self.risks:             # <<<<<<<<<<<<<<
 *                     self.config[CONFIG_TRADING][CONFIG_TRADER_RISK] = risk
 *                     eval_conf_history = []
 */
      if (unlikely(__pyx_v_self->risks == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
        __PYX_ERR(0, 116, __pyx_L5_error)
      }
      __pyx_t_2 = __pyx_v_self->risks; __Pyx_INCREF(__pyx_t_2); __pyx_t_8 = 0;
      for (;;) {
        if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_8); __Pyx_INCREF(__pyx_t_6); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 116, __pyx_L5_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_2, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 116, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
        __Pyx_XDECREF_SET(__pyx_v_risk, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "octobot/strategy_optimizer/strategy_optimizer.py":117
 *                 # test with several risks
 *                 for risk in self.risks:
 *                     self.config[CONFIG_TRADING][CONFIG_TRADER_RISK] = risk             # <<<<<<<<<<<<<<
 *                     eval_conf_history = []
 *                     # test with several evaluators
 */
        if (unlikely(__pyx_v_self->config == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 117, __pyx_L5_error)
        }
        __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_CONFIG_TRADING); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 117, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_self->config, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 117, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_CONFIG_TRADER_RISK); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 117, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_t_6, __pyx_v_risk) < 0)) __PYX_ERR(0, 117, __pyx_L5_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "octobot/strategy_optimizer/strategy_optimizer.py":118
 *                 for risk in self.risks:
 *                     self.config[CONFIG_TRADING][CONFIG_TRADER_RISK] = risk
 *                     eval_conf_history = []             # <<<<<<<<<<<<<<
 *                     # test with several evaluators
 *                     for evaluator_conf_iteration in range(nb_TAs):
 */
        __pyx_t_6 = PyList_New(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 118, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_XDECREF_SET(__pyx_v_eval_conf_history, ((PyObject*)__pyx_t_6));
        __pyx_t_6 = 0;

        /* "octobot/strategy_optimizer/strategy_optimizer.py":120
 *                     eval_conf_history = []
 *                     # test with several evaluators
 *                     for evaluator_conf_iteration in range(nb_TAs):             # <<<<<<<<<<<<<<
 *                         current_forced_evaluator = self.all_TAs[evaluator_conf_iteration]
 *                         # test with 1-n evaluators at a time
 */
        __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_v_nb_TAs); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 120, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
          __pyx_t_3 = __pyx_t_6; __Pyx_INCREF(__pyx_t_3); __pyx_t_11 = 0;
          __pyx_t_12 = NULL;
        } else {
          __pyx_t_11 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 120, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_12 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 120, __pyx_L5_error)
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        for (;;) {
          if (likely(!__pyx_t_12)) {
            if (likely(PyList_CheckExact(__pyx_t_3))) {
              if (__pyx_t_11 >= PyList_GET_SIZE(__pyx_t_3)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_6 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_11); __Pyx_INCREF(__pyx_t_6); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 120, __pyx_L5_error)
              #else
              __pyx_t_6 = PySequence_ITEM(__pyx_t_3, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 120, __pyx_L5_error)
              __Pyx_GOTREF(__pyx_t_6);
              #endif
            } else {
              if (__pyx_t_11 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_11); __Pyx_INCREF(__pyx_t_6); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 120, __pyx_L5_error)
              #else
              __pyx_t_6 = PySequence_ITEM(__pyx_t_3, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 120, __pyx_L5_error)
              __Pyx_GOTREF(__pyx_t_6);
              #endif
            }
          } else {
            __pyx_t_6 = __pyx_t_12(__pyx_t_3);
            if (unlikely(!__pyx_t_6)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 120, __pyx_L5_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_6);
          }
          __Pyx_XDECREF_SET(__pyx_v_evaluator_conf_iteration, __pyx_t_6);
          __pyx_t_6 = 0;

          /* "octobot/strategy_optimizer/strategy_optimizer.py":121
 *                     # test with several evaluators
 *                     for evaluator_conf_iteration in range(nb_TAs):
 *                         current_forced_evaluator = self.all_TAs[evaluator_conf_iteration]             # <<<<<<<<<<<<<<
 *                         # test with 1-n evaluators at a time
 *                         for nb_evaluators in range(1, nb_TAs+1):
 */
          if (unlikely(__pyx_v_self->all_TAs == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(0, 121, __pyx_L5_error)
          }
          __pyx_t_6 = __Pyx_PyObject_GetItem(__pyx_v_self->all_TAs, __pyx_v_evaluator_conf_iteration); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 121, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_XDECREF_SET(__pyx_v_current_forced_evaluator, __pyx_t_6);
          __pyx_t_6 = 0;

          /* "octobot/strategy_optimizer/strategy_optimizer.py":123
 *                         current_forced_evaluator = self.all_TAs[evaluator_conf_iteration]
 *                         # test with 1-n evaluators at a time
 *                         for nb_evaluators in range(1, nb_TAs+1):             # <<<<<<<<<<<<<<
 *                             # test different configurations
 *                             for _ in range(nb_TAs):
 */
          __pyx_t_6 = __Pyx_PyInt_AddObjC(__pyx_v_nb_TAs, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 123, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 123, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_INCREF(__pyx_int_1);
          __Pyx_GIVEREF(__pyx_int_1);
          PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_int_1);
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_range, __pyx_t_1, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 123, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
            __pyx_t_1 = __pyx_t_6; __Pyx_INCREF(__pyx_t_1); __pyx_t_13 = 0;
            __pyx_t_14 = NULL;
          } else {
            __pyx_t_13 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 123, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_14 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 123, __pyx_L5_error)
          }
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          for (;;) {
            if (likely(!__pyx_t_14)) {
              if (likely(PyList_CheckExact(__pyx_t_1))) {
                if (__pyx_t_13 >= PyList_GET_SIZE(__pyx_t_1)) break;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_6 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_13); __Pyx_INCREF(__pyx_t_6); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 123, __pyx_L5_error)
                #else
                __pyx_t_6 = PySequence_ITEM(__pyx_t_1, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 123, __pyx_L5_error)
                __Pyx_GOTREF(__pyx_t_6);
                #endif
              } else {
                if (__pyx_t_13 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_13); __Pyx_INCREF(__pyx_t_6); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 123, __pyx_L5_error)
                #else
                __pyx_t_6 = PySequence_ITEM(__pyx_t_1, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 123, __pyx_L5_error)
                __Pyx_GOTREF(__pyx_t_6);
                #endif
              }
            } else {
              __pyx_t_6 = __pyx_t_14(__pyx_t_1);
              if (unlikely(!__pyx_t_6)) {
                PyObject* exc_type = PyErr_Occurred();
                if (exc_type) {
                  if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                  else __PYX_ERR(0, 123, __pyx_L5_error)
                }
                break;
              }
              __Pyx_GOTREF(__pyx_t_6);
            }
            __Pyx_XDECREF_SET(__pyx_v_nb_evaluators, __pyx_t_6);
            __pyx_t_6 = 0;

            /* "octobot/strategy_optimizer/strategy_optimizer.py":125
 *                         for nb_evaluators in range(1, nb_TAs+1):
 *                             # test different configurations
 *                             for _ in range(nb_TAs):             # <<<<<<<<<<<<<<
 *                                 activated_evaluators = self._get_activated_element(self.all_TAs,
 *                                                                                    current_forced_evaluator,
 */
            __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_v_nb_TAs); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 125, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_6);
            if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
              __pyx_t_10 = __pyx_t_6; __Pyx_INCREF(__pyx_t_10); __pyx_t_15 = 0;
              __pyx_t_16 = NULL;
            } else {
              __pyx_t_15 = -1; __pyx_t_10 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 125, __pyx_L5_error)
              __Pyx_GOTREF(__pyx_t_10);
              __pyx_t_16 = Py_TYPE(__pyx_t_10)->tp_iternext; if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 125, __pyx_L5_error)
            }
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            for (;;) {
              if (likely(!__pyx_t_16)) {
                if (likely(PyList_CheckExact(__pyx_t_10))) {
                  if (__pyx_t_15 >= PyList_GET_SIZE(__pyx_t_10)) break;
                  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  __pyx_t_6 = PyList_GET_ITEM(__pyx_t_10, __pyx_t_15); __Pyx_INCREF(__pyx_t_6); __pyx_t_15++; if (unlikely(0 < 0)) __PYX_ERR(0, 125, __pyx_L5_error)
                  #else
                  __pyx_t_6 = PySequence_ITEM(__pyx_t_10, __pyx_t_15); __pyx_t_15++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 125, __pyx_L5_error)
                  __Pyx_GOTREF(__pyx_t_6);
                  #endif
                } else {
                  if (__pyx_t_15 >= PyTuple_GET_SIZE(__pyx_t_10)) break;
                  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_10, __pyx_t_15); __Pyx_INCREF(__pyx_t_6); __pyx_t_15++; if (unlikely(0 < 0)) __PYX_ERR(0, 125, __pyx_L5_error)
                  #else
                  __pyx_t_6 = PySequence_ITEM(__pyx_t_10, __pyx_t_15); __pyx_t_15++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 125, __pyx_L5_error)
                  __Pyx_GOTREF(__pyx_t_6);
                  #endif
                }
              } else {
                __pyx_t_6 = __pyx_t_16(__pyx_t_10);
                if (unlikely(!__pyx_t_6)) {
                  PyObject* exc_type = PyErr_Occurred();
                  if (exc_type) {
                    if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                    else __PYX_ERR(0, 125, __pyx_L5_error)
                  }
                  break;
                }
                __Pyx_GOTREF(__pyx_t_6);
              }
              __Pyx_XDECREF_SET(__pyx_v__, __pyx_t_6);
              __pyx_t_6 = 0;

              /* "octobot/strategy_optimizer/strategy_optimizer.py":126
 *                             # test different configurations
 *                             for _ in range(nb_TAs):
 *                                 activated_evaluators = self._get_activated_element(self.all_TAs,             # <<<<<<<<<<<<<<
 *                                                                                    current_forced_evaluator,
 *                                                                                    nb_evaluators, eval_conf_history,
 */
              __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_activated_element); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 126, __pyx_L5_error)
              __Pyx_GOTREF(__pyx_t_4);

              /* "octobot/strategy_optimizer/strategy_optimizer.py":129
 *                                                                                    current_forced_evaluator,
 *                                                                                    nb_evaluators, eval_conf_history,
 *                                                                                    self.strategy_class.get_name(),             # <<<<<<<<<<<<<<
 *                                                                                    True)
 *                                 if activated_evaluators is not None:
 */
              __pyx_t_18 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->strategy_class, __pyx_n_s_get_name); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 129, __pyx_L5_error)
              __Pyx_GOTREF(__pyx_t_18);
              __pyx_t_19 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_18))) {
                __pyx_t_19 = PyMethod_GET_SELF(__pyx_t_18);
                if (likely(__pyx_t_19)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_18);
                  __Pyx_INCREF(__pyx_t_19);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_18, function);
                }
              }
              __pyx_t_17 = (__pyx_t_19) ? __Pyx_PyObject_CallOneArg(__pyx_t_18, __pyx_t_19) : __Pyx_PyObject_CallNoArg(__pyx_t_18);
              __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
              if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 129, __pyx_L5_error)
              __Pyx_GOTREF(__pyx_t_17);
              __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;

              /* "octobot/strategy_optimizer/strategy_optimizer.py":130
 *                                                                                    nb_evaluators, eval_conf_history,
 *                                                                                    self.strategy_class.get_name(),
 *                                                                                    True)             # <<<<<<<<<<<<<<
 *                                 if activated_evaluators is not None:
 *                                     self._adapt_tentacles_config(activated_evaluators)
 */
              __pyx_t_18 = NULL;
              __pyx_t_5 = 0;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
                __pyx_t_18 = PyMethod_GET_SELF(__pyx_t_4);
                if (likely(__pyx_t_18)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
                  __Pyx_INCREF(__pyx_t_18);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_4, function);
                  __pyx_t_5 = 1;
                }
              }
              #if CYTHON_FAST_PYCALL
              if (PyFunction_Check(__pyx_t_4)) {
                PyObject *__pyx_temp[7] = {__pyx_t_18, __pyx_v_self->all_TAs, __pyx_v_current_forced_evaluator, __pyx_v_nb_evaluators, __pyx_v_eval_conf_history, __pyx_t_17, Py_True};
                __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_5, 6+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 126, __pyx_L5_error)
                __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
                __Pyx_GOTREF(__pyx_t_6);
                __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
              } else
              #endif
              #if CYTHON_FAST_PYCCALL
              if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
                PyObject *__pyx_temp[7] = {__pyx_t_18, __pyx_v_self->all_TAs, __pyx_v_current_forced_evaluator, __pyx_v_nb_evaluators, __pyx_v_eval_conf_history, __pyx_t_17, Py_True};
                __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_5, 6+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 126, __pyx_L5_error)
                __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
                __Pyx_GOTREF(__pyx_t_6);
                __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
              } else
              #endif
              {
                __pyx_t_19 = PyTuple_New(6+__pyx_t_5); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 126, __pyx_L5_error)
                __Pyx_GOTREF(__pyx_t_19);
                if (__pyx_t_18) {
                  __Pyx_GIVEREF(__pyx_t_18); PyTuple_SET_ITEM(__pyx_t_19, 0, __pyx_t_18); __pyx_t_18 = NULL;
                }
                __Pyx_INCREF(__pyx_v_self->all_TAs);
                __Pyx_GIVEREF(__pyx_v_self->all_TAs);
                PyTuple_SET_ITEM(__pyx_t_19, 0+__pyx_t_5, __pyx_v_self->all_TAs);
                __Pyx_INCREF(__pyx_v_current_forced_evaluator);
                __Pyx_GIVEREF(__pyx_v_current_forced_evaluator);
                PyTuple_SET_ITEM(__pyx_t_19, 1+__pyx_t_5, __pyx_v_current_forced_evaluator);
                __Pyx_INCREF(__pyx_v_nb_evaluators);
                __Pyx_GIVEREF(__pyx_v_nb_evaluators);
                PyTuple_SET_ITEM(__pyx_t_19, 2+__pyx_t_5, __pyx_v_nb_evaluators);
                __Pyx_INCREF(__pyx_v_eval_conf_history);
                __Pyx_GIVEREF(__pyx_v_eval_conf_history);
                PyTuple_SET_ITEM(__pyx_t_19, 3+__pyx_t_5, __pyx_v_eval_conf_history);
                __Pyx_GIVEREF(__pyx_t_17);
                PyTuple_SET_ITEM(__pyx_t_19, 4+__pyx_t_5, __pyx_t_17);
                __Pyx_INCREF(Py_True);
                __Pyx_GIVEREF(Py_True);
                PyTuple_SET_ITEM(__pyx_t_19, 5+__pyx_t_5, Py_True);
                __pyx_t_17 = 0;
                __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_19, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 126, __pyx_L5_error)
                __Pyx_GOTREF(__pyx_t_6);
                __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
              }
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_XDECREF_SET(__pyx_v_activated_evaluators, __pyx_t_6);
              __pyx_t_6 = 0;

              /* "octobot/strategy_optimizer/strategy_optimizer.py":131
 *                                                                                    self.strategy_class.get_name(),
 *                                                                                    True)
 *                                 if activated_evaluators is not None:             # <<<<<<<<<<<<<<
 *                                     self._adapt_tentacles_config(activated_evaluators)
 *                                     time_frames_conf_history = []
 */
              __pyx_t_7 = (__pyx_v_activated_evaluators != Py_None);
              __pyx_t_20 = (__pyx_t_7 != 0);
              if (__pyx_t_20) {

                /* "octobot/strategy_optimizer/strategy_optimizer.py":132
 *                                                                                    True)
 *                                 if activated_evaluators is not None:
 *                                     self._adapt_tentacles_config(activated_evaluators)             # <<<<<<<<<<<<<<
 *                                     time_frames_conf_history = []
 *                                     # test different time frames
 */
                if (!(likely(PyDict_CheckExact(__pyx_v_activated_evaluators))||((__pyx_v_activated_evaluators) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_v_activated_evaluators)->tp_name), 0))) __PYX_ERR(0, 132, __pyx_L5_error)
                ((struct __pyx_vtabstruct_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self->__pyx_vtab)->_adapt_tentacles_config(__pyx_v_self, ((PyObject*)__pyx_v_activated_evaluators));

                /* "octobot/strategy_optimizer/strategy_optimizer.py":133
 *                                 if activated_evaluators is not None:
 *                                     self._adapt_tentacles_config(activated_evaluators)
 *                                     time_frames_conf_history = []             # <<<<<<<<<<<<<<
 *                                     # test different time frames
 *                                     for time_frame_conf_iteration in range(nb_TFs):
 */
                __pyx_t_6 = PyList_New(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 133, __pyx_L5_error)
                __Pyx_GOTREF(__pyx_t_6);
                __Pyx_XDECREF_SET(__pyx_v_time_frames_conf_history, ((PyObject*)__pyx_t_6));
                __pyx_t_6 = 0;

                /* "octobot/strategy_optimizer/strategy_optimizer.py":135
 *                                     time_frames_conf_history = []
 *                                     # test different time frames
 *                                     for time_frame_conf_iteration in range(nb_TFs):             # <<<<<<<<<<<<<<
 *                                         current_forced_time_frame = self.all_time_frames[time_frame_conf_iteration]
 *                                         # test with 1-n time frames at a time
 */
                __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_v_nb_TFs); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 135, __pyx_L5_error)
                __Pyx_GOTREF(__pyx_t_6);
                if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
                  __pyx_t_4 = __pyx_t_6; __Pyx_INCREF(__pyx_t_4); __pyx_t_21 = 0;
                  __pyx_t_22 = NULL;
                } else {
                  __pyx_t_21 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 135, __pyx_L5_error)
                  __Pyx_GOTREF(__pyx_t_4);
                  __pyx_t_22 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 135, __pyx_L5_error)
                }
                __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
                for (;;) {
                  if (likely(!__pyx_t_22)) {
                    if (likely(PyList_CheckExact(__pyx_t_4))) {
                      if (__pyx_t_21 >= PyList_GET_SIZE(__pyx_t_4)) break;
                      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                      __pyx_t_6 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_21); __Pyx_INCREF(__pyx_t_6); __pyx_t_21++; if (unlikely(0 < 0)) __PYX_ERR(0, 135, __pyx_L5_error)
                      #else
                      __pyx_t_6 = PySequence_ITEM(__pyx_t_4, __pyx_t_21); __pyx_t_21++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 135, __pyx_L5_error)
                      __Pyx_GOTREF(__pyx_t_6);
                      #endif
                    } else {
                      if (__pyx_t_21 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
                      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                      __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_21); __Pyx_INCREF(__pyx_t_6); __pyx_t_21++; if (unlikely(0 < 0)) __PYX_ERR(0, 135, __pyx_L5_error)
                      #else
                      __pyx_t_6 = PySequence_ITEM(__pyx_t_4, __pyx_t_21); __pyx_t_21++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 135, __pyx_L5_error)
                      __Pyx_GOTREF(__pyx_t_6);
                      #endif
                    }
                  } else {
                    __pyx_t_6 = __pyx_t_22(__pyx_t_4);
                    if (unlikely(!__pyx_t_6)) {
                      PyObject* exc_type = PyErr_Occurred();
                      if (exc_type) {
                        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                        else __PYX_ERR(0, 135, __pyx_L5_error)
                      }
                      break;
                    }
                    __Pyx_GOTREF(__pyx_t_6);
                  }
                  __Pyx_XDECREF_SET(__pyx_v_time_frame_conf_iteration, __pyx_t_6);
                  __pyx_t_6 = 0;

                  /* "octobot/strategy_optimizer/strategy_optimizer.py":136
 *                                     # test different time frames
 *                                     for time_frame_conf_iteration in range(nb_TFs):
 *                                         current_forced_time_frame = self.all_time_frames[time_frame_conf_iteration]             # <<<<<<<<<<<<<<
 *                                         # test with 1-n time frames at a time
 *                                         for nb_time_frames in range(1, nb_TFs+1):
 */
                  if (unlikely(__pyx_v_self->all_time_frames == Py_None)) {
                    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
                    __PYX_ERR(0, 136, __pyx_L5_error)
                  }
                  __pyx_t_6 = __Pyx_PyObject_GetItem(__pyx_v_self->all_time_frames, __pyx_v_time_frame_conf_iteration); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 136, __pyx_L5_error)
                  __Pyx_GOTREF(__pyx_t_6);
                  __Pyx_XDECREF_SET(__pyx_v_current_forced_time_frame, __pyx_t_6);
                  __pyx_t_6 = 0;

                  /* "octobot/strategy_optimizer/strategy_optimizer.py":138
 *                                         current_forced_time_frame = self.all_time_frames[time_frame_conf_iteration]
 *                                         # test with 1-n time frames at a time
 *                                         for nb_time_frames in range(1, nb_TFs+1):             # <<<<<<<<<<<<<<
 *                                             # test different configurations
 *                                             for _ in range(nb_TFs):
 */
                  __pyx_t_6 = __Pyx_PyInt_AddObjC(__pyx_v_nb_TFs, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 138, __pyx_L5_error)
                  __Pyx_GOTREF(__pyx_t_6);
                  __pyx_t_19 = PyTuple_New(2); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 138, __pyx_L5_error)
                  __Pyx_GOTREF(__pyx_t_19);
                  __Pyx_INCREF(__pyx_int_1);
                  __Pyx_GIVEREF(__pyx_int_1);
                  PyTuple_SET_ITEM(__pyx_t_19, 0, __pyx_int_1);
                  __Pyx_GIVEREF(__pyx_t_6);
                  PyTuple_SET_ITEM(__pyx_t_19, 1, __pyx_t_6);
                  __pyx_t_6 = 0;
                  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_range, __pyx_t_19, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 138, __pyx_L5_error)
                  __Pyx_GOTREF(__pyx_t_6);
                  __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
                  if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
                    __pyx_t_19 = __pyx_t_6; __Pyx_INCREF(__pyx_t_19); __pyx_t_23 = 0;
                    __pyx_t_24 = NULL;
                  } else {
                    __pyx_t_23 = -1; __pyx_t_19 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 138, __pyx_L5_error)
                    __Pyx_GOTREF(__pyx_t_19);
                    __pyx_t_24 = Py_TYPE(__pyx_t_19)->tp_iternext; if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 138, __pyx_L5_error)
                  }
                  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
                  for (;;) {
                    if (likely(!__pyx_t_24)) {
                      if (likely(PyList_CheckExact(__pyx_t_19))) {
                        if (__pyx_t_23 >= PyList_GET_SIZE(__pyx_t_19)) break;
                        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                        __pyx_t_6 = PyList_GET_ITEM(__pyx_t_19, __pyx_t_23); __Pyx_INCREF(__pyx_t_6); __pyx_t_23++; if (unlikely(0 < 0)) __PYX_ERR(0, 138, __pyx_L5_error)
                        #else
                        __pyx_t_6 = PySequence_ITEM(__pyx_t_19, __pyx_t_23); __pyx_t_23++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 138, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_6);
                        #endif
                      } else {
                        if (__pyx_t_23 >= PyTuple_GET_SIZE(__pyx_t_19)) break;
                        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                        __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_19, __pyx_t_23); __Pyx_INCREF(__pyx_t_6); __pyx_t_23++; if (unlikely(0 < 0)) __PYX_ERR(0, 138, __pyx_L5_error)
                        #else
                        __pyx_t_6 = PySequence_ITEM(__pyx_t_19, __pyx_t_23); __pyx_t_23++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 138, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_6);
                        #endif
                      }
                    } else {
                      __pyx_t_6 = __pyx_t_24(__pyx_t_19);
                      if (unlikely(!__pyx_t_6)) {
                        PyObject* exc_type = PyErr_Occurred();
                        if (exc_type) {
                          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                          else __PYX_ERR(0, 138, __pyx_L5_error)
                        }
                        break;
                      }
                      __Pyx_GOTREF(__pyx_t_6);
                    }
                    __Pyx_XDECREF_SET(__pyx_v_nb_time_frames, __pyx_t_6);
                    __pyx_t_6 = 0;

                    /* "octobot/strategy_optimizer/strategy_optimizer.py":140
 *                                         for nb_time_frames in range(1, nb_TFs+1):
 *                                             # test different configurations
 *                                             for _ in range(nb_TFs):             # <<<<<<<<<<<<<<
 *                                                 activated_time_frames = \
 *                                                     self._get_activated_element(self.all_time_frames,
 */
                    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_range, __pyx_v_nb_TFs); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 140, __pyx_L5_error)
                    __Pyx_GOTREF(__pyx_t_6);
                    if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
                      __pyx_t_17 = __pyx_t_6; __Pyx_INCREF(__pyx_t_17); __pyx_t_25 = 0;
                      __pyx_t_26 = NULL;
                    } else {
                      __pyx_t_25 = -1; __pyx_t_17 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 140, __pyx_L5_error)
                      __Pyx_GOTREF(__pyx_t_17);
                      __pyx_t_26 = Py_TYPE(__pyx_t_17)->tp_iternext; if (unlikely(!__pyx_t_26)) __PYX_ERR(0, 140, __pyx_L5_error)
                    }
                    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
                    for (;;) {
                      if (likely(!__pyx_t_26)) {
                        if (likely(PyList_CheckExact(__pyx_t_17))) {
                          if (__pyx_t_25 >= PyList_GET_SIZE(__pyx_t_17)) break;
                          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                          __pyx_t_6 = PyList_GET_ITEM(__pyx_t_17, __pyx_t_25); __Pyx_INCREF(__pyx_t_6); __pyx_t_25++; if (unlikely(0 < 0)) __PYX_ERR(0, 140, __pyx_L5_error)
                          #else
                          __pyx_t_6 = PySequence_ITEM(__pyx_t_17, __pyx_t_25); __pyx_t_25++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 140, __pyx_L5_error)
                          __Pyx_GOTREF(__pyx_t_6);
                          #endif
                        } else {
                          if (__pyx_t_25 >= PyTuple_GET_SIZE(__pyx_t_17)) break;
                          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                          __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_17, __pyx_t_25); __Pyx_INCREF(__pyx_t_6); __pyx_t_25++; if (unlikely(0 < 0)) __PYX_ERR(0, 140, __pyx_L5_error)
                          #else
                          __pyx_t_6 = PySequence_ITEM(__pyx_t_17, __pyx_t_25); __pyx_t_25++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 140, __pyx_L5_error)
                          __Pyx_GOTREF(__pyx_t_6);
                          #endif
                        }
                      } else {
                        __pyx_t_6 = __pyx_t_26(__pyx_t_17);
                        if (unlikely(!__pyx_t_6)) {
                          PyObject* exc_type = PyErr_Occurred();
                          if (exc_type) {
                            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                            else __PYX_ERR(0, 140, __pyx_L5_error)
                          }
                          break;
                        }
                        __Pyx_GOTREF(__pyx_t_6);
                      }
                      __Pyx_DECREF_SET(__pyx_v__, __pyx_t_6);
                      __pyx_t_6 = 0;

                      /* "octobot/strategy_optimizer/strategy_optimizer.py":142
 *                                             for _ in range(nb_TFs):
 *                                                 activated_time_frames = \
 *                                                     self._get_activated_element(self.all_time_frames,             # <<<<<<<<<<<<<<
 *                                                                                 current_forced_time_frame,
 *                                                                                 nb_time_frames,
 */
                      __pyx_t_18 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_activated_element); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 142, __pyx_L5_error)
                      __Pyx_GOTREF(__pyx_t_18);

                      /* "octobot/strategy_optimizer/strategy_optimizer.py":145
 *                                                                                 current_forced_time_frame,
 *                                                                                 nb_time_frames,
 *                                                                                 time_frames_conf_history)             # <<<<<<<<<<<<<<
 *                                                 if activated_time_frames is not None:
 *                                                     self.config[CONFIG_FORCED_TIME_FRAME] = activated_time_frames
 */
                      __pyx_t_27 = NULL;
                      __pyx_t_5 = 0;
                      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_18))) {
                        __pyx_t_27 = PyMethod_GET_SELF(__pyx_t_18);
                        if (likely(__pyx_t_27)) {
                          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_18);
                          __Pyx_INCREF(__pyx_t_27);
                          __Pyx_INCREF(function);
                          __Pyx_DECREF_SET(__pyx_t_18, function);
                          __pyx_t_5 = 1;
                        }
                      }
                      #if CYTHON_FAST_PYCALL
                      if (PyFunction_Check(__pyx_t_18)) {
                        PyObject *__pyx_temp[5] = {__pyx_t_27, __pyx_v_self->all_time_frames, __pyx_v_current_forced_time_frame, __pyx_v_nb_time_frames, __pyx_v_time_frames_conf_history};
                        __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_18, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 142, __pyx_L5_error)
                        __Pyx_XDECREF(__pyx_t_27); __pyx_t_27 = 0;
                        __Pyx_GOTREF(__pyx_t_6);
                      } else
                      #endif
                      #if CYTHON_FAST_PYCCALL
                      if (__Pyx_PyFastCFunction_Check(__pyx_t_18)) {
                        PyObject *__pyx_temp[5] = {__pyx_t_27, __pyx_v_self->all_time_frames, __pyx_v_current_forced_time_frame, __pyx_v_nb_time_frames, __pyx_v_time_frames_conf_history};
                        __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_18, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 142, __pyx_L5_error)
                        __Pyx_XDECREF(__pyx_t_27); __pyx_t_27 = 0;
                        __Pyx_GOTREF(__pyx_t_6);
                      } else
                      #endif
                      {
                        __pyx_t_28 = PyTuple_New(4+__pyx_t_5); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 142, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_28);
                        if (__pyx_t_27) {
                          __Pyx_GIVEREF(__pyx_t_27); PyTuple_SET_ITEM(__pyx_t_28, 0, __pyx_t_27); __pyx_t_27 = NULL;
                        }
                        __Pyx_INCREF(__pyx_v_self->all_time_frames);
                        __Pyx_GIVEREF(__pyx_v_self->all_time_frames);
                        PyTuple_SET_ITEM(__pyx_t_28, 0+__pyx_t_5, __pyx_v_self->all_time_frames);
                        __Pyx_INCREF(__pyx_v_current_forced_time_frame);
                        __Pyx_GIVEREF(__pyx_v_current_forced_time_frame);
                        PyTuple_SET_ITEM(__pyx_t_28, 1+__pyx_t_5, __pyx_v_current_forced_time_frame);
                        __Pyx_INCREF(__pyx_v_nb_time_frames);
                        __Pyx_GIVEREF(__pyx_v_nb_time_frames);
                        PyTuple_SET_ITEM(__pyx_t_28, 2+__pyx_t_5, __pyx_v_nb_time_frames);
                        __Pyx_INCREF(__pyx_v_time_frames_conf_history);
                        __Pyx_GIVEREF(__pyx_v_time_frames_conf_history);
                        PyTuple_SET_ITEM(__pyx_t_28, 3+__pyx_t_5, __pyx_v_time_frames_conf_history);
                        __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_18, __pyx_t_28, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 142, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_6);
                        __Pyx_DECREF(__pyx_t_28); __pyx_t_28 = 0;
                      }
                      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
                      __Pyx_XDECREF_SET(__pyx_v_activated_time_frames, __pyx_t_6);
                      __pyx_t_6 = 0;

                      /* "octobot/strategy_optimizer/strategy_optimizer.py":146
 *                                                                                 nb_time_frames,
 *                                                                                 time_frames_conf_history)
 *                                                 if activated_time_frames is not None:             # <<<<<<<<<<<<<<
 *                                                     self.config[CONFIG_FORCED_TIME_FRAME] = activated_time_frames
 *                                                     print(f"{self.run_id}/{self.total_nb_runs} Run with: evaluators: "
 */
                      __pyx_t_20 = (__pyx_v_activated_time_frames != Py_None);
                      __pyx_t_7 = (__pyx_t_20 != 0);
                      if (__pyx_t_7) {

                        /* "octobot/strategy_optimizer/strategy_optimizer.py":147
 *                                                                                 time_frames_conf_history)
 *                                                 if activated_time_frames is not None:
 *                                                     self.config[CONFIG_FORCED_TIME_FRAME] = activated_time_frames             # <<<<<<<<<<<<<<
 *                                                     print(f"{self.run_id}/{self.total_nb_runs} Run with: evaluators: "
 *                                                           f"{activated_evaluators}, "
 */
                        if (unlikely(__pyx_v_self->config == Py_None)) {
                          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
                          __PYX_ERR(0, 147, __pyx_L5_error)
                        }
                        __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_CONFIG_FORCED_TIME_FRAME); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 147, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_6);
                        if (unlikely(PyDict_SetItem(__pyx_v_self->config, __pyx_t_6, __pyx_v_activated_time_frames) < 0)) __PYX_ERR(0, 147, __pyx_L5_error)
                        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

                        /* "octobot/strategy_optimizer/strategy_optimizer.py":148
 *                                                 if activated_time_frames is not None:
 *                                                     self.config[CONFIG_FORCED_TIME_FRAME] = activated_time_frames
 *                                                     print(f"{self.run_id}/{self.total_nb_runs} Run with: evaluators: "             # <<<<<<<<<<<<<<
 *                                                           f"{activated_evaluators}, "
 *                                                           f"time frames :{activated_time_frames}, risk: {risk}")
 */
                        __pyx_t_6 = PyTuple_New(9); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 148, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_6);
                        __pyx_t_29 = 0;
                        __pyx_t_9 = 127;
                        __pyx_t_18 = __Pyx_PyUnicode_From_int(__pyx_v_self->run_id, 0, ' ', 'd'); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 148, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_18);
                        __pyx_t_29 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_18);
                        __Pyx_GIVEREF(__pyx_t_18);
                        PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_18);
                        __pyx_t_18 = 0;
                        __Pyx_INCREF(__pyx_kp_u_);
                        __pyx_t_29 += 1;
                        __Pyx_GIVEREF(__pyx_kp_u_);
                        PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_kp_u_);
                        __pyx_t_18 = __Pyx_PyUnicode_From_int(__pyx_v_self->total_nb_runs, 0, ' ', 'd'); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 148, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_18);
                        __pyx_t_29 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_18);
                        __Pyx_GIVEREF(__pyx_t_18);
                        PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_18);
                        __pyx_t_18 = 0;
                        __Pyx_INCREF(__pyx_kp_u_Run_with_evaluators);
                        __pyx_t_29 += 23;
                        __Pyx_GIVEREF(__pyx_kp_u_Run_with_evaluators);
                        PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_kp_u_Run_with_evaluators);

                        /* "octobot/strategy_optimizer/strategy_optimizer.py":149
 *                                                     self.config[CONFIG_FORCED_TIME_FRAME] = activated_time_frames
 *                                                     print(f"{self.run_id}/{self.total_nb_runs} Run with: evaluators: "
 *                                                           f"{activated_evaluators}, "             # <<<<<<<<<<<<<<
 *                                                           f"time frames :{activated_time_frames}, risk: {risk}")
 *                                                     self._run_test_suite(self.config, activated_evaluators)
 */
                        __pyx_t_18 = __Pyx_PyObject_FormatSimple(__pyx_v_activated_evaluators, __pyx_empty_unicode); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 149, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_18);
                        __pyx_t_9 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_18) > __pyx_t_9) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_18) : __pyx_t_9;
                        __pyx_t_29 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_18);
                        __Pyx_GIVEREF(__pyx_t_18);
                        PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_t_18);
                        __pyx_t_18 = 0;
                        __Pyx_INCREF(__pyx_kp_u_time_frames);
                        __pyx_t_29 += 15;
                        __Pyx_GIVEREF(__pyx_kp_u_time_frames);
                        PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_kp_u_time_frames);

                        /* "octobot/strategy_optimizer/strategy_optimizer.py":150
 *                                                     print(f"{self.run_id}/{self.total_nb_runs} Run with: evaluators: "
 *                                                           f"{activated_evaluators}, "
 *                                                           f"time frames :{activated_time_frames}, risk: {risk}")             # <<<<<<<<<<<<<<
 *                                                     self._run_test_suite(self.config, activated_evaluators)
 *                                                     print(f" => Result: "
 */
                        __pyx_t_18 = __Pyx_PyObject_FormatSimple(__pyx_v_activated_time_frames, __pyx_empty_unicode); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 150, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_18);
                        __pyx_t_9 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_18) > __pyx_t_9) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_18) : __pyx_t_9;
                        __pyx_t_29 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_18);
                        __Pyx_GIVEREF(__pyx_t_18);
                        PyTuple_SET_ITEM(__pyx_t_6, 6, __pyx_t_18);
                        __pyx_t_18 = 0;
                        __Pyx_INCREF(__pyx_kp_u_risk);
                        __pyx_t_29 += 8;
                        __Pyx_GIVEREF(__pyx_kp_u_risk);
                        PyTuple_SET_ITEM(__pyx_t_6, 7, __pyx_kp_u_risk);
                        __pyx_t_18 = __Pyx_PyObject_FormatSimple(__pyx_v_risk, __pyx_empty_unicode); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 150, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_18);
                        __pyx_t_9 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_18) > __pyx_t_9) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_18) : __pyx_t_9;
                        __pyx_t_29 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_18);
                        __Pyx_GIVEREF(__pyx_t_18);
                        PyTuple_SET_ITEM(__pyx_t_6, 8, __pyx_t_18);
                        __pyx_t_18 = 0;

                        /* "octobot/strategy_optimizer/strategy_optimizer.py":148
 *                                                 if activated_time_frames is not None:
 *                                                     self.config[CONFIG_FORCED_TIME_FRAME] = activated_time_frames
 *                                                     print(f"{self.run_id}/{self.total_nb_runs} Run with: evaluators: "             # <<<<<<<<<<<<<<
 *                                                           f"{activated_evaluators}, "
 *                                                           f"time frames :{activated_time_frames}, risk: {risk}")
 */
                        __pyx_t_18 = __Pyx_PyUnicode_Join(__pyx_t_6, 9, __pyx_t_29, __pyx_t_9); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 148, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_18);
                        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
                        __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_18); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 148, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_6);
                        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
                        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

                        /* "octobot/strategy_optimizer/strategy_optimizer.py":151
 *                                                           f"{activated_evaluators}, "
 *                                                           f"time frames :{activated_time_frames}, risk: {risk}")
 *                                                     self._run_test_suite(self.config, activated_evaluators)             # <<<<<<<<<<<<<<
 *                                                     print(f" => Result: "
 *                                                           f"{self.run_results[-1].get_result_string(False)}")
 */
                        __pyx_t_6 = __pyx_v_self->config;
                        __Pyx_INCREF(__pyx_t_6);
                        if (!(likely(PyDict_CheckExact(__pyx_v_activated_evaluators))||((__pyx_v_activated_evaluators) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_v_activated_evaluators)->tp_name), 0))) __PYX_ERR(0, 151, __pyx_L5_error)
                        ((struct __pyx_vtabstruct_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self->__pyx_vtab)->_run_test_suite(__pyx_v_self, ((PyObject*)__pyx_t_6), ((PyObject*)__pyx_v_activated_evaluators));
                        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

                        /* "octobot/strategy_optimizer/strategy_optimizer.py":153
 *                                                     self._run_test_suite(self.config, activated_evaluators)
 *                                                     print(f" => Result: "
 *                                                           f"{self.run_results[-1].get_result_string(False)}")             # <<<<<<<<<<<<<<
 *                                                     self.run_id += 1
 * 
 */
                        if (unlikely(__pyx_v_self->run_results == Py_None)) {
                          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
                          __PYX_ERR(0, 153, __pyx_L5_error)
                        }
                        __pyx_t_18 = __Pyx_GetItemInt_List(__pyx_v_self->run_results, -1L, long, 1, __Pyx_PyInt_From_long, 1, 1, 1); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 153, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_18);
                        __pyx_t_28 = __Pyx_PyObject_GetAttrStr(__pyx_t_18, __pyx_n_s_get_result_string); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 153, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_28);
                        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
                        __pyx_t_18 = NULL;
                        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_28))) {
                          __pyx_t_18 = PyMethod_GET_SELF(__pyx_t_28);
                          if (likely(__pyx_t_18)) {
                            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_28);
                            __Pyx_INCREF(__pyx_t_18);
                            __Pyx_INCREF(function);
                            __Pyx_DECREF_SET(__pyx_t_28, function);
                          }
                        }
                        __pyx_t_6 = (__pyx_t_18) ? __Pyx_PyObject_Call2Args(__pyx_t_28, __pyx_t_18, Py_False) : __Pyx_PyObject_CallOneArg(__pyx_t_28, Py_False);
                        __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
                        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 153, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_6);
                        __Pyx_DECREF(__pyx_t_28); __pyx_t_28 = 0;
                        __pyx_t_28 = __Pyx_PyObject_FormatSimple(__pyx_t_6, __pyx_empty_unicode); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 153, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_28);
                        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

                        /* "octobot/strategy_optimizer/strategy_optimizer.py":152
 *                                                           f"time frames :{activated_time_frames}, risk: {risk}")
 *                                                     self._run_test_suite(self.config, activated_evaluators)
 *                                                     print(f" => Result: "             # <<<<<<<<<<<<<<
 *                                                           f"{self.run_results[-1].get_result_string(False)}")
 *                                                     self.run_id += 1
 */
                        __pyx_t_6 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Result, __pyx_t_28); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 152, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_6);
                        __Pyx_DECREF(__pyx_t_28); __pyx_t_28 = 0;
                        __pyx_t_28 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_6); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 152, __pyx_L5_error)
                        __Pyx_GOTREF(__pyx_t_28);
                        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
                        __Pyx_DECREF(__pyx_t_28); __pyx_t_28 = 0;

                        /* "octobot/strategy_optimizer/strategy_optimizer.py":154
 *                                                     print(f" => Result: "
 *                                                           f"{self.run_results[-1].get_result_string(False)}")
 *                                                     self.run_id += 1             # <<<<<<<<<<<<<<
 * 
 *                 self._find_optimal_configuration_using_results()
 */
                        __pyx_v_self->run_id = (__pyx_v_self->run_id + 1);

                        /* "octobot/strategy_optimizer/strategy_optimizer.py":146
 *                                                                                 nb_time_frames,
 *                                                                                 time_frames_conf_history)
 *                                                 if activated_time_frames is not None:             # <<<<<<<<<<<<<<
 *                                                     self.config[CONFIG_FORCED_TIME_FRAME] = activated_time_frames
 *                                                     print(f"{self.run_id}/{self.total_nb_runs} Run with: evaluators: "
 */
                      }

                      /* "octobot/strategy_optimizer/strategy_optimizer.py":140
 *                                         for nb_time_frames in range(1, nb_TFs+1):
 *                                             # test different configurations
 *                                             for _ in range(nb_TFs):             # <<<<<<<<<<<<<<
 *                                                 activated_time_frames = \
 *                                                     self._get_activated_element(self.all_time_frames,
 */
                    }
                    __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

                    /* "octobot/strategy_optimizer/strategy_optimizer.py":138
 *                                         current_forced_time_frame = self.all_time_frames[time_frame_conf_iteration]
 *                                         # test with 1-n time frames at a time
 *                                         for nb_time_frames in range(1, nb_TFs+1):             # <<<<<<<<<<<<<<
 *                                             # test different configurations
 *                                             for _ in range(nb_TFs):
 */
                  }
                  __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;

                  /* "octobot/strategy_optimizer/strategy_optimizer.py":135
 *                                     time_frames_conf_history = []
 *                                     # test different time frames
 *                                     for time_frame_conf_iteration in range(nb_TFs):             # <<<<<<<<<<<<<<
 *                                         current_forced_time_frame = self.all_time_frames[time_frame_conf_iteration]
 *                                         # test with 1-n time frames at a time
 */
                }
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

                /* "octobot/strategy_optimizer/strategy_optimizer.py":131
 *                                                                                    self.strategy_class.get_name(),
 *                                                                                    True)
 *                                 if activated_evaluators is not None:             # <<<<<<<<<<<<<<
 *                                     self._adapt_tentacles_config(activated_evaluators)
 *                                     time_frames_conf_history = []
 */
              }

              /* "octobot/strategy_optimizer/strategy_optimizer.py":125
 *                         for nb_evaluators in range(1, nb_TAs+1):
 *                             # test different configurations
 *                             for _ in range(nb_TAs):             # <<<<<<<<<<<<<<
 *                                 activated_evaluators = self._get_activated_element(self.all_TAs,
 *                                                                                    current_forced_evaluator,
 */
            }
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

            /* "octobot/strategy_optimizer/strategy_optimizer.py":123
 *                         current_forced_evaluator = self.all_TAs[evaluator_conf_iteration]
 *                         # test with 1-n evaluators at a time
 *                         for nb_evaluators in range(1, nb_TAs+1):             # <<<<<<<<<<<<<<
 *                             # test different configurations
 *                             for _ in range(nb_TAs):
 */
          }
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "octobot/strategy_optimizer/strategy_optimizer.py":120
 *                     eval_conf_history = []
 *                     # test with several evaluators
 *                     for evaluator_conf_iteration in range(nb_TAs):             # <<<<<<<<<<<<<<
 *                         current_forced_evaluator = self.all_TAs[evaluator_conf_iteration]
 *                         # test with 1-n evaluators at a time
 */
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "octobot/strategy_optimizer/strategy_optimizer.py":116
 *                 self.run_id = 1
 *                 # test with several risks
 *                 for risk in self.risks:             # <<<<<<<<<<<<<<
 *                     self.config[CONFIG_TRADING][CONFIG_TRADER_RISK] = risk
 *                     eval_conf_history = []
 */
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":156
 *                                                     self.run_id += 1
 * 
 *                 self._find_optimal_configuration_using_results()             # <<<<<<<<<<<<<<
 *             finally:
 *                 self.current_test_suite = None
 */
      ((struct __pyx_vtabstruct_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self->__pyx_vtab)->_find_optimal_configuration_using_results(__pyx_v_self);
    }

    /* "octobot/strategy_optimizer/strategy_optimizer.py":158
 *                 self._find_optimal_configuration_using_results()
 *             finally:
 *                 self.current_test_suite = None             # <<<<<<<<<<<<<<
 *                 set_global_logger_level(previous_log_level)
 *                 self.is_computing = False
 */
    /*finally:*/ {
      /*normal exit:*/{
        __Pyx_INCREF(Py_None);
        __Pyx_GIVEREF(Py_None);
        __Pyx_GOTREF(__pyx_v_self->current_test_suite);
        __Pyx_DECREF(__pyx_v_self->current_test_suite);
        __pyx_v_self->current_test_suite = Py_None;

        /* "octobot/strategy_optimizer/strategy_optimizer.py":159
 *             finally:
 *                 self.current_test_suite = None
 *                 set_global_logger_level(previous_log_level)             # <<<<<<<<<<<<<<
 *                 self.is_computing = False
 *                 self.logger.info(f"{self.get_name()} finished computation.")
 */
        __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_set_global_logger_level); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 159, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_1 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_1)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_1, __pyx_v_previous_log_level) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_previous_log_level);
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 159, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "octobot/strategy_optimizer/strategy_optimizer.py":160
 *                 self.current_test_suite = None
 *                 set_global_logger_level(previous_log_level)
 *                 self.is_computing = False             # <<<<<<<<<<<<<<
 *                 self.logger.info(f"{self.get_name()} finished computation.")
 *                 self.logger.info("Logging level restored.")
 */
        __pyx_v_self->is_computing = 0;

        /* "octobot/strategy_optimizer/strategy_optimizer.py":161
 *                 set_global_logger_level(previous_log_level)
 *                 self.is_computing = False
 *                 self.logger.info(f"{self.get_name()} finished computation.")             # <<<<<<<<<<<<<<
 *                 self.logger.info("Logging level restored.")
 *         else:
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_info); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 161, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_name); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 161, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
          }
        }
        __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_10);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 161, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_10 = __Pyx_PyObject_FormatSimple(__pyx_t_1, __pyx_empty_unicode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 161, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyUnicode_Concat(__pyx_t_10, __pyx_kp_u_finished_computation); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 161, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_10 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_10)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_10);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_10, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1);
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 161, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "octobot/strategy_optimizer/strategy_optimizer.py":162
 *                 self.is_computing = False
 *                 self.logger.info(f"{self.get_name()} finished computation.")
 *                 self.logger.info("Logging level restored.")             # <<<<<<<<<<<<<<
 *         else:
 *             raise RuntimeError(f"{self.get_name()} is already computing: processed "
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_info); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 162, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_1 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_1)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_1, __pyx_kp_u_Logging_level_restored) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_Logging_level_restored);
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 162, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        goto __pyx_L6;
      }
      __pyx_L5_error:;
      /*exception exit:*/{
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __pyx_t_32 = 0; __pyx_t_33 = 0; __pyx_t_34 = 0; __pyx_t_35 = 0; __pyx_t_36 = 0; __pyx_t_37 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
        __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
        __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_27); __pyx_t_27 = 0;
        __Pyx_XDECREF(__pyx_t_28); __pyx_t_28 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_35, &__pyx_t_36, &__pyx_t_37);
        if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_32, &__pyx_t_33, &__pyx_t_34) < 0)) __Pyx_ErrFetch(&__pyx_t_32, &__pyx_t_33, &__pyx_t_34);
        __Pyx_XGOTREF(__pyx_t_32);
        __Pyx_XGOTREF(__pyx_t_33);
        __Pyx_XGOTREF(__pyx_t_34);
        __Pyx_XGOTREF(__pyx_t_35);
        __Pyx_XGOTREF(__pyx_t_36);
        __Pyx_XGOTREF(__pyx_t_37);
        __pyx_t_5 = __pyx_lineno; __pyx_t_30 = __pyx_clineno; __pyx_t_31 = __pyx_filename;
        {

          /* "octobot/strategy_optimizer/strategy_optimizer.py":158
 *                 self._find_optimal_configuration_using_results()
 *             finally:
 *                 self.current_test_suite = None             # <<<<<<<<<<<<<<
 *                 set_global_logger_level(previous_log_level)
 *                 self.is_computing = False
 */
          __Pyx_INCREF(Py_None);
          __Pyx_GIVEREF(Py_None);
          __Pyx_GOTREF(__pyx_v_self->current_test_suite);
          __Pyx_DECREF(__pyx_v_self->current_test_suite);
          __pyx_v_self->current_test_suite = Py_None;

          /* "octobot/strategy_optimizer/strategy_optimizer.py":159
 *             finally:
 *                 self.current_test_suite = None
 *                 set_global_logger_level(previous_log_level)             # <<<<<<<<<<<<<<
 *                 self.is_computing = False
 *                 self.logger.info(f"{self.get_name()} finished computation.")
 */
          __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_set_global_logger_level); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 159, __pyx_L24_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_1 = NULL;
          if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
            if (likely(__pyx_t_1)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_1);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_3, function);
            }
          }
          __pyx_t_2 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_1, __pyx_v_previous_log_level) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_previous_log_level);
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 159, __pyx_L24_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "octobot/strategy_optimizer/strategy_optimizer.py":160
 *                 self.current_test_suite = None
 *                 set_global_logger_level(previous_log_level)
 *                 self.is_computing = False             # <<<<<<<<<<<<<<
 *                 self.logger.info(f"{self.get_name()} finished computation.")
 *                 self.logger.info("Logging level restored.")
 */
          __pyx_v_self->is_computing = 0;

          /* "octobot/strategy_optimizer/strategy_optimizer.py":161
 *                 set_global_logger_level(previous_log_level)
 *                 self.is_computing = False
 *                 self.logger.info(f"{self.get_name()} finished computation.")             # <<<<<<<<<<<<<<
 *                 self.logger.info("Logging level restored.")
 *         else:
 */
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_info); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 161, __pyx_L24_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_10 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_name); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 161, __pyx_L24_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_4 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
            __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_10);
            if (likely(__pyx_t_4)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
              __Pyx_INCREF(__pyx_t_4);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_10, function);
            }
          }
          __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_10);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 161, __pyx_L24_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __pyx_t_10 = __Pyx_PyObject_FormatSimple(__pyx_t_1, __pyx_empty_unicode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 161, __pyx_L24_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_1 = __Pyx_PyUnicode_Concat(__pyx_t_10, __pyx_kp_u_finished_computation); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 161, __pyx_L24_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __pyx_t_10 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
            if (likely(__pyx_t_10)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_10);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_3, function);
            }
          }
          __pyx_t_2 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_10, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1);
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 161, __pyx_L24_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "octobot/strategy_optimizer/strategy_optimizer.py":162
 *                 self.is_computing = False
 *                 self.logger.info(f"{self.get_name()} finished computation.")
 *                 self.logger.info("Logging level restored.")             # <<<<<<<<<<<<<<
 *         else:
 *             raise RuntimeError(f"{self.get_name()} is already computing: processed "
 */
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_info); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 162, __pyx_L24_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_1 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
            if (likely(__pyx_t_1)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_1);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_3, function);
            }
          }
          __pyx_t_2 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_1, __pyx_kp_u_Logging_level_restored) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_Logging_level_restored);
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 162, __pyx_L24_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }
        if (PY_MAJOR_VERSION >= 3) {
          __Pyx_XGIVEREF(__pyx_t_35);
          __Pyx_XGIVEREF(__pyx_t_36);
          __Pyx_XGIVEREF(__pyx_t_37);
          __Pyx_ExceptionReset(__pyx_t_35, __pyx_t_36, __pyx_t_37);
        }
        __Pyx_XGIVEREF(__pyx_t_32);
        __Pyx_XGIVEREF(__pyx_t_33);
        __Pyx_XGIVEREF(__pyx_t_34);
        __Pyx_ErrRestore(__pyx_t_32, __pyx_t_33, __pyx_t_34);
        __pyx_t_32 = 0; __pyx_t_33 = 0; __pyx_t_34 = 0; __pyx_t_35 = 0; __pyx_t_36 = 0; __pyx_t_37 = 0;
        __pyx_lineno = __pyx_t_5; __pyx_clineno = __pyx_t_30; __pyx_filename = __pyx_t_31;
        goto __pyx_L1_error;
        __pyx_L24_error:;
        if (PY_MAJOR_VERSION >= 3) {
          __Pyx_XGIVEREF(__pyx_t_35);
          __Pyx_XGIVEREF(__pyx_t_36);
          __Pyx_XGIVEREF(__pyx_t_37);
          __Pyx_ExceptionReset(__pyx_t_35, __pyx_t_36, __pyx_t_37);
        }
        __Pyx_XDECREF(__pyx_t_32); __pyx_t_32 = 0;
        __Pyx_XDECREF(__pyx_t_33); __pyx_t_33 = 0;
        __Pyx_XDECREF(__pyx_t_34); __pyx_t_34 = 0;
        __pyx_t_35 = 0; __pyx_t_36 = 0; __pyx_t_37 = 0;
        goto __pyx_L1_error;
      }
      __pyx_L6:;
    }

    /* "octobot/strategy_optimizer/strategy_optimizer.py":82
 * 
 *     def find_optimal_configuration(self, TAs=None, time_frames=None, risks=None):
 *         if not self.is_computing:             # <<<<<<<<<<<<<<
 * 
 *             # set is_computing to True to prevent any simultaneous start
 */
    goto __pyx_L3;
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":164
 *                 self.logger.info("Logging level restored.")
 *         else:
 *             raise RuntimeError(f"{self.get_name()} is already computing: processed "             # <<<<<<<<<<<<<<
 *                                f"{self.run_id}/{self.total_nb_runs} processed")
 * 
 */
  /*else*/ {
    __pyx_t_2 = PyTuple_New(6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = 0;
    __pyx_t_9 = 127;
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_3 = (__pyx_t_10) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_10) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_t_3, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_9 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) > __pyx_t_9) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) : __pyx_t_9;
    __pyx_t_8 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __pyx_t_1 = 0;
    __Pyx_INCREF(__pyx_kp_u_is_already_computing_processed);
    __pyx_t_8 += 33;
    __Pyx_GIVEREF(__pyx_kp_u_is_already_computing_processed);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u_is_already_computing_processed);

    /* "octobot/strategy_optimizer/strategy_optimizer.py":165
 *         else:
 *             raise RuntimeError(f"{self.get_name()} is already computing: processed "
 *                                f"{self.run_id}/{self.total_nb_runs} processed")             # <<<<<<<<<<<<<<
 * 
 *     def _run_test_suite(self, config, evaluators):
 */
    __pyx_t_1 = __Pyx_PyUnicode_From_int(__pyx_v_self->run_id, 0, ' ', 'd'); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_1);
    __pyx_t_1 = 0;
    __Pyx_INCREF(__pyx_kp_u_);
    __pyx_t_8 += 1;
    __Pyx_GIVEREF(__pyx_kp_u_);
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_kp_u_);
    __pyx_t_1 = __Pyx_PyUnicode_From_int(__pyx_v_self->total_nb_runs, 0, ' ', 'd'); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 4, __pyx_t_1);
    __pyx_t_1 = 0;
    __Pyx_INCREF(__pyx_kp_u_processed);
    __pyx_t_8 += 10;
    __Pyx_GIVEREF(__pyx_kp_u_processed);
    PyTuple_SET_ITEM(__pyx_t_2, 5, __pyx_kp_u_processed);

    /* "octobot/strategy_optimizer/strategy_optimizer.py":164
 *                 self.logger.info("Logging level restored.")
 *         else:
 *             raise RuntimeError(f"{self.get_name()} is already computing: processed "             # <<<<<<<<<<<<<<
 *                                f"{self.run_id}/{self.total_nb_runs} processed")
 * 
 */
    __pyx_t_1 = __Pyx_PyUnicode_Join(__pyx_t_2, 6, __pyx_t_8, __pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_RuntimeError, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 164, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":81
 *             self.is_properly_initialized = True
 * 
 *     def find_optimal_configuration(self, TAs=None, time_frames=None, risks=None):             # <<<<<<<<<<<<<<
 *         if not self.is_computing:
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_XDECREF(__pyx_t_19);
  __Pyx_XDECREF(__pyx_t_27);
  __Pyx_XDECREF(__pyx_t_28);
  __Pyx_WriteUnraisable("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.find_optimal_configuration", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_previous_log_level);
  __Pyx_XDECREF(__pyx_v_nb_TAs);
  __Pyx_XDECREF(__pyx_v_nb_TFs);
  __Pyx_XDECREF(__pyx_v_risk);
  __Pyx_XDECREF(__pyx_v_eval_conf_history);
  __Pyx_XDECREF(__pyx_v_evaluator_conf_iteration);
  __Pyx_XDECREF(__pyx_v_current_forced_evaluator);
  __Pyx_XDECREF(__pyx_v_nb_evaluators);
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_XDECREF(__pyx_v_activated_evaluators);
  __Pyx_XDECREF(__pyx_v_time_frames_conf_history);
  __Pyx_XDECREF(__pyx_v_time_frame_conf_iteration);
  __Pyx_XDECREF(__pyx_v_current_forced_time_frame);
  __Pyx_XDECREF(__pyx_v_nb_time_frames);
  __Pyx_XDECREF(__pyx_v_activated_time_frames);
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_3find_optimal_configuration(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_3find_optimal_configuration = {"find_optimal_configuration", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_3find_optimal_configuration, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_3find_optimal_configuration(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_TAs = 0;
  PyObject *__pyx_v_time_frames = 0;
  PyObject *__pyx_v_risks = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("find_optimal_configuration (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_TAs,&__pyx_n_s_time_frames_2,&__pyx_n_s_risks,0};
    PyObject* values[3] = {0,0,0};
    values[0] = ((PyObject*)Py_None);
    values[1] = ((PyObject*)Py_None);
    values[2] = ((PyObject*)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_TAs);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_time_frames_2);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_risks);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "find_optimal_configuration") < 0)) __PYX_ERR(0, 81, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_TAs = ((PyObject*)values[0]);
    __pyx_v_time_frames = ((PyObject*)values[1]);
    __pyx_v_risks = ((PyObject*)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("find_optimal_configuration", 0, 0, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 81, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.find_optimal_configuration", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_TAs), (&PyList_Type), 1, "TAs", 1))) __PYX_ERR(0, 81, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_time_frames), (&PyList_Type), 1, "time_frames", 1))) __PYX_ERR(0, 81, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_risks), (&PyList_Type), 1, "risks", 1))) __PYX_ERR(0, 81, __pyx_L1_error)
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_2find_optimal_configuration(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self), __pyx_v_TAs, __pyx_v_time_frames, __pyx_v_risks);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_2find_optimal_configuration(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_TAs, PyObject *__pyx_v_time_frames, PyObject *__pyx_v_risks) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  struct __pyx_opt_args_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_find_optimal_configuration __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("find_optimal_configuration", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1.__pyx_n = 3;
  __pyx_t_1.TAs = __pyx_v_TAs;
  __pyx_t_1.time_frames = __pyx_v_time_frames;
  __pyx_t_1.risks = __pyx_v_risks;
  __pyx_vtabptr_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->find_optimal_configuration(__pyx_v_self, 1, &__pyx_t_1); 
  __pyx_t_2 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.find_optimal_configuration", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":167
 *                                f"{self.run_id}/{self.total_nb_runs} processed")
 * 
 *     def _run_test_suite(self, config, evaluators):             # <<<<<<<<<<<<<<
 *         self.current_test_suite = StrategyTestSuite()
 *         self.current_test_suite.evaluators = evaluators
 */

static void __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer__run_test_suite(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_config, PyObject *__pyx_v_evaluators) {
  PyObject *__pyx_v_no_error = NULL;
  PyObject *__pyx_v_run_result = NULL;
  PyObject *__pyx_7genexpr__pyx_v_e = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *(*__pyx_t_10)(PyObject *);
  int __pyx_t_11;
  __Pyx_RefNannySetupContext("_run_test_suite", 0);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":168
 * 
 *     def _run_test_suite(self, config, evaluators):
 *         self.current_test_suite = StrategyTestSuite()             # <<<<<<<<<<<<<<
 *         self.current_test_suite.evaluators = evaluators
 *         self.current_test_suite.initialize_with_strategy(self.strategy_class,
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_StrategyTestSuite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->current_test_suite);
  __Pyx_DECREF(__pyx_v_self->current_test_suite);
  __pyx_v_self->current_test_suite = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":169
 *     def _run_test_suite(self, config, evaluators):
 *         self.current_test_suite = StrategyTestSuite()
 *         self.current_test_suite.evaluators = evaluators             # <<<<<<<<<<<<<<
 *         self.current_test_suite.initialize_with_strategy(self.strategy_class,
 *                                                          self.tentacles_setup_config,
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self->current_test_suite, __pyx_n_s_evaluators, __pyx_v_evaluators) < 0) __PYX_ERR(0, 169, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":170
 *         self.current_test_suite = StrategyTestSuite()
 *         self.current_test_suite.evaluators = evaluators
 *         self.current_test_suite.initialize_with_strategy(self.strategy_class,             # <<<<<<<<<<<<<<
 *                                                          self.tentacles_setup_config,
 *                                                          deepcopy(config))
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->current_test_suite, __pyx_n_s_initialize_with_strategy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":172
 *         self.current_test_suite.initialize_with_strategy(self.strategy_class,
 *                                                          self.tentacles_setup_config,
 *                                                          deepcopy(config))             # <<<<<<<<<<<<<<
 *         no_error = asyncio.run(self.current_test_suite.run_test_suite(self.current_test_suite),
 *                                debug=OPTIMIZER_FORCE_ASYNCIO_DEBUG_OPTION)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_deepcopy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_v_config) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_config);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_self->strategy_class, __pyx_v_self->tentacles_setup_config, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_self->strategy_class, __pyx_v_self->tentacles_setup_config, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_self->strategy_class);
    __Pyx_GIVEREF(__pyx_v_self->strategy_class);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_6, __pyx_v_self->strategy_class);
    __Pyx_INCREF(__pyx_v_self->tentacles_setup_config);
    __Pyx_GIVEREF(__pyx_v_self->tentacles_setup_config);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_6, __pyx_v_self->tentacles_setup_config);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_6, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":173
 *                                                          self.tentacles_setup_config,
 *                                                          deepcopy(config))
 *         no_error = asyncio.run(self.current_test_suite.run_test_suite(self.current_test_suite),             # <<<<<<<<<<<<<<
 *                                debug=OPTIMIZER_FORCE_ASYNCIO_DEBUG_OPTION)
 *         if not no_error:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_asyncio); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_run); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->current_test_suite, __pyx_n_s_run_test_suite); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_3, __pyx_v_self->current_test_suite) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_self->current_test_suite);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":174
 *                                                          deepcopy(config))
 *         no_error = asyncio.run(self.current_test_suite.run_test_suite(self.current_test_suite),
 *                                debug=OPTIMIZER_FORCE_ASYNCIO_DEBUG_OPTION)             # <<<<<<<<<<<<<<
 *         if not no_error:
 *             self.errors = self.errors.union(set([str(e) for e in self.current_test_suite.exceptions]))
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_OPTIMIZER_FORCE_ASYNCIO_DEBUG_OP); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_debug, __pyx_t_3) < 0) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":173
 *                                                          self.tentacles_setup_config,
 *                                                          deepcopy(config))
 *         no_error = asyncio.run(self.current_test_suite.run_test_suite(self.current_test_suite),             # <<<<<<<<<<<<<<
 *                                debug=OPTIMIZER_FORCE_ASYNCIO_DEBUG_OPTION)
 *         if not no_error:
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_no_error = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":175
 *         no_error = asyncio.run(self.current_test_suite.run_test_suite(self.current_test_suite),
 *                                debug=OPTIMIZER_FORCE_ASYNCIO_DEBUG_OPTION)
 *         if not no_error:             # <<<<<<<<<<<<<<
 *             self.errors = self.errors.union(set([str(e) for e in self.current_test_suite.exceptions]))
 *         run_result = self.current_test_suite.get_test_suite_result()
 */
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_no_error); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 175, __pyx_L1_error)
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "octobot/strategy_optimizer/strategy_optimizer.py":176
 *                                debug=OPTIMIZER_FORCE_ASYNCIO_DEBUG_OPTION)
 *         if not no_error:
 *             self.errors = self.errors.union(set([str(e) for e in self.current_test_suite.exceptions]))             # <<<<<<<<<<<<<<
 *         run_result = self.current_test_suite.get_test_suite_result()
 *         self.run_results.append(run_result)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->errors, __pyx_n_s_union); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 176, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    { /* enter inner scope */
      __pyx_t_5 = PyList_New(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 176, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->current_test_suite, __pyx_n_s_exceptions); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 176, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
        __pyx_t_4 = __pyx_t_2; __Pyx_INCREF(__pyx_t_4); __pyx_t_9 = 0;
        __pyx_t_10 = NULL;
      } else {
        __pyx_t_9 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 176, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_10 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 176, __pyx_L6_error)
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      for (;;) {
        if (likely(!__pyx_t_10)) {
          if (likely(PyList_CheckExact(__pyx_t_4))) {
            if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_4)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_2 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 176, __pyx_L6_error)
            #else
            __pyx_t_2 = PySequence_ITEM(__pyx_t_4, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 176, __pyx_L6_error)
            __Pyx_GOTREF(__pyx_t_2);
            #endif
          } else {
            if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 176, __pyx_L6_error)
            #else
            __pyx_t_2 = PySequence_ITEM(__pyx_t_4, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 176, __pyx_L6_error)
            __Pyx_GOTREF(__pyx_t_2);
            #endif
          }
        } else {
          __pyx_t_2 = __pyx_t_10(__pyx_t_4);
          if (unlikely(!__pyx_t_2)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 176, __pyx_L6_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_XDECREF_SET(__pyx_7genexpr__pyx_v_e, __pyx_t_2);
        __pyx_t_2 = 0;
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyUnicode_Type)), __pyx_7genexpr__pyx_v_e); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 176, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_5, (PyObject*)__pyx_t_2))) __PYX_ERR(0, 176, __pyx_L6_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_7genexpr__pyx_v_e); __pyx_7genexpr__pyx_v_e = 0;
      goto __pyx_L9_exit_scope;
      __pyx_L6_error:;
      __Pyx_XDECREF(__pyx_7genexpr__pyx_v_e); __pyx_7genexpr__pyx_v_e = 0;
      goto __pyx_L1_error;
      __pyx_L9_exit_scope:;
    } /* exit inner scope */
    __pyx_t_4 = PySet_New(__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 176, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 176, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (!(likely(PySet_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "set", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 176, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->errors);
    __Pyx_DECREF(__pyx_v_self->errors);
    __pyx_v_self->errors = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":175
 *         no_error = asyncio.run(self.current_test_suite.run_test_suite(self.current_test_suite),
 *                                debug=OPTIMIZER_FORCE_ASYNCIO_DEBUG_OPTION)
 *         if not no_error:             # <<<<<<<<<<<<<<
 *             self.errors = self.errors.union(set([str(e) for e in self.current_test_suite.exceptions]))
 *         run_result = self.current_test_suite.get_test_suite_result()
 */
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":177
 *         if not no_error:
 *             self.errors = self.errors.union(set([str(e) for e in self.current_test_suite.exceptions]))
 *         run_result = self.current_test_suite.get_test_suite_result()             # <<<<<<<<<<<<<<
 *         self.run_results.append(run_result)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->current_test_suite, __pyx_n_s_get_test_suite_result); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_run_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":178
 *             self.errors = self.errors.union(set([str(e) for e in self.current_test_suite.exceptions]))
 *         run_result = self.current_test_suite.get_test_suite_result()
 *         self.run_results.append(run_result)             # <<<<<<<<<<<<<<
 * 
 *     def _adapt_tentacles_config(self, activated_evaluators):
 */
  if (unlikely(__pyx_v_self->run_results == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(0, 178, __pyx_L1_error)
  }
  __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_self->run_results, __pyx_v_run_result); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 178, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":167
 *                                f"{self.run_id}/{self.total_nb_runs} processed")
 * 
 *     def _run_test_suite(self, config, evaluators):             # <<<<<<<<<<<<<<
 *         self.current_test_suite = StrategyTestSuite()
 *         self.current_test_suite.evaluators = evaluators
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer._run_test_suite", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_no_error);
  __Pyx_XDECREF(__pyx_v_run_result);
  __Pyx_XDECREF(__pyx_7genexpr__pyx_v_e);
  __Pyx_RefNannyFinishContext();
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":180
 *         self.run_results.append(run_result)
 * 
 *     def _adapt_tentacles_config(self, activated_evaluators):             # <<<<<<<<<<<<<<
 *         to_update_config = {}
 *         for tentacle_class_name in get_tentacles_activation(self.tentacles_setup_config):
 */

static void __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer__adapt_tentacles_config(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_activated_evaluators) {
  PyObject *__pyx_v_to_update_config = NULL;
  PyObject *__pyx_v_tentacle_class_name = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  __Pyx_RefNannySetupContext("_adapt_tentacles_config", 0);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":181
 * 
 *     def _adapt_tentacles_config(self, activated_evaluators):
 *         to_update_config = {}             # <<<<<<<<<<<<<<
 *         for tentacle_class_name in get_tentacles_activation(self.tentacles_setup_config):
 *             if tentacle_class_name in activated_evaluators:
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_to_update_config = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":182
 *     def _adapt_tentacles_config(self, activated_evaluators):
 *         to_update_config = {}
 *         for tentacle_class_name in get_tentacles_activation(self.tentacles_setup_config):             # <<<<<<<<<<<<<<
 *             if tentacle_class_name in activated_evaluators:
 *                 to_update_config[tentacle_class_name] = True
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_get_tentacles_activation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_self->tentacles_setup_config) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_self->tentacles_setup_config);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 182, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 182, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 182, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 182, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 182, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 182, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_5(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 182, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_tentacle_class_name, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":183
 *         to_update_config = {}
 *         for tentacle_class_name in get_tentacles_activation(self.tentacles_setup_config):
 *             if tentacle_class_name in activated_evaluators:             # <<<<<<<<<<<<<<
 *                 to_update_config[tentacle_class_name] = True
 *             elif get_class_from_string(tentacle_class_name, StrategyEvaluator, Strategies,
 */
    if (unlikely(__pyx_v_activated_evaluators == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 183, __pyx_L1_error)
    }
    __pyx_t_6 = (__Pyx_PyDict_ContainsTF(__pyx_v_tentacle_class_name, __pyx_v_activated_evaluators, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 183, __pyx_L1_error)
    __pyx_t_7 = (__pyx_t_6 != 0);
    if (__pyx_t_7) {

      /* "octobot/strategy_optimizer/strategy_optimizer.py":184
 *         for tentacle_class_name in get_tentacles_activation(self.tentacles_setup_config):
 *             if tentacle_class_name in activated_evaluators:
 *                 to_update_config[tentacle_class_name] = True             # <<<<<<<<<<<<<<
 *             elif get_class_from_string(tentacle_class_name, StrategyEvaluator, Strategies,
 *                                        evaluator_parent_inspection) is None and \
 */
      if (unlikely(PyDict_SetItem(__pyx_v_to_update_config, __pyx_v_tentacle_class_name, Py_True) < 0)) __PYX_ERR(0, 184, __pyx_L1_error)

      /* "octobot/strategy_optimizer/strategy_optimizer.py":183
 *         to_update_config = {}
 *         for tentacle_class_name in get_tentacles_activation(self.tentacles_setup_config):
 *             if tentacle_class_name in activated_evaluators:             # <<<<<<<<<<<<<<
 *                 to_update_config[tentacle_class_name] = True
 *             elif get_class_from_string(tentacle_class_name, StrategyEvaluator, Strategies,
 */
      goto __pyx_L5;
    }

    /* "octobot/strategy_optimizer/strategy_optimizer.py":185
 *             if tentacle_class_name in activated_evaluators:
 *                 to_update_config[tentacle_class_name] = True
 *             elif get_class_from_string(tentacle_class_name, StrategyEvaluator, Strategies,             # <<<<<<<<<<<<<<
 *                                        evaluator_parent_inspection) is None and \
 *                 get_class_from_string(tentacle_class_name, AbstractTradingMode, Mode,
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_get_class_from_string); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_StrategyEvaluator); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_Strategies); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);

    /* "octobot/strategy_optimizer/strategy_optimizer.py":186
 *                 to_update_config[tentacle_class_name] = True
 *             elif get_class_from_string(tentacle_class_name, StrategyEvaluator, Strategies,
 *                                        evaluator_parent_inspection) is None and \             # <<<<<<<<<<<<<<
 *                 get_class_from_string(tentacle_class_name, AbstractTradingMode, Mode,
 *                                       trading_mode_parent_inspection) is None:
 */
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_evaluator_parent_inspection); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 186, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = NULL;
    __pyx_t_12 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_12 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[5] = {__pyx_t_11, __pyx_v_tentacle_class_name, __pyx_t_8, __pyx_t_9, __pyx_t_10};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_12, 4+__pyx_t_12); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 185, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[5] = {__pyx_t_11, __pyx_v_tentacle_class_name, __pyx_t_8, __pyx_t_9, __pyx_t_10};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_12, 4+__pyx_t_12); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 185, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    } else
    #endif
    {
      __pyx_t_13 = PyTuple_New(4+__pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 185, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      if (__pyx_t_11) {
        __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_11); __pyx_t_11 = NULL;
      }
      __Pyx_INCREF(__pyx_v_tentacle_class_name);
      __Pyx_GIVEREF(__pyx_v_tentacle_class_name);
      PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_12, __pyx_v_tentacle_class_name);
      __Pyx_GIVEREF(__pyx_t_8);
      PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_12, __pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_12, __pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_10);
      PyTuple_SET_ITEM(__pyx_t_13, 3+__pyx_t_12, __pyx_t_10);
      __pyx_t_8 = 0;
      __pyx_t_9 = 0;
      __pyx_t_10 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_13, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 185, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = (__pyx_t_1 == Py_None);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_14 = (__pyx_t_6 != 0);
    if (__pyx_t_14) {
    } else {
      __pyx_t_7 = __pyx_t_14;
      goto __pyx_L6_bool_binop_done;
    }

    /* "octobot/strategy_optimizer/strategy_optimizer.py":187
 *             elif get_class_from_string(tentacle_class_name, StrategyEvaluator, Strategies,
 *                                        evaluator_parent_inspection) is None and \
 *                 get_class_from_string(tentacle_class_name, AbstractTradingMode, Mode,             # <<<<<<<<<<<<<<
 *                                       trading_mode_parent_inspection) is None:
 *                 to_update_config[tentacle_class_name] = False
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_get_class_from_string); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_n_s_AbstractTradingMode); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_Mode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);

    /* "octobot/strategy_optimizer/strategy_optimizer.py":188
 *                                        evaluator_parent_inspection) is None and \
 *                 get_class_from_string(tentacle_class_name, AbstractTradingMode, Mode,
 *                                       trading_mode_parent_inspection) is None:             # <<<<<<<<<<<<<<
 *                 to_update_config[tentacle_class_name] = False
 *         update_activation_configuration(self.tentacles_setup_config, to_update_config, False)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_trading_mode_parent_inspection); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = NULL;
    __pyx_t_12 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_12 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[5] = {__pyx_t_8, __pyx_v_tentacle_class_name, __pyx_t_13, __pyx_t_10, __pyx_t_9};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_12, 4+__pyx_t_12); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[5] = {__pyx_t_8, __pyx_v_tentacle_class_name, __pyx_t_13, __pyx_t_10, __pyx_t_9};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_12, 4+__pyx_t_12); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    } else
    #endif
    {
      __pyx_t_11 = PyTuple_New(4+__pyx_t_12); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 187, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      if (__pyx_t_8) {
        __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_8); __pyx_t_8 = NULL;
      }
      __Pyx_INCREF(__pyx_v_tentacle_class_name);
      __Pyx_GIVEREF(__pyx_v_tentacle_class_name);
      PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_12, __pyx_v_tentacle_class_name);
      __Pyx_GIVEREF(__pyx_t_13);
      PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_12, __pyx_t_13);
      __Pyx_GIVEREF(__pyx_t_10);
      PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_12, __pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_11, 3+__pyx_t_12, __pyx_t_9);
      __pyx_t_13 = 0;
      __pyx_t_10 = 0;
      __pyx_t_9 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_14 = (__pyx_t_1 == Py_None);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = (__pyx_t_14 != 0);
    __pyx_t_7 = __pyx_t_6;
    __pyx_L6_bool_binop_done:;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":185
 *             if tentacle_class_name in activated_evaluators:
 *                 to_update_config[tentacle_class_name] = True
 *             elif get_class_from_string(tentacle_class_name, StrategyEvaluator, Strategies,             # <<<<<<<<<<<<<<
 *                                        evaluator_parent_inspection) is None and \
 *                 get_class_from_string(tentacle_class_name, AbstractTradingMode, Mode,
 */
    if (__pyx_t_7) {

      /* "octobot/strategy_optimizer/strategy_optimizer.py":189
 *                 get_class_from_string(tentacle_class_name, AbstractTradingMode, Mode,
 *                                       trading_mode_parent_inspection) is None:
 *                 to_update_config[tentacle_class_name] = False             # <<<<<<<<<<<<<<
 *         update_activation_configuration(self.tentacles_setup_config, to_update_config, False)
 * 
 */
      if (unlikely(PyDict_SetItem(__pyx_v_to_update_config, __pyx_v_tentacle_class_name, Py_False) < 0)) __PYX_ERR(0, 189, __pyx_L1_error)

      /* "octobot/strategy_optimizer/strategy_optimizer.py":185
 *             if tentacle_class_name in activated_evaluators:
 *                 to_update_config[tentacle_class_name] = True
 *             elif get_class_from_string(tentacle_class_name, StrategyEvaluator, Strategies,             # <<<<<<<<<<<<<<
 *                                        evaluator_parent_inspection) is None and \
 *                 get_class_from_string(tentacle_class_name, AbstractTradingMode, Mode,
 */
    }
    __pyx_L5:;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":182
 *     def _adapt_tentacles_config(self, activated_evaluators):
 *         to_update_config = {}
 *         for tentacle_class_name in get_tentacles_activation(self.tentacles_setup_config):             # <<<<<<<<<<<<<<
 *             if tentacle_class_name in activated_evaluators:
 *                 to_update_config[tentacle_class_name] = True
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":190
 *                                       trading_mode_parent_inspection) is None:
 *                 to_update_config[tentacle_class_name] = False
 *         update_activation_configuration(self.tentacles_setup_config, to_update_config, False)             # <<<<<<<<<<<<<<
 * 
 *     def _find_optimal_configuration_using_results(self):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_update_activation_configuration); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_12 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_self->tentacles_setup_config, __pyx_v_to_update_config, Py_False};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_12, 3+__pyx_t_12); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_self->tentacles_setup_config, __pyx_v_to_update_config, Py_False};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_12, 3+__pyx_t_12); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  {
    __pyx_t_11 = PyTuple_New(3+__pyx_t_12); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_self->tentacles_setup_config);
    __Pyx_GIVEREF(__pyx_v_self->tentacles_setup_config);
    PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_12, __pyx_v_self->tentacles_setup_config);
    __Pyx_INCREF(__pyx_v_to_update_config);
    __Pyx_GIVEREF(__pyx_v_to_update_config);
    PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_12, __pyx_v_to_update_config);
    __Pyx_INCREF(Py_False);
    __Pyx_GIVEREF(Py_False);
    PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_12, Py_False);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":180
 *         self.run_results.append(run_result)
 * 
 *     def _adapt_tentacles_config(self, activated_evaluators):             # <<<<<<<<<<<<<<
 *         to_update_config = {}
 *         for tentacle_class_name in get_tentacles_activation(self.tentacles_setup_config):
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_WriteUnraisable("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer._adapt_tentacles_config", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_to_update_config);
  __Pyx_XDECREF(__pyx_v_tentacle_class_name);
  __Pyx_RefNannyFinishContext();
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":210
 * 
 *         # inner function to replace uncythonizable lambda expression
 *         def _sorting_key(elem):             # <<<<<<<<<<<<<<
 *             return elem[RANK]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_41_find_optimal_configuration_using_results_1_sorting_key(PyObject *__pyx_self, PyObject *__pyx_v_elem); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_41_find_optimal_configuration_using_results_1_sorting_key = {"_sorting_key", (PyCFunction)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_41_find_optimal_configuration_using_results_1_sorting_key, METH_O, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_41_find_optimal_configuration_using_results_1_sorting_key(PyObject *__pyx_self, PyObject *__pyx_v_elem) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_sorting_key (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_41_find_optimal_configuration_using_results__sorting_key(__pyx_self, ((PyObject *)__pyx_v_elem));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_41_find_optimal_configuration_using_results__sorting_key(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_elem) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("_sorting_key", 0);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":211
 *         # inner function to replace uncythonizable lambda expression
 *         def _sorting_key(elem):
 *             return elem[RANK]             # <<<<<<<<<<<<<<
 * 
 *         self.sorted_results_through_all_time_frame = sorted(result_list, key=_sorting_key)
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_RANK); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_elem, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":210
 * 
 *         # inner function to replace uncythonizable lambda expression
 *         def _sorting_key(elem):             # <<<<<<<<<<<<<<
 *             return elem[RANK]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer._find_optimal_configuration_using_results._sorting_key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":192
 *         update_activation_configuration(self.tentacles_setup_config, to_update_config, False)
 * 
 *     def _find_optimal_configuration_using_results(self):             # <<<<<<<<<<<<<<
 *         for time_frame in self.all_time_frames:
 *             time_frame_sorted_results = self._get_sorted_results(self.run_results, time_frame)
 */

static void __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer__find_optimal_configuration_using_results(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_v_time_frame = NULL;
  PyObject *__pyx_v_time_frame_sorted_results = NULL;
  PyObject *__pyx_v_results_through_all_time_frame = NULL;
  PyObject *__pyx_v_results = NULL;
  PyObject *__pyx_v_rank = NULL;
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_v_result_summary = NULL;
  PyObject *__pyx_v_result_list = NULL;
  PyObject *_sorting_key = 0;
  PyObject *__pyx_8genexpr1__pyx_v_result = NULL;
  PyObject *__pyx_8genexpr1__pyx_v_trades_and_rank = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *(*__pyx_t_11)(PyObject *);
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  int __pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  __Pyx_RefNannySetupContext("_find_optimal_configuration_using_results", 0);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":193
 * 
 *     def _find_optimal_configuration_using_results(self):
 *         for time_frame in self.all_time_frames:             # <<<<<<<<<<<<<<
 *             time_frame_sorted_results = self._get_sorted_results(self.run_results, time_frame)
 *             self.sorted_results_by_time_frame[time_frame.value] = time_frame_sorted_results
 */
  if (unlikely(__pyx_v_self->all_time_frames == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 193, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->all_time_frames; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 193, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_time_frame, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":194
 *     def _find_optimal_configuration_using_results(self):
 *         for time_frame in self.all_time_frames:
 *             time_frame_sorted_results = self._get_sorted_results(self.run_results, time_frame)             # <<<<<<<<<<<<<<
 *             self.sorted_results_by_time_frame[time_frame.value] = time_frame_sorted_results
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_sorted_results); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_self->run_results, __pyx_v_time_frame};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 194, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_self->run_results, __pyx_v_time_frame};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 194, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 194, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_self->run_results);
      __Pyx_GIVEREF(__pyx_v_self->run_results);
      PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_self->run_results);
      __Pyx_INCREF(__pyx_v_time_frame);
      __Pyx_GIVEREF(__pyx_v_time_frame);
      PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_v_time_frame);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 194, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_time_frame_sorted_results, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":195
 *         for time_frame in self.all_time_frames:
 *             time_frame_sorted_results = self._get_sorted_results(self.run_results, time_frame)
 *             self.sorted_results_by_time_frame[time_frame.value] = time_frame_sorted_results             # <<<<<<<<<<<<<<
 * 
 *         results_through_all_time_frame = {}
 */
    if (unlikely(__pyx_v_self->sorted_results_by_time_frame == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 195, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_time_frame, __pyx_n_s_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(PyDict_SetItem(__pyx_v_self->sorted_results_by_time_frame, __pyx_t_3, __pyx_v_time_frame_sorted_results) < 0)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":193
 * 
 *     def _find_optimal_configuration_using_results(self):
 *         for time_frame in self.all_time_frames:             # <<<<<<<<<<<<<<
 *             time_frame_sorted_results = self._get_sorted_results(self.run_results, time_frame)
 *             self.sorted_results_by_time_frame[time_frame.value] = time_frame_sorted_results
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":197
 *             self.sorted_results_by_time_frame[time_frame.value] = time_frame_sorted_results
 * 
 *         results_through_all_time_frame = {}             # <<<<<<<<<<<<<<
 *         for results in self.sorted_results_by_time_frame.values():
 *             for rank, result in enumerate(results):
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_results_through_all_time_frame = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":198
 * 
 *         results_through_all_time_frame = {}
 *         for results in self.sorted_results_by_time_frame.values():             # <<<<<<<<<<<<<<
 *             for rank, result in enumerate(results):
 *                 result_summary = result.get_config_summary()
 */
  __pyx_t_2 = 0;
  if (unlikely(__pyx_v_self->sorted_results_by_time_frame == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
    __PYX_ERR(0, 198, __pyx_L1_error)
  }
  __pyx_t_3 = __Pyx_dict_iterator(__pyx_v_self->sorted_results_by_time_frame, 1, __pyx_n_s_values, (&__pyx_t_8), (&__pyx_t_6)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_1);
  __pyx_t_1 = __pyx_t_3;
  __pyx_t_3 = 0;
  while (1) {
    __pyx_t_9 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_8, &__pyx_t_2, NULL, &__pyx_t_3, NULL, __pyx_t_6);
    if (unlikely(__pyx_t_9 == 0)) break;
    if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_results, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":199
 *         results_through_all_time_frame = {}
 *         for results in self.sorted_results_by_time_frame.values():
 *             for rank, result in enumerate(results):             # <<<<<<<<<<<<<<
 *                 result_summary = result.get_config_summary()
 *                 if result_summary not in results_through_all_time_frame:
 */
    __Pyx_INCREF(__pyx_int_0);
    __pyx_t_3 = __pyx_int_0;
    if (likely(PyList_CheckExact(__pyx_v_results)) || PyTuple_CheckExact(__pyx_v_results)) {
      __pyx_t_4 = __pyx_v_results; __Pyx_INCREF(__pyx_t_4); __pyx_t_10 = 0;
      __pyx_t_11 = NULL;
    } else {
      __pyx_t_10 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_v_results); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 199, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_11 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 199, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_11)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          if (__pyx_t_10 >= PyList_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_10); __Pyx_INCREF(__pyx_t_7); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 199, __pyx_L1_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 199, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        } else {
          if (__pyx_t_10 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_10); __Pyx_INCREF(__pyx_t_7); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 199, __pyx_L1_error)
          #else
          __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 199, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
        }
      } else {
        __pyx_t_7 = __pyx_t_11(__pyx_t_4);
        if (unlikely(!__pyx_t_7)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 199, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_XDECREF_SET(__pyx_v_result, __pyx_t_7);
      __pyx_t_7 = 0;
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_XDECREF_SET(__pyx_v_rank, __pyx_t_3);
      __pyx_t_7 = __Pyx_PyInt_AddObjC(__pyx_t_3, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 199, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_3);
      __pyx_t_3 = __pyx_t_7;
      __pyx_t_7 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":200
 *         for results in self.sorted_results_by_time_frame.values():
 *             for rank, result in enumerate(results):
 *                 result_summary = result.get_config_summary()             # <<<<<<<<<<<<<<
 *                 if result_summary not in results_through_all_time_frame:
 *                     results_through_all_time_frame[result_summary] = [[], 0]
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_result, __pyx_n_s_get_config_summary); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 200, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_12 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_12)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_12);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_7 = (__pyx_t_12) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_12) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 200, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF_SET(__pyx_v_result_summary, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":201
 *             for rank, result in enumerate(results):
 *                 result_summary = result.get_config_summary()
 *                 if result_summary not in results_through_all_time_frame:             # <<<<<<<<<<<<<<
 *                     results_through_all_time_frame[result_summary] = [[], 0]
 *                 results_through_all_time_frame[result_summary][RANK] += rank
 */
      __pyx_t_13 = (__Pyx_PyDict_ContainsTF(__pyx_v_result_summary, __pyx_v_results_through_all_time_frame, Py_NE)); if (unlikely(__pyx_t_13 < 0)) __PYX_ERR(0, 201, __pyx_L1_error)
      __pyx_t_14 = (__pyx_t_13 != 0);
      if (__pyx_t_14) {

        /* "octobot/strategy_optimizer/strategy_optimizer.py":202
 *                 result_summary = result.get_config_summary()
 *                 if result_summary not in results_through_all_time_frame:
 *                     results_through_all_time_frame[result_summary] = [[], 0]             # <<<<<<<<<<<<<<
 *                 results_through_all_time_frame[result_summary][RANK] += rank
 *                 results_through_all_time_frame[result_summary][TRADES] += result.trades_counts
 */
        __pyx_t_7 = PyList_New(0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 202, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_5 = PyList_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 202, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_7);
        PyList_SET_ITEM(__pyx_t_5, 0, __pyx_t_7);
        __Pyx_INCREF(__pyx_int_0);
        __Pyx_GIVEREF(__pyx_int_0);
        PyList_SET_ITEM(__pyx_t_5, 1, __pyx_int_0);
        __pyx_t_7 = 0;
        if (unlikely(PyDict_SetItem(__pyx_v_results_through_all_time_frame, __pyx_v_result_summary, __pyx_t_5) < 0)) __PYX_ERR(0, 202, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "octobot/strategy_optimizer/strategy_optimizer.py":201
 *             for rank, result in enumerate(results):
 *                 result_summary = result.get_config_summary()
 *                 if result_summary not in results_through_all_time_frame:             # <<<<<<<<<<<<<<
 *                     results_through_all_time_frame[result_summary] = [[], 0]
 *                 results_through_all_time_frame[result_summary][RANK] += rank
 */
      }

      /* "octobot/strategy_optimizer/strategy_optimizer.py":203
 *                 if result_summary not in results_through_all_time_frame:
 *                     results_through_all_time_frame[result_summary] = [[], 0]
 *                 results_through_all_time_frame[result_summary][RANK] += rank             # <<<<<<<<<<<<<<
 *                 results_through_all_time_frame[result_summary][TRADES] += result.trades_counts
 * 
 */
      __pyx_t_5 = __Pyx_PyDict_GetItem(__pyx_v_results_through_all_time_frame, __pyx_v_result_summary); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 203, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_RANK); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 203, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_12 = __Pyx_PyObject_GetItem(__pyx_t_5, __pyx_t_7); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 203, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_15 = PyNumber_InPlaceAdd(__pyx_t_12, __pyx_v_rank); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 203, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      if (unlikely(PyObject_SetItem(__pyx_t_5, __pyx_t_7, __pyx_t_15) < 0)) __PYX_ERR(0, 203, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":204
 *                     results_through_all_time_frame[result_summary] = [[], 0]
 *                 results_through_all_time_frame[result_summary][RANK] += rank
 *                 results_through_all_time_frame[result_summary][TRADES] += result.trades_counts             # <<<<<<<<<<<<<<
 * 
 *         result_list = [(result, trades_and_rank[RANK], mean(trades_and_rank[TRADES]))
 */
      __pyx_t_5 = __Pyx_PyDict_GetItem(__pyx_v_results_through_all_time_frame, __pyx_v_result_summary); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 204, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_TRADES); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 204, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_15 = __Pyx_PyObject_GetItem(__pyx_t_5, __pyx_t_7); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 204, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_result, __pyx_n_s_trades_counts); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 204, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_16 = PyNumber_InPlaceAdd(__pyx_t_15, __pyx_t_12); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 204, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      if (unlikely(PyObject_SetItem(__pyx_t_5, __pyx_t_7, __pyx_t_16) < 0)) __PYX_ERR(0, 204, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":199
 *         results_through_all_time_frame = {}
 *         for results in self.sorted_results_by_time_frame.values():
 *             for rank, result in enumerate(results):             # <<<<<<<<<<<<<<
 *                 result_summary = result.get_config_summary()
 *                 if result_summary not in results_through_all_time_frame:
 */
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":206
 *                 results_through_all_time_frame[result_summary][TRADES] += result.trades_counts
 * 
 *         result_list = [(result, trades_and_rank[RANK], mean(trades_and_rank[TRADES]))             # <<<<<<<<<<<<<<
 *                        for result, trades_and_rank in results_through_all_time_frame.items()]
 * 
 */
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 206, __pyx_L12_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "octobot/strategy_optimizer/strategy_optimizer.py":207
 * 
 *         result_list = [(result, trades_and_rank[RANK], mean(trades_and_rank[TRADES]))
 *                        for result, trades_and_rank in results_through_all_time_frame.items()]             # <<<<<<<<<<<<<<
 * 
 *         # inner function to replace uncythonizable lambda expression
 */
    __pyx_t_8 = 0;
    __pyx_t_4 = __Pyx_dict_iterator(__pyx_v_results_through_all_time_frame, 1, __pyx_n_s_items, (&__pyx_t_2), (&__pyx_t_6)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 207, __pyx_L12_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3);
    __pyx_t_3 = __pyx_t_4;
    __pyx_t_4 = 0;
    while (1) {
      __pyx_t_9 = __Pyx_dict_iter_next(__pyx_t_3, __pyx_t_2, &__pyx_t_8, &__pyx_t_4, &__pyx_t_5, NULL, __pyx_t_6);
      if (unlikely(__pyx_t_9 == 0)) break;
      if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 207, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_XDECREF_SET(__pyx_8genexpr1__pyx_v_result, __pyx_t_4);
      __pyx_t_4 = 0;
      __Pyx_XDECREF_SET(__pyx_8genexpr1__pyx_v_trades_and_rank, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":206
 *                 results_through_all_time_frame[result_summary][TRADES] += result.trades_counts
 * 
 *         result_list = [(result, trades_and_rank[RANK], mean(trades_and_rank[TRADES]))             # <<<<<<<<<<<<<<
 *                        for result, trades_and_rank in results_through_all_time_frame.items()]
 * 
 */
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_RANK); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 206, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_8genexpr1__pyx_v_trades_and_rank, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 206, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_mean); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 206, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_n_s_TRADES); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 206, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_16);
      __pyx_t_12 = __Pyx_PyObject_GetItem(__pyx_8genexpr1__pyx_v_trades_and_rank, __pyx_t_16); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 206, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      __pyx_t_16 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_16)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_16);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_5 = (__pyx_t_16) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_16, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_12);
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 206, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 206, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_INCREF(__pyx_8genexpr1__pyx_v_result);
      __Pyx_GIVEREF(__pyx_8genexpr1__pyx_v_result);
      PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_8genexpr1__pyx_v_result);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_t_5);
      __pyx_t_4 = 0;
      __pyx_t_5 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 206, __pyx_L12_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_result); __pyx_8genexpr1__pyx_v_result = 0;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_trades_and_rank); __pyx_8genexpr1__pyx_v_trades_and_rank = 0;
    goto __pyx_L15_exit_scope;
    __pyx_L12_error:;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_result); __pyx_8genexpr1__pyx_v_result = 0;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_trades_and_rank); __pyx_8genexpr1__pyx_v_trades_and_rank = 0;
    goto __pyx_L1_error;
    __pyx_L15_exit_scope:;
  } /* exit inner scope */
  __pyx_v_result_list = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":210
 * 
 *         # inner function to replace uncythonizable lambda expression
 *         def _sorting_key(elem):             # <<<<<<<<<<<<<<
 *             return elem[RANK]
 * 
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_41_find_optimal_configuration_using_results_1_sorting_key, 0, __pyx_n_s_StrategyOptimizer__find_optimal, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__3)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  _sorting_key = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":213
 *             return elem[RANK]
 * 
 *         self.sorted_results_through_all_time_frame = sorted(result_list, key=_sorting_key)             # <<<<<<<<<<<<<<
 * 
 *     def print_report(self):
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_result_list);
  __Pyx_GIVEREF(__pyx_v_result_list);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_result_list);
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_key, _sorting_key) < 0) __PYX_ERR(0, 213, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_builtin_sorted, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(PyDict_CheckExact(__pyx_t_7))||((__pyx_t_7) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_7)->tp_name), 0))) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_7);
  __Pyx_GOTREF(__pyx_v_self->sorted_results_through_all_time_frame);
  __Pyx_DECREF(__pyx_v_self->sorted_results_through_all_time_frame);
  __pyx_v_self->sorted_results_through_all_time_frame = ((PyObject*)__pyx_t_7);
  __pyx_t_7 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":192
 *         update_activation_configuration(self.tentacles_setup_config, to_update_config, False)
 * 
 *     def _find_optimal_configuration_using_results(self):             # <<<<<<<<<<<<<<
 *         for time_frame in self.all_time_frames:
 *             time_frame_sorted_results = self._get_sorted_results(self.run_results, time_frame)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_WriteUnraisable("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer._find_optimal_configuration_using_results", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_time_frame);
  __Pyx_XDECREF(__pyx_v_time_frame_sorted_results);
  __Pyx_XDECREF(__pyx_v_results_through_all_time_frame);
  __Pyx_XDECREF(__pyx_v_results);
  __Pyx_XDECREF(__pyx_v_rank);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_result_summary);
  __Pyx_XDECREF(__pyx_v_result_list);
  __Pyx_XDECREF(_sorting_key);
  __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_result);
  __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_trades_and_rank);
  __Pyx_RefNannyFinishContext();
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":215
 *         self.sorted_results_through_all_time_frame = sorted(result_list, key=_sorting_key)
 * 
 *     def print_report(self):             # <<<<<<<<<<<<<<
 *         self.logger.info("Full execution sorted results: Minimum time frames are defining the range of the run "
 *                          "since it finishes at the end of the first data, aka minimum time frame. Therefore all "
 */

static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5print_report(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static void __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_print_report(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_time_frame = NULL;
  PyObject *__pyx_v_results = NULL;
  PyObject *__pyx_v_rank = NULL;
  PyObject *__pyx_v_result = NULL;
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_v_best_result = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_UCS4 __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *(*__pyx_t_12)(PyObject *);
  PyObject *__pyx_t_13 = NULL;
  Py_ssize_t __pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  long __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  __Pyx_RefNannySetupContext("print_report", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_print_report); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 215, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5print_report)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 215, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":216
 * 
 *     def print_report(self):
 *         self.logger.info("Full execution sorted results: Minimum time frames are defining the range of the run "             # <<<<<<<<<<<<<<
 *                          "since it finishes at the end of the first data, aka minimum time frame. Therefore all "
 *                          "different time frames are different price actions and can't be compared independently.")
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_info); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_u_Full_execution_sorted_results_Mi) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_Full_execution_sorted_results_Mi);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":219
 *                          "since it finishes at the end of the first data, aka minimum time frame. Therefore all "
 *                          "different time frames are different price actions and can't be compared independently.")
 *         for time_frame, results in self.sorted_results_by_time_frame.items():             # <<<<<<<<<<<<<<
 *             self.logger.info(f" *** {time_frame} minimum time frame ranking*** ")
 *             for rank, result in enumerate(results):
 */
  __pyx_t_5 = 0;
  if (unlikely(__pyx_v_self->sorted_results_by_time_frame == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
    __PYX_ERR(0, 219, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_dict_iterator(__pyx_v_self->sorted_results_by_time_frame, 1, __pyx_n_s_items, (&__pyx_t_6), (&__pyx_t_7)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_1);
  __pyx_t_1 = __pyx_t_2;
  __pyx_t_2 = 0;
  while (1) {
    __pyx_t_8 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_6, &__pyx_t_5, &__pyx_t_2, &__pyx_t_3, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_8 == 0)) break;
    if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 219, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_time_frame, __pyx_t_2);
    __pyx_t_2 = 0;
    __Pyx_XDECREF_SET(__pyx_v_results, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":220
 *                          "different time frames are different price actions and can't be compared independently.")
 *         for time_frame, results in self.sorted_results_by_time_frame.items():
 *             self.logger.info(f" *** {time_frame} minimum time frame ranking*** ")             # <<<<<<<<<<<<<<
 *             for rank, result in enumerate(results):
 *                 self.logger.info(f"{rank}: {result.get_result_string()}")
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_info); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 220, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 220, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_9 = 0;
    __pyx_t_10 = 127;
    __Pyx_INCREF(__pyx_kp_u__4);
    __pyx_t_9 += 5;
    __Pyx_GIVEREF(__pyx_kp_u__4);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_kp_u__4);
    __pyx_t_11 = __Pyx_PyObject_FormatSimple(__pyx_v_time_frame, __pyx_empty_unicode); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 220, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_11) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_11) : __pyx_t_10;
    __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_11);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_11);
    __pyx_t_11 = 0;
    __Pyx_INCREF(__pyx_kp_u_minimum_time_frame_ranking);
    __pyx_t_9 += 31;
    __Pyx_GIVEREF(__pyx_kp_u_minimum_time_frame_ranking);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_kp_u_minimum_time_frame_ranking);
    __pyx_t_11 = __Pyx_PyUnicode_Join(__pyx_t_4, 3, __pyx_t_9, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 220, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_11) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_11);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 220, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":221
 *         for time_frame, results in self.sorted_results_by_time_frame.items():
 *             self.logger.info(f" *** {time_frame} minimum time frame ranking*** ")
 *             for rank, result in enumerate(results):             # <<<<<<<<<<<<<<
 *                 self.logger.info(f"{rank}: {result.get_result_string()}")
 *         self.logger.info(f" *** Top rankings per time frame *** ")
 */
    __Pyx_INCREF(__pyx_int_0);
    __pyx_t_3 = __pyx_int_0;
    if (likely(PyList_CheckExact(__pyx_v_results)) || PyTuple_CheckExact(__pyx_v_results)) {
      __pyx_t_2 = __pyx_v_results; __Pyx_INCREF(__pyx_t_2); __pyx_t_9 = 0;
      __pyx_t_12 = NULL;
    } else {
      __pyx_t_9 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_results); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 221, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_12 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 221, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_12)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_11 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_9); __Pyx_INCREF(__pyx_t_11); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 221, __pyx_L1_error)
          #else
          __pyx_t_11 = PySequence_ITEM(__pyx_t_2, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 221, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          #endif
        } else {
          if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_11 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_9); __Pyx_INCREF(__pyx_t_11); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 221, __pyx_L1_error)
          #else
          __pyx_t_11 = PySequence_ITEM(__pyx_t_2, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 221, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          #endif
        }
      } else {
        __pyx_t_11 = __pyx_t_12(__pyx_t_2);
        if (unlikely(!__pyx_t_11)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 221, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_11);
      }
      __Pyx_XDECREF_SET(__pyx_v_result, __pyx_t_11);
      __pyx_t_11 = 0;
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_XDECREF_SET(__pyx_v_rank, __pyx_t_3);
      __pyx_t_11 = __Pyx_PyInt_AddObjC(__pyx_t_3, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 221, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_3);
      __pyx_t_3 = __pyx_t_11;
      __pyx_t_11 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":222
 *             self.logger.info(f" *** {time_frame} minimum time frame ranking*** ")
 *             for rank, result in enumerate(results):
 *                 self.logger.info(f"{rank}: {result.get_result_string()}")             # <<<<<<<<<<<<<<
 *         self.logger.info(f" *** Top rankings per time frame *** ")
 *         for time_frame, results in self.sorted_results_by_time_frame.items():
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_info); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 222, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_13 = PyTuple_New(3); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 222, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_14 = 0;
      __pyx_t_10 = 127;
      __pyx_t_15 = __Pyx_PyObject_FormatSimple(__pyx_v_rank, __pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 222, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15) : __pyx_t_10;
      __pyx_t_14 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_15);
      __Pyx_GIVEREF(__pyx_t_15);
      PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_15);
      __pyx_t_15 = 0;
      __Pyx_INCREF(__pyx_kp_u__5);
      __pyx_t_14 += 2;
      __Pyx_GIVEREF(__pyx_kp_u__5);
      PyTuple_SET_ITEM(__pyx_t_13, 1, __pyx_kp_u__5);
      __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_v_result, __pyx_n_s_get_result_string); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 222, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      __pyx_t_17 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_16))) {
        __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_16);
        if (likely(__pyx_t_17)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_16);
          __Pyx_INCREF(__pyx_t_17);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_16, function);
        }
      }
      __pyx_t_15 = (__pyx_t_17) ? __Pyx_PyObject_CallOneArg(__pyx_t_16, __pyx_t_17) : __Pyx_PyObject_CallNoArg(__pyx_t_16);
      __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
      if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 222, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      __pyx_t_16 = __Pyx_PyObject_FormatSimple(__pyx_t_15, __pyx_empty_unicode); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 222, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_16) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_16) : __pyx_t_10;
      __pyx_t_14 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_16);
      __Pyx_GIVEREF(__pyx_t_16);
      PyTuple_SET_ITEM(__pyx_t_13, 2, __pyx_t_16);
      __pyx_t_16 = 0;
      __pyx_t_16 = __Pyx_PyUnicode_Join(__pyx_t_13, 3, __pyx_t_14, __pyx_t_10); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 222, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_13 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_11 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_13, __pyx_t_16) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_16);
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 222, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":221
 *         for time_frame, results in self.sorted_results_by_time_frame.items():
 *             self.logger.info(f" *** {time_frame} minimum time frame ranking*** ")
 *             for rank, result in enumerate(results):             # <<<<<<<<<<<<<<
 *                 self.logger.info(f"{rank}: {result.get_result_string()}")
 *         self.logger.info(f" *** Top rankings per time frame *** ")
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":223
 *             for rank, result in enumerate(results):
 *                 self.logger.info(f"{rank}: {result.get_result_string()}")
 *         self.logger.info(f" *** Top rankings per time frame *** ")             # <<<<<<<<<<<<<<
 *         for time_frame, results in self.sorted_results_by_time_frame.items():
 *             for i in range(0, min(len(results), 5)):
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_info); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_kp_u_Top_rankings_per_time_frame) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_Top_rankings_per_time_frame);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":224
 *                 self.logger.info(f"{rank}: {result.get_result_string()}")
 *         self.logger.info(f" *** Top rankings per time frame *** ")
 *         for time_frame, results in self.sorted_results_by_time_frame.items():             # <<<<<<<<<<<<<<
 *             for i in range(0, min(len(results), 5)):
 *                 self.logger.info(f"{time_frame}: {results[i].get_result_string(False)}")
 */
  __pyx_t_6 = 0;
  if (unlikely(__pyx_v_self->sorted_results_by_time_frame == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
    __PYX_ERR(0, 224, __pyx_L1_error)
  }
  __pyx_t_3 = __Pyx_dict_iterator(__pyx_v_self->sorted_results_by_time_frame, 1, __pyx_n_s_items, (&__pyx_t_5), (&__pyx_t_7)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_1);
  __pyx_t_1 = __pyx_t_3;
  __pyx_t_3 = 0;
  while (1) {
    __pyx_t_8 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_5, &__pyx_t_6, &__pyx_t_3, &__pyx_t_2, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_8 == 0)) break;
    if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 224, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_time_frame, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_results, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":225
 *         self.logger.info(f" *** Top rankings per time frame *** ")
 *         for time_frame, results in self.sorted_results_by_time_frame.items():
 *             for i in range(0, min(len(results), 5)):             # <<<<<<<<<<<<<<
 *                 self.logger.info(f"{time_frame}: {results[i].get_result_string(False)}")
 *         self.logger.info(f" *** Top rankings through all time frames *** ")
 */
    __pyx_t_18 = 5;
    __pyx_t_9 = PyObject_Length(__pyx_v_results); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 225, __pyx_L1_error)
    if (((__pyx_t_18 < __pyx_t_9) != 0)) {
      __pyx_t_14 = __pyx_t_18;
    } else {
      __pyx_t_14 = __pyx_t_9;
    }
    __pyx_t_9 = __pyx_t_14;
    __pyx_t_14 = __pyx_t_9;
    for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_14; __pyx_t_19+=1) {
      __pyx_v_i = __pyx_t_19;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":226
 *         for time_frame, results in self.sorted_results_by_time_frame.items():
 *             for i in range(0, min(len(results), 5)):
 *                 self.logger.info(f"{time_frame}: {results[i].get_result_string(False)}")             # <<<<<<<<<<<<<<
 *         self.logger.info(f" *** Top rankings through all time frames *** ")
 *         for rank, result in enumerate(self.sorted_results_through_all_time_frame[0:25]):
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_info); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 226, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_11 = PyTuple_New(3); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 226, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_20 = 0;
      __pyx_t_10 = 127;
      __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_v_time_frame, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 226, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) : __pyx_t_10;
      __pyx_t_20 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4);
      __pyx_t_4 = 0;
      __Pyx_INCREF(__pyx_kp_u__5);
      __pyx_t_20 += 2;
      __Pyx_GIVEREF(__pyx_kp_u__5);
      PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_kp_u__5);
      __pyx_t_16 = __Pyx_GetItemInt(__pyx_v_results, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 226, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_16, __pyx_n_s_get_result_string); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 226, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      __pyx_t_16 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {
        __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_13);
        if (likely(__pyx_t_16)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
          __Pyx_INCREF(__pyx_t_16);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_13, function);
        }
      }
      __pyx_t_4 = (__pyx_t_16) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_16, Py_False) : __Pyx_PyObject_CallOneArg(__pyx_t_13, Py_False);
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 226, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_13 = __Pyx_PyObject_FormatSimple(__pyx_t_4, __pyx_empty_unicode); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 226, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13) : __pyx_t_10;
      __pyx_t_20 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_13);
      __Pyx_GIVEREF(__pyx_t_13);
      PyTuple_SET_ITEM(__pyx_t_11, 2, __pyx_t_13);
      __pyx_t_13 = 0;
      __pyx_t_13 = __Pyx_PyUnicode_Join(__pyx_t_11, 3, __pyx_t_20, __pyx_t_10); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 226, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_11 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_11)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_11);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_2 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_11, __pyx_t_13) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_13);
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 226, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":227
 *             for i in range(0, min(len(results), 5)):
 *                 self.logger.info(f"{time_frame}: {results[i].get_result_string(False)}")
 *         self.logger.info(f" *** Top rankings through all time frames *** ")             # <<<<<<<<<<<<<<
 *         for rank, result in enumerate(self.sorted_results_through_all_time_frame[0:25]):
 *             self.logger.info(f"{rank}: {result[CONFIG].get_result_string()} (time frame rank sum: {result[RANK]}) "
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_info); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_u_Top_rankings_through_all_time_f) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_Top_rankings_through_all_time_f);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":228
 *                 self.logger.info(f"{time_frame}: {results[i].get_result_string(False)}")
 *         self.logger.info(f" *** Top rankings through all time frames *** ")
 *         for rank, result in enumerate(self.sorted_results_through_all_time_frame[0:25]):             # <<<<<<<<<<<<<<
 *             self.logger.info(f"{rank}: {result[CONFIG].get_result_string()} (time frame rank sum: {result[RANK]}) "
 *                              f"average trades count: {result[TRADES_IN_RESULT]:f}")
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_t_1 = __pyx_int_0;
  __pyx_t_5 = 0;
  if (unlikely(__pyx_v_self->sorted_results_through_all_time_frame == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 228, __pyx_L1_error)
  }
  __pyx_t_3 = PySequence_GetSlice(__pyx_v_self->sorted_results_through_all_time_frame, 0, 25); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(__pyx_t_3 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 228, __pyx_L1_error)
  }
  __pyx_t_13 = __Pyx_dict_iterator(__pyx_t_3, 1, ((PyObject *)NULL), (&__pyx_t_6), (&__pyx_t_7)); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_XDECREF(__pyx_t_2);
  __pyx_t_2 = __pyx_t_13;
  __pyx_t_13 = 0;
  while (1) {
    __pyx_t_8 = __Pyx_dict_iter_next(__pyx_t_2, __pyx_t_6, &__pyx_t_5, &__pyx_t_13, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_8 == 0)) break;
    if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 228, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_XDECREF_SET(__pyx_v_result, __pyx_t_13);
    __pyx_t_13 = 0;
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_rank, __pyx_t_1);
    __pyx_t_13 = __Pyx_PyInt_AddObjC(__pyx_t_1, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 228, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_DECREF(__pyx_t_1);
    __pyx_t_1 = __pyx_t_13;
    __pyx_t_13 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":229
 *         self.logger.info(f" *** Top rankings through all time frames *** ")
 *         for rank, result in enumerate(self.sorted_results_through_all_time_frame[0:25]):
 *             self.logger.info(f"{rank}: {result[CONFIG].get_result_string()} (time frame rank sum: {result[RANK]}) "             # <<<<<<<<<<<<<<
 *                              f"average trades count: {result[TRADES_IN_RESULT]:f}")
 *         self.logger.info(f" *** Overall best configuration for {self.strategy_class.get_name()} using "
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_info); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_11 = PyTuple_New(7); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_9 = 0;
    __pyx_t_10 = 127;
    __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_v_rank, __pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) : __pyx_t_10;
    __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4);
    __pyx_t_4 = 0;
    __Pyx_INCREF(__pyx_kp_u__5);
    __pyx_t_9 += 2;
    __Pyx_GIVEREF(__pyx_kp_u__5);
    PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_kp_u__5);
    __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_n_s_CONFIG); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __pyx_t_15 = __Pyx_PyObject_GetItem(__pyx_v_result, __pyx_t_16); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_t_15, __pyx_n_s_get_result_string); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    __pyx_t_15 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_16))) {
      __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_16);
      if (likely(__pyx_t_15)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_15);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_16, function);
      }
    }
    __pyx_t_4 = (__pyx_t_15) ? __Pyx_PyObject_CallOneArg(__pyx_t_16, __pyx_t_15) : __Pyx_PyObject_CallNoArg(__pyx_t_16);
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    __pyx_t_16 = __Pyx_PyObject_FormatSimple(__pyx_t_4, __pyx_empty_unicode); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_16) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_16) : __pyx_t_10;
    __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_16);
    __Pyx_GIVEREF(__pyx_t_16);
    PyTuple_SET_ITEM(__pyx_t_11, 2, __pyx_t_16);
    __pyx_t_16 = 0;
    __Pyx_INCREF(__pyx_kp_u_time_frame_rank_sum);
    __pyx_t_9 += 23;
    __Pyx_GIVEREF(__pyx_kp_u_time_frame_rank_sum);
    PyTuple_SET_ITEM(__pyx_t_11, 3, __pyx_kp_u_time_frame_rank_sum);
    __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_n_s_RANK); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_v_result, __pyx_t_16); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    __pyx_t_16 = __Pyx_PyObject_FormatSimple(__pyx_t_4, __pyx_empty_unicode); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_16) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_16) : __pyx_t_10;
    __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_16);
    __Pyx_GIVEREF(__pyx_t_16);
    PyTuple_SET_ITEM(__pyx_t_11, 4, __pyx_t_16);
    __pyx_t_16 = 0;
    __Pyx_INCREF(__pyx_kp_u_average_trades_count);
    __pyx_t_9 += 24;
    __Pyx_GIVEREF(__pyx_kp_u_average_trades_count);
    PyTuple_SET_ITEM(__pyx_t_11, 5, __pyx_kp_u_average_trades_count);

    /* "octobot/strategy_optimizer/strategy_optimizer.py":230
 *         for rank, result in enumerate(self.sorted_results_through_all_time_frame[0:25]):
 *             self.logger.info(f"{rank}: {result[CONFIG].get_result_string()} (time frame rank sum: {result[RANK]}) "
 *                              f"average trades count: {result[TRADES_IN_RESULT]:f}")             # <<<<<<<<<<<<<<
 *         self.logger.info(f" *** Overall best configuration for {self.strategy_class.get_name()} using "
 *                          f"{self.trading_mode.get_name()} trading mode *** ")
 */
    __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_n_s_TRADES_IN_RESULT); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 230, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_v_result, __pyx_t_16); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 230, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    __pyx_t_16 = __Pyx_PyObject_Format(__pyx_t_4, __pyx_n_u_f); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 230, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_16) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_16) : __pyx_t_10;
    __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_16);
    __Pyx_GIVEREF(__pyx_t_16);
    PyTuple_SET_ITEM(__pyx_t_11, 6, __pyx_t_16);
    __pyx_t_16 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":229
 *         self.logger.info(f" *** Top rankings through all time frames *** ")
 *         for rank, result in enumerate(self.sorted_results_through_all_time_frame[0:25]):
 *             self.logger.info(f"{rank}: {result[CONFIG].get_result_string()} (time frame rank sum: {result[RANK]}) "             # <<<<<<<<<<<<<<
 *                              f"average trades count: {result[TRADES_IN_RESULT]:f}")
 *         self.logger.info(f" *** Overall best configuration for {self.strategy_class.get_name()} using "
 */
    __pyx_t_16 = __Pyx_PyUnicode_Join(__pyx_t_11, 7, __pyx_t_9, __pyx_t_10); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_11 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_13 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_11, __pyx_t_16) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_16);
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":231
 *             self.logger.info(f"{rank}: {result[CONFIG].get_result_string()} (time frame rank sum: {result[RANK]}) "
 *                              f"average trades count: {result[TRADES_IN_RESULT]:f}")
 *         self.logger.info(f" *** Overall best configuration for {self.strategy_class.get_name()} using "             # <<<<<<<<<<<<<<
 *                          f"{self.trading_mode.get_name()} trading mode *** ")
 *         best_result = self.sorted_results_through_all_time_frame[0]
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_info); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_13 = PyTuple_New(5); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_6 = 0;
  __pyx_t_10 = 127;
  __Pyx_INCREF(__pyx_kp_u_Overall_best_configuration_for);
  __pyx_t_6 += 36;
  __Pyx_GIVEREF(__pyx_kp_u_Overall_best_configuration_for);
  PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_kp_u_Overall_best_configuration_for);
  __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->strategy_class, __pyx_n_s_get_name); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __pyx_t_11 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_16))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_16);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_16);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_16, function);
    }
  }
  __pyx_t_3 = (__pyx_t_11) ? __Pyx_PyObject_CallOneArg(__pyx_t_16, __pyx_t_11) : __Pyx_PyObject_CallNoArg(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  __pyx_t_16 = __Pyx_PyObject_FormatSimple(__pyx_t_3, __pyx_empty_unicode); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_16) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_16) : __pyx_t_10;
  __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_16);
  __Pyx_GIVEREF(__pyx_t_16);
  PyTuple_SET_ITEM(__pyx_t_13, 1, __pyx_t_16);
  __pyx_t_16 = 0;
  __Pyx_INCREF(__pyx_kp_u_using);
  __pyx_t_6 += 7;
  __Pyx_GIVEREF(__pyx_kp_u_using);
  PyTuple_SET_ITEM(__pyx_t_13, 2, __pyx_kp_u_using);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":232
 *                              f"average trades count: {result[TRADES_IN_RESULT]:f}")
 *         self.logger.info(f" *** Overall best configuration for {self.strategy_class.get_name()} using "
 *                          f"{self.trading_mode.get_name()} trading mode *** ")             # <<<<<<<<<<<<<<
 *         best_result = self.sorted_results_through_all_time_frame[0]
 *         self.logger.info(f"{best_result[CONFIG].get_result_string()} "
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->trading_mode, __pyx_n_s_get_name); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_11 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_16 = (__pyx_t_11) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_11) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_t_16, __pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_10;
  __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_13, 3, __pyx_t_3);
  __pyx_t_3 = 0;
  __Pyx_INCREF(__pyx_kp_u_trading_mode_2);
  __pyx_t_6 += 18;
  __Pyx_GIVEREF(__pyx_kp_u_trading_mode_2);
  PyTuple_SET_ITEM(__pyx_t_13, 4, __pyx_kp_u_trading_mode_2);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":231
 *             self.logger.info(f"{rank}: {result[CONFIG].get_result_string()} (time frame rank sum: {result[RANK]}) "
 *                              f"average trades count: {result[TRADES_IN_RESULT]:f}")
 *         self.logger.info(f" *** Overall best configuration for {self.strategy_class.get_name()} using "             # <<<<<<<<<<<<<<
 *                          f"{self.trading_mode.get_name()} trading mode *** ")
 *         best_result = self.sorted_results_through_all_time_frame[0]
 */
  __pyx_t_3 = __Pyx_PyUnicode_Join(__pyx_t_13, 5, __pyx_t_6, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_13)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_13);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_13, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":233
 *         self.logger.info(f" *** Overall best configuration for {self.strategy_class.get_name()} using "
 *                          f"{self.trading_mode.get_name()} trading mode *** ")
 *         best_result = self.sorted_results_through_all_time_frame[0]             # <<<<<<<<<<<<<<
 *         self.logger.info(f"{best_result[CONFIG].get_result_string()} "
 *                          f"average trades count: {best_result[TRADES_IN_RESULT]:f}")
 */
  if (unlikely(__pyx_v_self->sorted_results_through_all_time_frame == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 233, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->sorted_results_through_all_time_frame, __pyx_int_0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_best_result = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":234
 *                          f"{self.trading_mode.get_name()} trading mode *** ")
 *         best_result = self.sorted_results_through_all_time_frame[0]
 *         self.logger.info(f"{best_result[CONFIG].get_result_string()} "             # <<<<<<<<<<<<<<
 *                          f"average trades count: {best_result[TRADES_IN_RESULT]:f}")
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->logger, __pyx_n_s_info); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = 0;
  __pyx_t_10 = 127;
  __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_n_s_CONFIG); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __pyx_t_11 = __Pyx_PyObject_GetItem(__pyx_v_best_result, __pyx_t_16); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_n_s_get_result_string); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_16))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_16);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_16);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_16, function);
    }
  }
  __pyx_t_13 = (__pyx_t_11) ? __Pyx_PyObject_CallOneArg(__pyx_t_16, __pyx_t_11) : __Pyx_PyObject_CallNoArg(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  __pyx_t_16 = __Pyx_PyObject_FormatSimple(__pyx_t_13, __pyx_empty_unicode); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_16) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_16) : __pyx_t_10;
  __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_16);
  __Pyx_GIVEREF(__pyx_t_16);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_16);
  __pyx_t_16 = 0;
  __Pyx_INCREF(__pyx_kp_u_average_trades_count_2);
  __pyx_t_6 += 23;
  __Pyx_GIVEREF(__pyx_kp_u_average_trades_count_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_kp_u_average_trades_count_2);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":235
 *         best_result = self.sorted_results_through_all_time_frame[0]
 *         self.logger.info(f"{best_result[CONFIG].get_result_string()} "
 *                          f"average trades count: {best_result[TRADES_IN_RESULT]:f}")             # <<<<<<<<<<<<<<
 * 
 *     def get_overall_progress(self):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_n_s_TRADES_IN_RESULT); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __pyx_t_13 = __Pyx_PyObject_GetItem(__pyx_v_best_result, __pyx_t_16); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  __pyx_t_16 = __Pyx_PyObject_Format(__pyx_t_13, __pyx_n_u_f); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_16) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_16) : __pyx_t_10;
  __pyx_t_6 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_16);
  __Pyx_GIVEREF(__pyx_t_16);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_16);
  __pyx_t_16 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":234
 *                          f"{self.trading_mode.get_name()} trading mode *** ")
 *         best_result = self.sorted_results_through_all_time_frame[0]
 *         self.logger.info(f"{best_result[CONFIG].get_result_string()} "             # <<<<<<<<<<<<<<
 *                          f"average trades count: {best_result[TRADES_IN_RESULT]:f}")
 * 
 */
  __pyx_t_16 = __Pyx_PyUnicode_Join(__pyx_t_3, 3, __pyx_t_6, __pyx_t_10); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_t_16) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_16);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":215
 *         self.sorted_results_through_all_time_frame = sorted(result_list, key=_sorting_key)
 * 
 *     def print_report(self):             # <<<<<<<<<<<<<<
 *         self.logger.info("Full execution sorted results: Minimum time frames are defining the range of the run "
 *                          "since it finishes at the end of the first data, aka minimum time frame. Therefore all "
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_WriteUnraisable("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.print_report", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_time_frame);
  __Pyx_XDECREF(__pyx_v_results);
  __Pyx_XDECREF(__pyx_v_rank);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_best_result);
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5print_report(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5print_report = {"print_report", (PyCFunction)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5print_report, METH_NOARGS, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5print_report(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("print_report (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_4print_report(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_4print_report(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("print_report", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_print_report(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.print_report", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":237
 *                          f"average trades count: {best_result[TRADES_IN_RESULT]:f}")
 * 
 *     def get_overall_progress(self):             # <<<<<<<<<<<<<<
 *         return int((self.run_id-1) / self.total_nb_runs * 100) if self.total_nb_runs else 0
 * 
 */

static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7get_overall_progress(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_overall_progress(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  long __pyx_t_6;
  __Pyx_RefNannySetupContext("get_overall_progress", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_overall_progress); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 237, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7get_overall_progress)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 237, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 237, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":238
 * 
 *     def get_overall_progress(self):
 *         return int((self.run_id-1) / self.total_nb_runs * 100) if self.total_nb_runs else 0             # <<<<<<<<<<<<<<
 * 
 *     def is_in_progress(self):
 */
  if ((__pyx_v_self->total_nb_runs != 0)) {
    __pyx_t_6 = (__pyx_v_self->run_id - 1);
    if (unlikely(__pyx_v_self->total_nb_runs == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      __PYX_ERR(0, 238, __pyx_L1_error)
    }
    __pyx_t_5 = ((int)((((double)__pyx_t_6) / ((double)__pyx_v_self->total_nb_runs)) * 100.0));
  } else {
    __pyx_t_5 = 0;
  }
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":237
 *                          f"average trades count: {best_result[TRADES_IN_RESULT]:f}")
 * 
 *     def get_overall_progress(self):             # <<<<<<<<<<<<<<
 *         return int((self.run_id-1) / self.total_nb_runs * 100) if self.total_nb_runs else 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.get_overall_progress", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7get_overall_progress(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7get_overall_progress = {"get_overall_progress", (PyCFunction)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7get_overall_progress, METH_NOARGS, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7get_overall_progress(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_overall_progress (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_6get_overall_progress(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_6get_overall_progress(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("get_overall_progress", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_overall_progress(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.get_overall_progress", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":240
 *         return int((self.run_id-1) / self.total_nb_runs * 100) if self.total_nb_runs else 0
 * 
 *     def is_in_progress(self):             # <<<<<<<<<<<<<<
 *         return self.get_overall_progress() != 100
 * 
 */

static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_9is_in_progress(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_is_in_progress(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("is_in_progress", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_in_progress); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 240, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_9is_in_progress)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 240, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 240, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":241
 * 
 *     def is_in_progress(self):
 *         return self.get_overall_progress() != 100             # <<<<<<<<<<<<<<
 * 
 *     def get_current_test_suite_progress(self):
 */
  __pyx_r = (((struct __pyx_vtabstruct_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self->__pyx_vtab)->get_overall_progress(__pyx_v_self, 0) != 0x64);
  goto __pyx_L0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":240
 *         return int((self.run_id-1) / self.total_nb_runs * 100) if self.total_nb_runs else 0
 * 
 *     def is_in_progress(self):             # <<<<<<<<<<<<<<
 *         return self.get_overall_progress() != 100
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.is_in_progress", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_9is_in_progress(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_9is_in_progress = {"is_in_progress", (PyCFunction)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_9is_in_progress, METH_NOARGS, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_9is_in_progress(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_in_progress (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_8is_in_progress(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_8is_in_progress(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("is_in_progress", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_is_in_progress(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.is_in_progress", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":243
 *         return self.get_overall_progress() != 100
 * 
 *     def get_current_test_suite_progress(self):             # <<<<<<<<<<<<<<
 *         return self.current_test_suite.current_progress if self.current_test_suite else 0
 * 
 */

static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11get_current_test_suite_progress(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_current_test_suite_progress(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("get_current_test_suite_progress", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_current_test_suite_progress); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 243, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11get_current_test_suite_progress)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 243, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 243, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":244
 * 
 *     def get_current_test_suite_progress(self):
 *         return self.current_test_suite.current_progress if self.current_test_suite else 0             # <<<<<<<<<<<<<<
 * 
 *     def get_report(self):
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_self->current_test_suite); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 244, __pyx_L1_error)
  if (__pyx_t_6) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->current_test_suite, __pyx_n_s_current_progress); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 244, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 244, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = __pyx_t_7;
  } else {
    __pyx_t_5 = 0;
  }
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":243
 *         return self.get_overall_progress() != 100
 * 
 *     def get_current_test_suite_progress(self):             # <<<<<<<<<<<<<<
 *         return self.current_test_suite.current_progress if self.current_test_suite else 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.get_current_test_suite_progress", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11get_current_test_suite_progress(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11get_current_test_suite_progress = {"get_current_test_suite_progress", (PyCFunction)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11get_current_test_suite_progress, METH_NOARGS, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11get_current_test_suite_progress(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_current_test_suite_progress (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_10get_current_test_suite_progress(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_10get_current_test_suite_progress(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("get_current_test_suite_progress", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_current_test_suite_progress(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.get_current_test_suite_progress", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":246
 *         return self.current_test_suite.current_progress if self.current_test_suite else 0
 * 
 *     def get_report(self):             # <<<<<<<<<<<<<<
 *         # index, evaluators, risk, score, trades
 *         if self.sorted_results_through_all_time_frame:
 */

static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_13get_report(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_report(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_results = NULL;
  PyObject *__pyx_8genexpr2__pyx_v_rank = NULL;
  PyObject *__pyx_8genexpr2__pyx_v_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  __Pyx_RefNannySetupContext("get_report", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_report); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 246, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_13get_report)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 246, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 246, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":248
 *     def get_report(self):
 *         # index, evaluators, risk, score, trades
 *         if self.sorted_results_through_all_time_frame:             # <<<<<<<<<<<<<<
 *             results = [TestSuiteResult.convert_result_into_dict(rank, result[CONFIG].evaluators, "",
 *                                                                 result[CONFIG].risk, result[RANK],
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_self->sorted_results_through_all_time_frame); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 248, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "octobot/strategy_optimizer/strategy_optimizer.py":249
 *         # index, evaluators, risk, score, trades
 *         if self.sorted_results_through_all_time_frame:
 *             results = [TestSuiteResult.convert_result_into_dict(rank, result[CONFIG].evaluators, "",             # <<<<<<<<<<<<<<
 *                                                                 result[CONFIG].risk, result[RANK],
 *                                                                 round(result[TRADES_IN_RESULT], 5))
 */
    { /* enter inner scope */
      __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_INCREF(__pyx_int_0);
      __pyx_t_2 = __pyx_int_0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":252
 *                                                                 result[CONFIG].risk, result[RANK],
 *                                                                 round(result[TRADES_IN_RESULT], 5))
 *                        for rank, result in enumerate(self.sorted_results_through_all_time_frame[0:100])]             # <<<<<<<<<<<<<<
 *         else:
 *             results = []
 */
      __pyx_t_6 = 0;
      if (unlikely(__pyx_v_self->sorted_results_through_all_time_frame == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 252, __pyx_L6_error)
      }
      __pyx_t_4 = PySequence_GetSlice(__pyx_v_self->sorted_results_through_all_time_frame, 0, 0x64); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 252, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(__pyx_t_4 == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
        __PYX_ERR(0, 252, __pyx_L6_error)
      }
      __pyx_t_9 = __Pyx_dict_iterator(__pyx_t_4, 1, ((PyObject *)NULL), (&__pyx_t_7), (&__pyx_t_8)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 252, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_3);
      __pyx_t_3 = __pyx_t_9;
      __pyx_t_9 = 0;
      while (1) {
        __pyx_t_10 = __Pyx_dict_iter_next(__pyx_t_3, __pyx_t_7, &__pyx_t_6, &__pyx_t_9, NULL, NULL, __pyx_t_8);
        if (unlikely(__pyx_t_10 == 0)) break;
        if (unlikely(__pyx_t_10 == -1)) __PYX_ERR(0, 252, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_XDECREF_SET(__pyx_8genexpr2__pyx_v_result, __pyx_t_9);
        __pyx_t_9 = 0;
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_8genexpr2__pyx_v_rank, __pyx_t_2);
        __pyx_t_9 = __Pyx_PyInt_AddObjC(__pyx_t_2, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 252, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_2);
        __pyx_t_2 = __pyx_t_9;
        __pyx_t_9 = 0;

        /* "octobot/strategy_optimizer/strategy_optimizer.py":249
 *         # index, evaluators, risk, score, trades
 *         if self.sorted_results_through_all_time_frame:
 *             results = [TestSuiteResult.convert_result_into_dict(rank, result[CONFIG].evaluators, "",             # <<<<<<<<<<<<<<
 *                                                                 result[CONFIG].risk, result[RANK],
 *                                                                 round(result[TRADES_IN_RESULT], 5))
 */
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_TestSuiteResult); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 249, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_convert_result_into_dict); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 249, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_CONFIG); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 249, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_12 = __Pyx_PyObject_GetItem(__pyx_8genexpr2__pyx_v_result, __pyx_t_4); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 249, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_evaluators); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 249, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

        /* "octobot/strategy_optimizer/strategy_optimizer.py":250
 *         if self.sorted_results_through_all_time_frame:
 *             results = [TestSuiteResult.convert_result_into_dict(rank, result[CONFIG].evaluators, "",
 *                                                                 result[CONFIG].risk, result[RANK],             # <<<<<<<<<<<<<<
 *                                                                 round(result[TRADES_IN_RESULT], 5))
 *                        for rank, result in enumerate(self.sorted_results_through_all_time_frame[0:100])]
 */
        __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_CONFIG); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 250, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_13 = __Pyx_PyObject_GetItem(__pyx_8genexpr2__pyx_v_result, __pyx_t_12); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 250, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_n_s_risk_2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 250, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_n_s_RANK); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 250, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_14 = __Pyx_PyObject_GetItem(__pyx_8genexpr2__pyx_v_result, __pyx_t_13); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 250, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

        /* "octobot/strategy_optimizer/strategy_optimizer.py":251
 *             results = [TestSuiteResult.convert_result_into_dict(rank, result[CONFIG].evaluators, "",
 *                                                                 result[CONFIG].risk, result[RANK],
 *                                                                 round(result[TRADES_IN_RESULT], 5))             # <<<<<<<<<<<<<<
 *                        for rank, result in enumerate(self.sorted_results_through_all_time_frame[0:100])]
 *         else:
 */
        __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_n_s_TRADES_IN_RESULT); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 251, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_15 = __Pyx_PyObject_GetItem(__pyx_8genexpr2__pyx_v_result, __pyx_t_13); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 251, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = PyTuple_New(2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 251, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_GIVEREF(__pyx_t_15);
        PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_15);
        __Pyx_INCREF(__pyx_int_5);
        __Pyx_GIVEREF(__pyx_int_5);
        PyTuple_SET_ITEM(__pyx_t_13, 1, __pyx_int_5);
        __pyx_t_15 = 0;
        __pyx_t_15 = __Pyx_PyObject_Call(__pyx_builtin_round, __pyx_t_13, NULL); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 251, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_10 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_11))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_11);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_11, function);
            __pyx_t_10 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_11)) {
          PyObject *__pyx_temp[7] = {__pyx_t_13, __pyx_8genexpr2__pyx_v_rank, __pyx_t_4, __pyx_kp_u__6, __pyx_t_12, __pyx_t_14, __pyx_t_15};
          __pyx_t_9 = __Pyx_PyFunction_FastCall(__pyx_t_11, __pyx_temp+1-__pyx_t_10, 6+__pyx_t_10); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 249, __pyx_L6_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_11)) {
          PyObject *__pyx_temp[7] = {__pyx_t_13, __pyx_8genexpr2__pyx_v_rank, __pyx_t_4, __pyx_kp_u__6, __pyx_t_12, __pyx_t_14, __pyx_t_15};
          __pyx_t_9 = __Pyx_PyCFunction_FastCall(__pyx_t_11, __pyx_temp+1-__pyx_t_10, 6+__pyx_t_10); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 249, __pyx_L6_error)
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        } else
        #endif
        {
          __pyx_t_16 = PyTuple_New(6+__pyx_t_10); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 249, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_16);
          if (__pyx_t_13) {
            __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_13); __pyx_t_13 = NULL;
          }
          __Pyx_INCREF(__pyx_8genexpr2__pyx_v_rank);
          __Pyx_GIVEREF(__pyx_8genexpr2__pyx_v_rank);
          PyTuple_SET_ITEM(__pyx_t_16, 0+__pyx_t_10, __pyx_8genexpr2__pyx_v_rank);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_16, 1+__pyx_t_10, __pyx_t_4);
          __Pyx_INCREF(__pyx_kp_u__6);
          __Pyx_GIVEREF(__pyx_kp_u__6);
          PyTuple_SET_ITEM(__pyx_t_16, 2+__pyx_t_10, __pyx_kp_u__6);
          __Pyx_GIVEREF(__pyx_t_12);
          PyTuple_SET_ITEM(__pyx_t_16, 3+__pyx_t_10, __pyx_t_12);
          __Pyx_GIVEREF(__pyx_t_14);
          PyTuple_SET_ITEM(__pyx_t_16, 4+__pyx_t_10, __pyx_t_14);
          __Pyx_GIVEREF(__pyx_t_15);
          PyTuple_SET_ITEM(__pyx_t_16, 5+__pyx_t_10, __pyx_t_15);
          __pyx_t_4 = 0;
          __pyx_t_12 = 0;
          __pyx_t_14 = 0;
          __pyx_t_15 = 0;
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_t_16, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 249, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        }
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_9))) __PYX_ERR(0, 249, __pyx_L6_error)
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_8genexpr2__pyx_v_rank); __pyx_8genexpr2__pyx_v_rank = 0;
      __Pyx_XDECREF(__pyx_8genexpr2__pyx_v_result); __pyx_8genexpr2__pyx_v_result = 0;
      goto __pyx_L9_exit_scope;
      __pyx_L6_error:;
      __Pyx_XDECREF(__pyx_8genexpr2__pyx_v_rank); __pyx_8genexpr2__pyx_v_rank = 0;
      __Pyx_XDECREF(__pyx_8genexpr2__pyx_v_result); __pyx_8genexpr2__pyx_v_result = 0;
      goto __pyx_L1_error;
      __pyx_L9_exit_scope:;
    } /* exit inner scope */
    __pyx_v_results = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":248
 *     def get_report(self):
 *         # index, evaluators, risk, score, trades
 *         if self.sorted_results_through_all_time_frame:             # <<<<<<<<<<<<<<
 *             results = [TestSuiteResult.convert_result_into_dict(rank, result[CONFIG].evaluators, "",
 *                                                                 result[CONFIG].risk, result[RANK],
 */
    goto __pyx_L3;
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":254
 *                        for rank, result in enumerate(self.sorted_results_through_all_time_frame[0:100])]
 *         else:
 *             results = []             # <<<<<<<<<<<<<<
 *         return results
 * 
 */
  /*else*/ {
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 254, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_results = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":255
 *         else:
 *             results = []
 *         return results             # <<<<<<<<<<<<<<
 * 
 *     def get_errors_description(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_results);
  __pyx_r = __pyx_v_results;
  goto __pyx_L0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":246
 *         return self.current_test_suite.current_progress if self.current_test_suite else 0
 * 
 *     def get_report(self):             # <<<<<<<<<<<<<<
 *         # index, evaluators, risk, score, trades
 *         if self.sorted_results_through_all_time_frame:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.get_report", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_results);
  __Pyx_XDECREF(__pyx_8genexpr2__pyx_v_rank);
  __Pyx_XDECREF(__pyx_8genexpr2__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_13get_report(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_13get_report = {"get_report", (PyCFunction)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_13get_report, METH_NOARGS, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_13get_report(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_report (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12get_report(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12get_report(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("get_report", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_report(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.get_report", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":257
 *         return results
 * 
 *     def get_errors_description(self):             # <<<<<<<<<<<<<<
 *         if self.errors:
 *             return f"{', '.join(self.errors)[0:350]} ..."
 */

static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15get_errors_description(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_errors_description(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("get_errors_description", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_errors_description); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 257, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15get_errors_description)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 257, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":258
 * 
 *     def get_errors_description(self):
 *         if self.errors:             # <<<<<<<<<<<<<<
 *             return f"{', '.join(self.errors)[0:350]} ..."
 *         else:
 */
  __pyx_t_5 = (__pyx_v_self->errors != Py_None)&&(PySet_GET_SIZE(__pyx_v_self->errors) != 0);
  if (__pyx_t_5) {

    /* "octobot/strategy_optimizer/strategy_optimizer.py":259
 *     def get_errors_description(self):
 *         if self.errors:
 *             return f"{', '.join(self.errors)[0:350]} ..."             # <<<<<<<<<<<<<<
 *         else:
 *             return None
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __pyx_v_self->errors;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = PyUnicode_Join(__pyx_kp_u__7, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 259, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyUnicode_Substring(__pyx_t_2, 0, 0x15E); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 259, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyUnicode_Concat(__pyx_t_1, __pyx_kp_u__8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 259, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":258
 * 
 *     def get_errors_description(self):
 *         if self.errors:             # <<<<<<<<<<<<<<
 *             return f"{', '.join(self.errors)[0:350]} ..."
 *         else:
 */
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":261
 *             return f"{', '.join(self.errors)[0:350]} ..."
 *         else:
 *             return None             # <<<<<<<<<<<<<<
 * 
 *     @classmethod
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":257
 *         return results
 * 
 *     def get_errors_description(self):             # <<<<<<<<<<<<<<
 *         if self.errors:
 *             return f"{', '.join(self.errors)[0:350]} ..."
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.get_errors_description", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15get_errors_description(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15get_errors_description = {"get_errors_description", (PyCFunction)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15get_errors_description, METH_NOARGS, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15get_errors_description(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_errors_description (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14get_errors_description(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14get_errors_description(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("get_errors_description", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_errors_description(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.get_errors_description", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":264
 * 
 *     @classmethod
 *     def get_name(cls):             # <<<<<<<<<<<<<<
 *         return cls.__name__
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_17get_name(PyObject *__pyx_v_cls, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_17get_name = {"get_name", (PyCFunction)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_17get_name, METH_NOARGS, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_17get_name(PyObject *__pyx_v_cls, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_name (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_16get_name(((PyTypeObject*)__pyx_v_cls));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_16get_name(PyTypeObject *__pyx_v_cls) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("get_name", 0);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":265
 *     @classmethod
 *     def get_name(cls):
 *         return cls.__name__             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_cls), __pyx_n_s_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":264
 * 
 *     @classmethod
 *     def get_name(cls):             # <<<<<<<<<<<<<<
 *         return cls.__name__
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.get_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":268
 * 
 *     @staticmethod
 *     def _get_activated_element(all_elements, current_forced_element, nb_elements_to_consider,             # <<<<<<<<<<<<<<
 *                                elem_conf_history, default_element=None, dict_shaped=False):
 *         eval_conf = {current_forced_element: True}
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_19_get_activated_element(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_19_get_activated_element = {"_get_activated_element", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_19_get_activated_element, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_19_get_activated_element(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_all_elements = 0;
  PyObject *__pyx_v_current_forced_element = 0;
  PyObject *__pyx_v_nb_elements_to_consider = 0;
  PyObject *__pyx_v_elem_conf_history = 0;
  PyObject *__pyx_v_default_element = 0;
  PyObject *__pyx_v_dict_shaped = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_activated_element (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_all_elements,&__pyx_n_s_current_forced_element,&__pyx_n_s_nb_elements_to_consider,&__pyx_n_s_elem_conf_history,&__pyx_n_s_default_element,&__pyx_n_s_dict_shaped,0};
    PyObject* values[6] = {0,0,0,0,0,0};

    /* "octobot/strategy_optimizer/strategy_optimizer.py":269
 *     @staticmethod
 *     def _get_activated_element(all_elements, current_forced_element, nb_elements_to_consider,
 *                                elem_conf_history, default_element=None, dict_shaped=False):             # <<<<<<<<<<<<<<
 *         eval_conf = {current_forced_element: True}
 *         additional_elements_count = 0
 */
    values[4] = ((PyObject *)Py_None);
    values[5] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_all_elements)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_current_forced_element)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_activated_element", 0, 4, 6, 1); __PYX_ERR(0, 268, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_nb_elements_to_consider)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_activated_element", 0, 4, 6, 2); __PYX_ERR(0, 268, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_elem_conf_history)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_activated_element", 0, 4, 6, 3); __PYX_ERR(0, 268, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_default_element);
          if (value) { values[4] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dict_shaped);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_get_activated_element") < 0)) __PYX_ERR(0, 268, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_all_elements = values[0];
    __pyx_v_current_forced_element = values[1];
    __pyx_v_nb_elements_to_consider = values[2];
    __pyx_v_elem_conf_history = values[3];
    __pyx_v_default_element = values[4];
    __pyx_v_dict_shaped = values[5];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_activated_element", 0, 4, 6, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 268, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer._get_activated_element", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_18_get_activated_element(__pyx_v_all_elements, __pyx_v_current_forced_element, __pyx_v_nb_elements_to_consider, __pyx_v_elem_conf_history, __pyx_v_default_element, __pyx_v_dict_shaped);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":268
 * 
 *     @staticmethod
 *     def _get_activated_element(all_elements, current_forced_element, nb_elements_to_consider,             # <<<<<<<<<<<<<<
 *                                elem_conf_history, default_element=None, dict_shaped=False):
 *         eval_conf = {current_forced_element: True}
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_18_get_activated_element(PyObject *__pyx_v_all_elements, PyObject *__pyx_v_current_forced_element, PyObject *__pyx_v_nb_elements_to_consider, PyObject *__pyx_v_elem_conf_history, PyObject *__pyx_v_default_element, PyObject *__pyx_v_dict_shaped) {
  PyObject *__pyx_v_eval_conf = NULL;
  PyObject *__pyx_v_additional_elements_count = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_current_elem = NULL;
  PyObject *__pyx_v_to_use_conf = NULL;
  PyObject *__pyx_8genexpr3__pyx_v_key = NULL;
  PyObject *__pyx_8genexpr4__pyx_v_key = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  __Pyx_RefNannySetupContext("_get_activated_element", 0);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":270
 *     def _get_activated_element(all_elements, current_forced_element, nb_elements_to_consider,
 *                                elem_conf_history, default_element=None, dict_shaped=False):
 *         eval_conf = {current_forced_element: True}             # <<<<<<<<<<<<<<
 *         additional_elements_count = 0
 *         if default_element is not None:
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_v_current_forced_element, Py_True) < 0) __PYX_ERR(0, 270, __pyx_L1_error)
  __pyx_v_eval_conf = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":271
 *                                elem_conf_history, default_element=None, dict_shaped=False):
 *         eval_conf = {current_forced_element: True}
 *         additional_elements_count = 0             # <<<<<<<<<<<<<<
 *         if default_element is not None:
 *             eval_conf[default_element] = True
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_additional_elements_count = __pyx_int_0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":272
 *         eval_conf = {current_forced_element: True}
 *         additional_elements_count = 0
 *         if default_element is not None:             # <<<<<<<<<<<<<<
 *             eval_conf[default_element] = True
 *             additional_elements_count = 1
 */
  __pyx_t_2 = (__pyx_v_default_element != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "octobot/strategy_optimizer/strategy_optimizer.py":273
 *         additional_elements_count = 0
 *         if default_element is not None:
 *             eval_conf[default_element] = True             # <<<<<<<<<<<<<<
 *             additional_elements_count = 1
 *         if nb_elements_to_consider > 1:
 */
    if (unlikely(PyDict_SetItem(__pyx_v_eval_conf, __pyx_v_default_element, Py_True) < 0)) __PYX_ERR(0, 273, __pyx_L1_error)

    /* "octobot/strategy_optimizer/strategy_optimizer.py":274
 *         if default_element is not None:
 *             eval_conf[default_element] = True
 *             additional_elements_count = 1             # <<<<<<<<<<<<<<
 *         if nb_elements_to_consider > 1:
 *             i = 0
 */
    __Pyx_INCREF(__pyx_int_1);
    __Pyx_DECREF_SET(__pyx_v_additional_elements_count, __pyx_int_1);

    /* "octobot/strategy_optimizer/strategy_optimizer.py":272
 *         eval_conf = {current_forced_element: True}
 *         additional_elements_count = 0
 *         if default_element is not None:             # <<<<<<<<<<<<<<
 *             eval_conf[default_element] = True
 *             additional_elements_count = 1
 */
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":275
 *             eval_conf[default_element] = True
 *             additional_elements_count = 1
 *         if nb_elements_to_consider > 1:             # <<<<<<<<<<<<<<
 *             i = 0
 *             while i < len(all_elements) and \
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_nb_elements_to_consider, __pyx_int_1, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "octobot/strategy_optimizer/strategy_optimizer.py":276
 *             additional_elements_count = 1
 *         if nb_elements_to_consider > 1:
 *             i = 0             # <<<<<<<<<<<<<<
 *             while i < len(all_elements) and \
 *                     len(eval_conf) < nb_elements_to_consider+additional_elements_count:
 */
    __Pyx_INCREF(__pyx_int_0);
    __pyx_v_i = __pyx_int_0;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":277
 *         if nb_elements_to_consider > 1:
 *             i = 0
 *             while i < len(all_elements) and \             # <<<<<<<<<<<<<<
 *                     len(eval_conf) < nb_elements_to_consider+additional_elements_count:
 *                 current_elem = all_elements[i]
 */
    while (1) {
      __pyx_t_4 = PyObject_Length(__pyx_v_all_elements); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 277, __pyx_L1_error)
      __pyx_t_1 = PyInt_FromSsize_t(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 277, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = PyObject_RichCompare(__pyx_v_i, __pyx_t_1, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 277, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 277, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (__pyx_t_2) {
      } else {
        __pyx_t_3 = __pyx_t_2;
        goto __pyx_L7_bool_binop_done;
      }

      /* "octobot/strategy_optimizer/strategy_optimizer.py":278
 *             i = 0
 *             while i < len(all_elements) and \
 *                     len(eval_conf) < nb_elements_to_consider+additional_elements_count:             # <<<<<<<<<<<<<<
 *                 current_elem = all_elements[i]
 *                 if current_elem not in eval_conf:
 */
      __pyx_t_4 = PyDict_Size(__pyx_v_eval_conf); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 278, __pyx_L1_error)
      __pyx_t_5 = PyInt_FromSsize_t(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 278, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_1 = PyNumber_Add(__pyx_v_nb_elements_to_consider, __pyx_v_additional_elements_count); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 278, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = PyObject_RichCompare(__pyx_t_5, __pyx_t_1, Py_LT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 278, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 278, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_3 = __pyx_t_2;
      __pyx_L7_bool_binop_done:;
      if (!__pyx_t_3) break;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":279
 *             while i < len(all_elements) and \
 *                     len(eval_conf) < nb_elements_to_consider+additional_elements_count:
 *                 current_elem = all_elements[i]             # <<<<<<<<<<<<<<
 *                 if current_elem not in eval_conf:
 *                     eval_conf[current_elem] = True
 */
      __pyx_t_6 = __Pyx_PyObject_GetItem(__pyx_v_all_elements, __pyx_v_i); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 279, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_XDECREF_SET(__pyx_v_current_elem, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":280
 *                     len(eval_conf) < nb_elements_to_consider+additional_elements_count:
 *                 current_elem = all_elements[i]
 *                 if current_elem not in eval_conf:             # <<<<<<<<<<<<<<
 *                     eval_conf[current_elem] = True
 *                     if len(eval_conf) == nb_elements_to_consider+additional_elements_count and \
 */
      __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_v_current_elem, __pyx_v_eval_conf, Py_NE)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 280, __pyx_L1_error)
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {

        /* "octobot/strategy_optimizer/strategy_optimizer.py":281
 *                 current_elem = all_elements[i]
 *                 if current_elem not in eval_conf:
 *                     eval_conf[current_elem] = True             # <<<<<<<<<<<<<<
 *                     if len(eval_conf) == nb_elements_to_consider+additional_elements_count and \
 *                             ((eval_conf if dict_shaped else sorted([key.value for key in eval_conf]))
 */
        if (unlikely(PyDict_SetItem(__pyx_v_eval_conf, __pyx_v_current_elem, Py_True) < 0)) __PYX_ERR(0, 281, __pyx_L1_error)

        /* "octobot/strategy_optimizer/strategy_optimizer.py":282
 *                 if current_elem not in eval_conf:
 *                     eval_conf[current_elem] = True
 *                     if len(eval_conf) == nb_elements_to_consider+additional_elements_count and \             # <<<<<<<<<<<<<<
 *                             ((eval_conf if dict_shaped else sorted([key.value for key in eval_conf]))
 *                              in elem_conf_history):
 */
        __pyx_t_4 = PyDict_Size(__pyx_v_eval_conf); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 282, __pyx_L1_error)
        __pyx_t_6 = PyInt_FromSsize_t(__pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 282, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_1 = PyNumber_Add(__pyx_v_nb_elements_to_consider, __pyx_v_additional_elements_count); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 282, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_5 = PyObject_RichCompare(__pyx_t_6, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 282, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 282, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (__pyx_t_3) {
        } else {
          __pyx_t_2 = __pyx_t_3;
          goto __pyx_L11_bool_binop_done;
        }

        /* "octobot/strategy_optimizer/strategy_optimizer.py":283
 *                     eval_conf[current_elem] = True
 *                     if len(eval_conf) == nb_elements_to_consider+additional_elements_count and \
 *                             ((eval_conf if dict_shaped else sorted([key.value for key in eval_conf]))             # <<<<<<<<<<<<<<
 *                              in elem_conf_history):
 *                         eval_conf.pop(current_elem)
 */
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_dict_shaped); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 283, __pyx_L1_error)
        if (__pyx_t_3) {
          __Pyx_INCREF(__pyx_v_eval_conf);
          __pyx_t_5 = __pyx_v_eval_conf;
        } else {
          { /* enter inner scope */
            __pyx_t_6 = PyList_New(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 283, __pyx_L15_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_t_4 = 0;
            __pyx_t_10 = __Pyx_dict_iterator(__pyx_v_eval_conf, 1, ((PyObject *)NULL), (&__pyx_t_8), (&__pyx_t_9)); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 283, __pyx_L15_error)
            __Pyx_GOTREF(__pyx_t_10);
            __Pyx_XDECREF(__pyx_t_7);
            __pyx_t_7 = __pyx_t_10;
            __pyx_t_10 = 0;
            while (1) {
              __pyx_t_11 = __Pyx_dict_iter_next(__pyx_t_7, __pyx_t_8, &__pyx_t_4, &__pyx_t_10, NULL, NULL, __pyx_t_9);
              if (unlikely(__pyx_t_11 == 0)) break;
              if (unlikely(__pyx_t_11 == -1)) __PYX_ERR(0, 283, __pyx_L15_error)
              __Pyx_GOTREF(__pyx_t_10);
              __Pyx_XDECREF_SET(__pyx_8genexpr3__pyx_v_key, __pyx_t_10);
              __pyx_t_10 = 0;
              __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_8genexpr3__pyx_v_key, __pyx_n_s_value); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 283, __pyx_L15_error)
              __Pyx_GOTREF(__pyx_t_10);
              if (unlikely(__Pyx_ListComp_Append(__pyx_t_6, (PyObject*)__pyx_t_10))) __PYX_ERR(0, 283, __pyx_L15_error)
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            }
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_XDECREF(__pyx_8genexpr3__pyx_v_key); __pyx_8genexpr3__pyx_v_key = 0;
            goto __pyx_L18_exit_scope;
            __pyx_L15_error:;
            __Pyx_XDECREF(__pyx_8genexpr3__pyx_v_key); __pyx_8genexpr3__pyx_v_key = 0;
            goto __pyx_L1_error;
            __pyx_L18_exit_scope:;
          } /* exit inner scope */
          __pyx_t_1 = ((PyObject*)__pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_12 = PyList_Sort(__pyx_t_1); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 283, __pyx_L1_error)
          __pyx_t_5 = __pyx_t_1;
          __pyx_t_1 = 0;
        }

        /* "octobot/strategy_optimizer/strategy_optimizer.py":284
 *                     if len(eval_conf) == nb_elements_to_consider+additional_elements_count and \
 *                             ((eval_conf if dict_shaped else sorted([key.value for key in eval_conf]))
 *                              in elem_conf_history):             # <<<<<<<<<<<<<<
 *                         eval_conf.pop(current_elem)
 *                 i += 1
 */
        __pyx_t_3 = (__Pyx_PySequence_ContainsTF(__pyx_t_5, __pyx_v_elem_conf_history, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 284, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_13 = (__pyx_t_3 != 0);
        __pyx_t_2 = __pyx_t_13;
        __pyx_L11_bool_binop_done:;

        /* "octobot/strategy_optimizer/strategy_optimizer.py":282
 *                 if current_elem not in eval_conf:
 *                     eval_conf[current_elem] = True
 *                     if len(eval_conf) == nb_elements_to_consider+additional_elements_count and \             # <<<<<<<<<<<<<<
 *                             ((eval_conf if dict_shaped else sorted([key.value for key in eval_conf]))
 *                              in elem_conf_history):
 */
        if (__pyx_t_2) {

          /* "octobot/strategy_optimizer/strategy_optimizer.py":285
 *                             ((eval_conf if dict_shaped else sorted([key.value for key in eval_conf]))
 *                              in elem_conf_history):
 *                         eval_conf.pop(current_elem)             # <<<<<<<<<<<<<<
 *                 i += 1
 *         if len(eval_conf) == nb_elements_to_consider+additional_elements_count:
 */
          __pyx_t_5 = __Pyx_PyDict_Pop(__pyx_v_eval_conf, __pyx_v_current_elem, ((PyObject *)NULL)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 285, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

          /* "octobot/strategy_optimizer/strategy_optimizer.py":282
 *                 if current_elem not in eval_conf:
 *                     eval_conf[current_elem] = True
 *                     if len(eval_conf) == nb_elements_to_consider+additional_elements_count and \             # <<<<<<<<<<<<<<
 *                             ((eval_conf if dict_shaped else sorted([key.value for key in eval_conf]))
 *                              in elem_conf_history):
 */
        }

        /* "octobot/strategy_optimizer/strategy_optimizer.py":280
 *                     len(eval_conf) < nb_elements_to_consider+additional_elements_count:
 *                 current_elem = all_elements[i]
 *                 if current_elem not in eval_conf:             # <<<<<<<<<<<<<<
 *                     eval_conf[current_elem] = True
 *                     if len(eval_conf) == nb_elements_to_consider+additional_elements_count and \
 */
      }

      /* "octobot/strategy_optimizer/strategy_optimizer.py":286
 *                              in elem_conf_history):
 *                         eval_conf.pop(current_elem)
 *                 i += 1             # <<<<<<<<<<<<<<
 *         if len(eval_conf) == nb_elements_to_consider+additional_elements_count:
 *             to_use_conf = eval_conf
 */
      __pyx_t_5 = __Pyx_PyInt_AddObjC(__pyx_v_i, __pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 286, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF_SET(__pyx_v_i, __pyx_t_5);
      __pyx_t_5 = 0;
    }

    /* "octobot/strategy_optimizer/strategy_optimizer.py":275
 *             eval_conf[default_element] = True
 *             additional_elements_count = 1
 *         if nb_elements_to_consider > 1:             # <<<<<<<<<<<<<<
 *             i = 0
 *             while i < len(all_elements) and \
 */
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":287
 *                         eval_conf.pop(current_elem)
 *                 i += 1
 *         if len(eval_conf) == nb_elements_to_consider+additional_elements_count:             # <<<<<<<<<<<<<<
 *             to_use_conf = eval_conf
 *             if not dict_shaped:
 */
  __pyx_t_8 = PyDict_Size(__pyx_v_eval_conf); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 287, __pyx_L1_error)
  __pyx_t_5 = PyInt_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = PyNumber_Add(__pyx_v_nb_elements_to_consider, __pyx_v_additional_elements_count); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = PyObject_RichCompare(__pyx_t_5, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (__pyx_t_2) {

    /* "octobot/strategy_optimizer/strategy_optimizer.py":288
 *                 i += 1
 *         if len(eval_conf) == nb_elements_to_consider+additional_elements_count:
 *             to_use_conf = eval_conf             # <<<<<<<<<<<<<<
 *             if not dict_shaped:
 *                 to_use_conf = sorted([key.value for key in eval_conf])
 */
    __Pyx_INCREF(__pyx_v_eval_conf);
    __pyx_v_to_use_conf = __pyx_v_eval_conf;

    /* "octobot/strategy_optimizer/strategy_optimizer.py":289
 *         if len(eval_conf) == nb_elements_to_consider+additional_elements_count:
 *             to_use_conf = eval_conf
 *             if not dict_shaped:             # <<<<<<<<<<<<<<
 *                 to_use_conf = sorted([key.value for key in eval_conf])
 *             if to_use_conf not in elem_conf_history:
 */
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_dict_shaped); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 289, __pyx_L1_error)
    __pyx_t_13 = ((!__pyx_t_2) != 0);
    if (__pyx_t_13) {

      /* "octobot/strategy_optimizer/strategy_optimizer.py":290
 *             to_use_conf = eval_conf
 *             if not dict_shaped:
 *                 to_use_conf = sorted([key.value for key in eval_conf])             # <<<<<<<<<<<<<<
 *             if to_use_conf not in elem_conf_history:
 *                 elem_conf_history.append(to_use_conf)
 */
      { /* enter inner scope */
        __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 290, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_8 = 0;
        __pyx_t_7 = __Pyx_dict_iterator(__pyx_v_eval_conf, 1, ((PyObject *)NULL), (&__pyx_t_4), (&__pyx_t_9)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 290, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_XDECREF(__pyx_t_5);
        __pyx_t_5 = __pyx_t_7;
        __pyx_t_7 = 0;
        while (1) {
          __pyx_t_11 = __Pyx_dict_iter_next(__pyx_t_5, __pyx_t_4, &__pyx_t_8, &__pyx_t_7, NULL, NULL, __pyx_t_9);
          if (unlikely(__pyx_t_11 == 0)) break;
          if (unlikely(__pyx_t_11 == -1)) __PYX_ERR(0, 290, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_XDECREF_SET(__pyx_8genexpr4__pyx_v_key, __pyx_t_7);
          __pyx_t_7 = 0;
          __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_8genexpr4__pyx_v_key, __pyx_n_s_value); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 290, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_7);
          if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 290, __pyx_L23_error)
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_8genexpr4__pyx_v_key); __pyx_8genexpr4__pyx_v_key = 0;
        goto __pyx_L26_exit_scope;
        __pyx_L23_error:;
        __Pyx_XDECREF(__pyx_8genexpr4__pyx_v_key); __pyx_8genexpr4__pyx_v_key = 0;
        goto __pyx_L1_error;
        __pyx_L26_exit_scope:;
      } /* exit inner scope */
      __pyx_t_6 = ((PyObject*)__pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_12 = PyList_Sort(__pyx_t_6); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 290, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_to_use_conf, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":289
 *         if len(eval_conf) == nb_elements_to_consider+additional_elements_count:
 *             to_use_conf = eval_conf
 *             if not dict_shaped:             # <<<<<<<<<<<<<<
 *                 to_use_conf = sorted([key.value for key in eval_conf])
 *             if to_use_conf not in elem_conf_history:
 */
    }

    /* "octobot/strategy_optimizer/strategy_optimizer.py":291
 *             if not dict_shaped:
 *                 to_use_conf = sorted([key.value for key in eval_conf])
 *             if to_use_conf not in elem_conf_history:             # <<<<<<<<<<<<<<
 *                 elem_conf_history.append(to_use_conf)
 *                 return to_use_conf
 */
    __pyx_t_13 = (__Pyx_PySequence_ContainsTF(__pyx_v_to_use_conf, __pyx_v_elem_conf_history, Py_NE)); if (unlikely(__pyx_t_13 < 0)) __PYX_ERR(0, 291, __pyx_L1_error)
    __pyx_t_2 = (__pyx_t_13 != 0);
    if (__pyx_t_2) {

      /* "octobot/strategy_optimizer/strategy_optimizer.py":292
 *                 to_use_conf = sorted([key.value for key in eval_conf])
 *             if to_use_conf not in elem_conf_history:
 *                 elem_conf_history.append(to_use_conf)             # <<<<<<<<<<<<<<
 *                 return to_use_conf
 *         return None
 */
      __pyx_t_12 = __Pyx_PyObject_Append(__pyx_v_elem_conf_history, __pyx_v_to_use_conf); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 292, __pyx_L1_error)

      /* "octobot/strategy_optimizer/strategy_optimizer.py":293
 *             if to_use_conf not in elem_conf_history:
 *                 elem_conf_history.append(to_use_conf)
 *                 return to_use_conf             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_to_use_conf);
      __pyx_r = __pyx_v_to_use_conf;
      goto __pyx_L0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":291
 *             if not dict_shaped:
 *                 to_use_conf = sorted([key.value for key in eval_conf])
 *             if to_use_conf not in elem_conf_history:             # <<<<<<<<<<<<<<
 *                 elem_conf_history.append(to_use_conf)
 *                 return to_use_conf
 */
    }

    /* "octobot/strategy_optimizer/strategy_optimizer.py":287
 *                         eval_conf.pop(current_elem)
 *                 i += 1
 *         if len(eval_conf) == nb_elements_to_consider+additional_elements_count:             # <<<<<<<<<<<<<<
 *             to_use_conf = eval_conf
 *             if not dict_shaped:
 */
  }

  /* "octobot/strategy_optimizer/strategy_optimizer.py":294
 *                 elem_conf_history.append(to_use_conf)
 *                 return to_use_conf
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":268
 * 
 *     @staticmethod
 *     def _get_activated_element(all_elements, current_forced_element, nb_elements_to_consider,             # <<<<<<<<<<<<<<
 *                                elem_conf_history, default_element=None, dict_shaped=False):
 *         eval_conf = {current_forced_element: True}
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer._get_activated_element", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_eval_conf);
  __Pyx_XDECREF(__pyx_v_additional_elements_count);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_current_elem);
  __Pyx_XDECREF(__pyx_v_to_use_conf);
  __Pyx_XDECREF(__pyx_8genexpr3__pyx_v_key);
  __Pyx_XDECREF(__pyx_8genexpr4__pyx_v_key);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":297
 * 
 *     @staticmethod
 *     def _get_filtered_results(results, time_frame=None):             # <<<<<<<<<<<<<<
 *         return [result for result in results if time_frame is None or result.min_time_frame == time_frame]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_21_get_filtered_results(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_21_get_filtered_results = {"_get_filtered_results", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_21_get_filtered_results, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_21_get_filtered_results(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_results = 0;
  PyObject *__pyx_v_time_frame = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_filtered_results (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_results,&__pyx_n_s_time_frame,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_results)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_time_frame);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_get_filtered_results") < 0)) __PYX_ERR(0, 297, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_results = values[0];
    __pyx_v_time_frame = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_filtered_results", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 297, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer._get_filtered_results", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_20_get_filtered_results(__pyx_v_results, __pyx_v_time_frame);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_20_get_filtered_results(PyObject *__pyx_v_results, PyObject *__pyx_v_time_frame) {
  PyObject *__pyx_8genexpr5__pyx_v_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("_get_filtered_results", 0);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":298
 *     @staticmethod
 *     def _get_filtered_results(results, time_frame=None):
 *         return [result for result in results if time_frame is None or result.min_time_frame == time_frame]             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 298, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (likely(PyList_CheckExact(__pyx_v_results)) || PyTuple_CheckExact(__pyx_v_results)) {
      __pyx_t_2 = __pyx_v_results; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
      __pyx_t_4 = NULL;
    } else {
      __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_results); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 298, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 298, __pyx_L5_error)
    }
    for (;;) {
      if (likely(!__pyx_t_4)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 298, __pyx_L5_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 298, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        } else {
          if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 298, __pyx_L5_error)
          #else
          __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 298, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
        }
      } else {
        __pyx_t_5 = __pyx_t_4(__pyx_t_2);
        if (unlikely(!__pyx_t_5)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 298, __pyx_L5_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_XDECREF_SET(__pyx_8genexpr5__pyx_v_result, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_7 = (__pyx_v_time_frame == Py_None);
      __pyx_t_8 = (__pyx_t_7 != 0);
      if (!__pyx_t_8) {
      } else {
        __pyx_t_6 = __pyx_t_8;
        goto __pyx_L9_bool_binop_done;
      }
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_8genexpr5__pyx_v_result, __pyx_n_s_min_time_frame); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 298, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_9 = PyObject_RichCompare(__pyx_t_5, __pyx_v_time_frame, Py_EQ); __Pyx_XGOTREF(__pyx_t_9); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 298, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 298, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_6 = __pyx_t_8;
      __pyx_L9_bool_binop_done:;
      if (__pyx_t_6) {
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_8genexpr5__pyx_v_result))) __PYX_ERR(0, 298, __pyx_L5_error)
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_8genexpr5__pyx_v_result); __pyx_8genexpr5__pyx_v_result = 0;
    goto __pyx_L11_exit_scope;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_8genexpr5__pyx_v_result); __pyx_8genexpr5__pyx_v_result = 0;
    goto __pyx_L1_error;
    __pyx_L11_exit_scope:;
  } /* exit inner scope */
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":297
 * 
 *     @staticmethod
 *     def _get_filtered_results(results, time_frame=None):             # <<<<<<<<<<<<<<
 *         return [result for result in results if time_frame is None or result.min_time_frame == time_frame]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer._get_filtered_results", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_8genexpr5__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":301
 * 
 *     @staticmethod
 *     def _get_sorted_results(results, time_frame=None):             # <<<<<<<<<<<<<<
 *         return sorted(StrategyOptimizer._get_filtered_results(results, time_frame),
 *                       key=lambda result: result.get_average_score(), reverse=True)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23_get_sorted_results(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23_get_sorted_results = {"_get_sorted_results", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23_get_sorted_results, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23_get_sorted_results(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_results = 0;
  PyObject *__pyx_v_time_frame = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_sorted_results (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_results,&__pyx_n_s_time_frame,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_results)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_time_frame);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_get_sorted_results") < 0)) __PYX_ERR(0, 301, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_results = values[0];
    __pyx_v_time_frame = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_sorted_results", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 301, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer._get_sorted_results", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_22_get_sorted_results(__pyx_v_results, __pyx_v_time_frame);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":303
 *     def _get_sorted_results(results, time_frame=None):
 *         return sorted(StrategyOptimizer._get_filtered_results(results, time_frame),
 *                       key=lambda result: result.get_average_score(), reverse=True)             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_19_get_sorted_results_lambda(PyObject *__pyx_self, PyObject *__pyx_v_result); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_19_get_sorted_results_lambda = {"lambda", (PyCFunction)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_19_get_sorted_results_lambda, METH_O, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_19_get_sorted_results_lambda(PyObject *__pyx_self, PyObject *__pyx_v_result) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lambda (wrapper)", 0);
  __pyx_r = __pyx_lambda_funcdef_lambda(__pyx_self, ((PyObject *)__pyx_v_result));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_lambda_funcdef_lambda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_result) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("lambda", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_result, __pyx_n_s_get_average_score); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer._get_sorted_results.lambda", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":301
 * 
 *     @staticmethod
 *     def _get_sorted_results(results, time_frame=None):             # <<<<<<<<<<<<<<
 *         return sorted(StrategyOptimizer._get_filtered_results(results, time_frame),
 *                       key=lambda result: result.get_average_score(), reverse=True)
 */

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_22_get_sorted_results(PyObject *__pyx_v_results, PyObject *__pyx_v_time_frame) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("_get_sorted_results", 0);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":302
 *     @staticmethod
 *     def _get_sorted_results(results, time_frame=None):
 *         return sorted(StrategyOptimizer._get_filtered_results(results, time_frame),             # <<<<<<<<<<<<<<
 *                       key=lambda result: result.get_average_score(), reverse=True)
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer), __pyx_n_s_get_filtered_results); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 302, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_results, __pyx_v_time_frame};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 302, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_results, __pyx_v_time_frame};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 302, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 302, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_results);
    __Pyx_GIVEREF(__pyx_v_results);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_results);
    __Pyx_INCREF(__pyx_v_time_frame);
    __Pyx_GIVEREF(__pyx_v_time_frame);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_time_frame);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 302, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 302, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":303
 *     def _get_sorted_results(results, time_frame=None):
 *         return sorted(StrategyOptimizer._get_filtered_results(results, time_frame),
 *                       key=lambda result: result.get_average_score(), reverse=True)             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_19_get_sorted_results_lambda, 0, __pyx_n_s_get_sorted_results_locals_lambd, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_key, __pyx_t_5) < 0) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_reverse, Py_True) < 0) __PYX_ERR(0, 303, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":302
 *     @staticmethod
 *     def _get_sorted_results(results, time_frame=None):
 *         return sorted(StrategyOptimizer._get_filtered_results(results, time_frame),             # <<<<<<<<<<<<<<
 *                       key=lambda result: result.get_average_score(), reverse=True)
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_sorted, __pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 302, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":301
 * 
 *     @staticmethod
 *     def _get_sorted_results(results, time_frame=None):             # <<<<<<<<<<<<<<
 *         return sorted(StrategyOptimizer._get_filtered_results(results, time_frame),
 *                       key=lambda result: result.get_average_score(), reverse=True)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer._get_sorted_results", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":306
 * 
 *     @staticmethod
 *     def _is_relevant_evaluation_config(evaluator):             # <<<<<<<<<<<<<<
 *         return get_class_from_string(evaluator, TAEvaluator, TA, evaluator_parent_inspection) is not None
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_25_is_relevant_evaluation_config(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_25_is_relevant_evaluation_config = {"_is_relevant_evaluation_config", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_25_is_relevant_evaluation_config, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_25_is_relevant_evaluation_config(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_evaluator = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_is_relevant_evaluation_config (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_evaluator,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_evaluator)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_is_relevant_evaluation_config") < 0)) __PYX_ERR(0, 306, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_evaluator = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_is_relevant_evaluation_config", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 306, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer._is_relevant_evaluation_config", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_24_is_relevant_evaluation_config(__pyx_v_evaluator);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_24_is_relevant_evaluation_config(PyObject *__pyx_v_evaluator) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  __Pyx_RefNannySetupContext("_is_relevant_evaluation_config", 0);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":307
 *     @staticmethod
 *     def _is_relevant_evaluation_config(evaluator):
 *         return get_class_from_string(evaluator, TAEvaluator, TA, evaluator_parent_inspection) is not None             # <<<<<<<<<<<<<<
 * 
 *     def _get_all_TA(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_get_class_from_string); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_TAEvaluator); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_TA); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_evaluator_parent_inspection); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_6, __pyx_v_evaluator, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 4+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_6, __pyx_v_evaluator, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 4+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(4+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_INCREF(__pyx_v_evaluator);
    __Pyx_GIVEREF(__pyx_v_evaluator);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_v_evaluator);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 3+__pyx_t_7, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_9 = (__pyx_t_1 != Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":306
 * 
 *     @staticmethod
 *     def _is_relevant_evaluation_config(evaluator):             # <<<<<<<<<<<<<<
 *         return get_class_from_string(evaluator, TAEvaluator, TA, evaluator_parent_inspection) is not None
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer._is_relevant_evaluation_config", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.py":309
 *         return get_class_from_string(evaluator, TAEvaluator, TA, evaluator_parent_inspection) is not None
 * 
 *     def _get_all_TA(self):             # <<<<<<<<<<<<<<
 *         return [evaluator
 *                 for evaluator, activated in get_tentacles_activation(self.tentacles_setup_config).items()
 */

static PyObject *__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer__get_all_TA(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_8genexpr6__pyx_v_evaluator = NULL;
  PyObject *__pyx_8genexpr6__pyx_v_activated = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  __Pyx_RefNannySetupContext("_get_all_TA", 0);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":310
 * 
 *     def _get_all_TA(self):
 *         return [evaluator             # <<<<<<<<<<<<<<
 *                 for evaluator, activated in get_tentacles_activation(self.tentacles_setup_config).items()
 *                 if activated and StrategyOptimizer._is_relevant_evaluation_config(evaluator)]
 */
  __Pyx_XDECREF(__pyx_r);
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 310, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "octobot/strategy_optimizer/strategy_optimizer.py":311
 *     def _get_all_TA(self):
 *         return [evaluator
 *                 for evaluator, activated in get_tentacles_activation(self.tentacles_setup_config).items()             # <<<<<<<<<<<<<<
 *                 if activated and StrategyOptimizer._is_relevant_evaluation_config(evaluator)]
 */
    __pyx_t_3 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_get_tentacles_activation); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 311, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_6 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_v_self->tentacles_setup_config) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_self->tentacles_setup_config);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 311, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_t_6 == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
      __PYX_ERR(0, 311, __pyx_L5_error)
    }
    __pyx_t_7 = __Pyx_dict_iterator(__pyx_t_6, 0, __pyx_n_s_items, (&__pyx_t_4), (&__pyx_t_5)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 311, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_2);
    __pyx_t_2 = __pyx_t_7;
    __pyx_t_7 = 0;
    while (1) {
      __pyx_t_9 = __Pyx_dict_iter_next(__pyx_t_2, __pyx_t_4, &__pyx_t_3, &__pyx_t_7, &__pyx_t_6, NULL, __pyx_t_5);
      if (unlikely(__pyx_t_9 == 0)) break;
      if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 311, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_XDECREF_SET(__pyx_8genexpr6__pyx_v_evaluator, __pyx_t_7);
      __pyx_t_7 = 0;
      __Pyx_XDECREF_SET(__pyx_8genexpr6__pyx_v_activated, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "octobot/strategy_optimizer/strategy_optimizer.py":312
 *         return [evaluator
 *                 for evaluator, activated in get_tentacles_activation(self.tentacles_setup_config).items()
 *                 if activated and StrategyOptimizer._is_relevant_evaluation_config(evaluator)]             # <<<<<<<<<<<<<<
 */
      __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_8genexpr6__pyx_v_activated); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 312, __pyx_L5_error)
      if (__pyx_t_11) {
      } else {
        __pyx_t_10 = __pyx_t_11;
        goto __pyx_L9_bool_binop_done;
      }
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer), __pyx_n_s_is_relevant_evaluation_config); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 312, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_6 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_8genexpr6__pyx_v_evaluator) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_8genexpr6__pyx_v_evaluator);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 312, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 312, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_10 = __pyx_t_11;
      __pyx_L9_bool_binop_done:;
      if (__pyx_t_10) {

        /* "octobot/strategy_optimizer/strategy_optimizer.py":310
 * 
 *     def _get_all_TA(self):
 *         return [evaluator             # <<<<<<<<<<<<<<
 *                 for evaluator, activated in get_tentacles_activation(self.tentacles_setup_config).items()
 *                 if activated and StrategyOptimizer._is_relevant_evaluation_config(evaluator)]
 */
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_8genexpr6__pyx_v_evaluator))) __PYX_ERR(0, 310, __pyx_L5_error)

        /* "octobot/strategy_optimizer/strategy_optimizer.py":312
 *         return [evaluator
 *                 for evaluator, activated in get_tentacles_activation(self.tentacles_setup_config).items()
 *                 if activated and StrategyOptimizer._is_relevant_evaluation_config(evaluator)]             # <<<<<<<<<<<<<<
 */
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_8genexpr6__pyx_v_activated); __pyx_8genexpr6__pyx_v_activated = 0;
    __Pyx_XDECREF(__pyx_8genexpr6__pyx_v_evaluator); __pyx_8genexpr6__pyx_v_evaluator = 0;
    goto __pyx_L11_exit_scope;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_8genexpr6__pyx_v_activated); __pyx_8genexpr6__pyx_v_activated = 0;
    __Pyx_XDECREF(__pyx_8genexpr6__pyx_v_evaluator); __pyx_8genexpr6__pyx_v_evaluator = 0;
    goto __pyx_L1_error;
    __pyx_L11_exit_scope:;
  } /* exit inner scope */
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":309
 *         return get_class_from_string(evaluator, TAEvaluator, TA, evaluator_parent_inspection) is not None
 * 
 *     def _get_all_TA(self):             # <<<<<<<<<<<<<<
 *         return [evaluator
 *                 for evaluator, activated in get_tentacles_activation(self.tentacles_setup_config).items()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer._get_all_TA", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_8genexpr6__pyx_v_evaluator);
  __Pyx_XDECREF(__pyx_8genexpr6__pyx_v_activated);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.pxd":29
 *     cdef int total_nb_runs
 * 
 *     cdef public bint is_properly_initialized             # <<<<<<<<<<<<<<
 *     cdef public object trading_mode
 *     cdef public object strategy_class
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23is_properly_initialized_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23is_properly_initialized_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23is_properly_initialized___get__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23is_properly_initialized___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->is_properly_initialized); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.is_properly_initialized.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23is_properly_initialized_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23is_properly_initialized_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23is_properly_initialized_2__set__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23is_properly_initialized_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 29, __pyx_L1_error)
  __pyx_v_self->is_properly_initialized = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.is_properly_initialized.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.pxd":30
 * 
 *     cdef public bint is_properly_initialized
 *     cdef public object trading_mode             # <<<<<<<<<<<<<<
 *     cdef public object strategy_class
 *     cdef public list all_time_frames
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode___get__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->trading_mode);
  __pyx_r = __pyx_v_self->trading_mode;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode_2__set__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->trading_mode);
  __Pyx_DECREF(__pyx_v_self->trading_mode);
  __pyx_v_self->trading_mode = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode_4__del__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode_4__del__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->trading_mode);
  __Pyx_DECREF(__pyx_v_self->trading_mode);
  __pyx_v_self->trading_mode = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.pxd":31
 *     cdef public bint is_properly_initialized
 *     cdef public object trading_mode
 *     cdef public object strategy_class             # <<<<<<<<<<<<<<
 *     cdef public list all_time_frames
 *     cdef public list all_TAs
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class___get__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->strategy_class);
  __pyx_r = __pyx_v_self->strategy_class;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class_2__set__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->strategy_class);
  __Pyx_DECREF(__pyx_v_self->strategy_class);
  __pyx_v_self->strategy_class = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class_4__del__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class_4__del__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->strategy_class);
  __Pyx_DECREF(__pyx_v_self->strategy_class);
  __pyx_v_self->strategy_class = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.pxd":32
 *     cdef public object trading_mode
 *     cdef public object strategy_class
 *     cdef public list all_time_frames             # <<<<<<<<<<<<<<
 *     cdef public list all_TAs
 *     cdef public list risks
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames___get__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->all_time_frames);
  __pyx_r = __pyx_v_self->all_time_frames;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames_2__set__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 32, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->all_time_frames);
  __Pyx_DECREF(__pyx_v_self->all_time_frames);
  __pyx_v_self->all_time_frames = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.all_time_frames.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames_4__del__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames_4__del__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->all_time_frames);
  __Pyx_DECREF(__pyx_v_self->all_time_frames);
  __pyx_v_self->all_time_frames = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.pxd":33
 *     cdef public object strategy_class
 *     cdef public list all_time_frames
 *     cdef public list all_TAs             # <<<<<<<<<<<<<<
 *     cdef public list risks
 *     cdef public bint is_computing
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs___get__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->all_TAs);
  __pyx_r = __pyx_v_self->all_TAs;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs_2__set__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 33, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->all_TAs);
  __Pyx_DECREF(__pyx_v_self->all_TAs);
  __pyx_v_self->all_TAs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.all_TAs.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs_4__del__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs_4__del__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->all_TAs);
  __Pyx_DECREF(__pyx_v_self->all_TAs);
  __pyx_v_self->all_TAs = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.pxd":34
 *     cdef public list all_time_frames
 *     cdef public list all_TAs
 *     cdef public list risks             # <<<<<<<<<<<<<<
 *     cdef public bint is_computing
 *     cdef public list run_results
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks___get__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->risks);
  __pyx_r = __pyx_v_self->risks;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks_2__set__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 34, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->risks);
  __Pyx_DECREF(__pyx_v_self->risks);
  __pyx_v_self->risks = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.risks.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks_4__del__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks_4__del__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->risks);
  __Pyx_DECREF(__pyx_v_self->risks);
  __pyx_v_self->risks = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.pxd":35
 *     cdef public list all_TAs
 *     cdef public list risks
 *     cdef public bint is_computing             # <<<<<<<<<<<<<<
 *     cdef public list run_results
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12is_computing_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12is_computing_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12is_computing___get__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12is_computing___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->is_computing); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.is_computing.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12is_computing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12is_computing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12is_computing_2__set__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12is_computing_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 35, __pyx_L1_error)
  __pyx_v_self->is_computing = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.is_computing.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "octobot/strategy_optimizer/strategy_optimizer.pxd":36
 *     cdef public list risks
 *     cdef public bint is_computing
 *     cdef public list run_results             # <<<<<<<<<<<<<<
 * 
 *     cpdef void find_optimal_configuration(self, list TAs=*, list time_frames=*, list risks=*)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results___get__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results___get__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->run_results);
  __pyx_r = __pyx_v_self->run_results;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results_2__set__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results_2__set__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 36, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->run_results);
  __Pyx_DECREF(__pyx_v_self->run_results);
  __pyx_v_self->run_results = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.run_results.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results_4__del__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results_4__del__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->run_results);
  __Pyx_DECREF(__pyx_v_self->run_results);
  __pyx_v_self->run_results = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_27__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_27__reduce_cython__ = {"__reduce_cython__", (PyCFunction)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_27__reduce_cython__, METH_NOARGS, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_27__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_26__reduce_cython__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_26__reduce_cython__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.all_TAs, self.all_time_frames, self.config, self.current_test_suite, self.errors, self.is_computing, self.is_properly_initialized, self.logger, self.risks, self.run_id, self.run_results, self.sorted_results_by_time_frame, self.sorted_results_through_all_time_frame, self.strategy_class, self.tentacles_setup_config, self.total_nb_runs, self.trading_mode)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->is_computing); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->is_properly_initialized); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->run_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_self->total_nb_runs); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(17); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_self->all_TAs);
  __Pyx_GIVEREF(__pyx_v_self->all_TAs);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_self->all_TAs);
  __Pyx_INCREF(__pyx_v_self->all_time_frames);
  __Pyx_GIVEREF(__pyx_v_self->all_time_frames);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_self->all_time_frames);
  __Pyx_INCREF(__pyx_v_self->config);
  __Pyx_GIVEREF(__pyx_v_self->config);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_self->config);
  __Pyx_INCREF(__pyx_v_self->current_test_suite);
  __Pyx_GIVEREF(__pyx_v_self->current_test_suite);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_v_self->current_test_suite);
  __Pyx_INCREF(__pyx_v_self->errors);
  __Pyx_GIVEREF(__pyx_v_self->errors);
  PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_v_self->errors);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 5, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 6, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->logger);
  __Pyx_GIVEREF(__pyx_v_self->logger);
  PyTuple_SET_ITEM(__pyx_t_5, 7, __pyx_v_self->logger);
  __Pyx_INCREF(__pyx_v_self->risks);
  __Pyx_GIVEREF(__pyx_v_self->risks);
  PyTuple_SET_ITEM(__pyx_t_5, 8, __pyx_v_self->risks);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 9, __pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->run_results);
  __Pyx_GIVEREF(__pyx_v_self->run_results);
  PyTuple_SET_ITEM(__pyx_t_5, 10, __pyx_v_self->run_results);
  __Pyx_INCREF(__pyx_v_self->sorted_results_by_time_frame);
  __Pyx_GIVEREF(__pyx_v_self->sorted_results_by_time_frame);
  PyTuple_SET_ITEM(__pyx_t_5, 11, __pyx_v_self->sorted_results_by_time_frame);
  __Pyx_INCREF(__pyx_v_self->sorted_results_through_all_time_frame);
  __Pyx_GIVEREF(__pyx_v_self->sorted_results_through_all_time_frame);
  PyTuple_SET_ITEM(__pyx_t_5, 12, __pyx_v_self->sorted_results_through_all_time_frame);
  __Pyx_INCREF(__pyx_v_self->strategy_class);
  __Pyx_GIVEREF(__pyx_v_self->strategy_class);
  PyTuple_SET_ITEM(__pyx_t_5, 13, __pyx_v_self->strategy_class);
  __Pyx_INCREF(__pyx_v_self->tentacles_setup_config);
  __Pyx_GIVEREF(__pyx_v_self->tentacles_setup_config);
  PyTuple_SET_ITEM(__pyx_t_5, 14, __pyx_v_self->tentacles_setup_config);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 15, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->trading_mode);
  __Pyx_GIVEREF(__pyx_v_self->trading_mode);
  PyTuple_SET_ITEM(__pyx_t_5, 16, __pyx_v_self->trading_mode);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.all_TAs, self.all_time_frames, self.config, self.current_test_suite, self.errors, self.is_computing, self.is_properly_initialized, self.logger, self.risks, self.run_id, self.run_results, self.sorted_results_by_time_frame, self.sorted_results_through_all_time_frame, self.strategy_class, self.tentacles_setup_config, self.total_nb_runs, self.trading_mode)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_5 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v__dict = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "(tree fragment)":7
 *     state = (self.all_TAs, self.all_time_frames, self.config, self.current_test_suite, self.errors, self.is_computing, self.is_properly_initialized, self.logger, self.risks, self.run_id, self.run_results, self.sorted_results_by_time_frame, self.sorted_results_through_all_time_frame, self.strategy_class, self.tentacles_setup_config, self.total_nb_runs, self.trading_mode)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  __pyx_t_7 = (__pyx_t_6 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.all_TAs is not None or self.all_time_frames is not None or self.config is not None or self.current_test_suite is not None or self.errors is not None or self.logger is not None or self.risks is not None or self.run_results is not None or self.sorted_results_by_time_frame is not None or self.sorted_results_through_all_time_frame is not None or self.strategy_class is not None or self.tentacles_setup_config is not None or self.trading_mode is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.all_TAs, self.all_time_frames, self.config, self.current_test_suite, self.errors, self.is_computing, self.is_properly_initialized, self.logger, self.risks, self.run_id, self.run_results, self.sorted_results_by_time_frame, self.sorted_results_through_all_time_frame, self.strategy_class, self.tentacles_setup_config, self.total_nb_runs, self.trading_mode)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.all_TAs is not None or self.all_time_frames is not None or self.config is not None or self.current_test_suite is not None or self.errors is not None or self.logger is not None or self.risks is not None or self.run_results is not None or self.sorted_results_by_time_frame is not None or self.sorted_results_through_all_time_frame is not None or self.strategy_class is not None or self.tentacles_setup_config is not None or self.trading_mode is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_StrategyOptimizer, (type(self), 0x2d2eb06, None), state
 */
  /*else*/ {
    __pyx_t_6 = (__pyx_v_self->all_TAs != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->all_time_frames != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_8 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->config != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->current_test_suite != Py_None);
    __pyx_t_6 = (__pyx_t_8 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->errors != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->logger != Py_None);
    __pyx_t_6 = (__pyx_t_8 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->risks != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->run_results != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_8 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->sorted_results_by_time_frame != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->sorted_results_through_all_time_frame != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_8 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->strategy_class != Py_None);
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->tentacles_setup_config != Py_None);
    __pyx_t_6 = (__pyx_t_8 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->trading_mode != Py_None);
    __pyx_t_8 = (__pyx_t_6 != 0);
    __pyx_t_7 = __pyx_t_8;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_7;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.all_TAs is not None or self.all_time_frames is not None or self.config is not None or self.current_test_suite is not None or self.errors is not None or self.logger is not None or self.risks is not None or self.run_results is not None or self.sorted_results_by_time_frame is not None or self.sorted_results_through_all_time_frame is not None or self.strategy_class is not None or self.tentacles_setup_config is not None or self.trading_mode is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_StrategyOptimizer, (type(self), 0x2d2eb06, None), state
 *     else:
 */
  __pyx_t_7 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":13
 *         use_setstate = self.all_TAs is not None or self.all_time_frames is not None or self.config is not None or self.current_test_suite is not None or self.errors is not None or self.logger is not None or self.risks is not None or self.run_results is not None or self.sorted_results_by_time_frame is not None or self.sorted_results_through_all_time_frame is not None or self.strategy_class is not None or self.tentacles_setup_config is not None or self.trading_mode is not None
 *     if use_setstate:
 *         return __pyx_unpickle_StrategyOptimizer, (type(self), 0x2d2eb06, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_StrategyOptimizer, (type(self), 0x2d2eb06, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_StrategyOptimizer); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_47377158);
    __Pyx_GIVEREF(__pyx_int_47377158);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_47377158);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_5, 2, Py_None);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_5);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.all_TAs is not None or self.all_time_frames is not None or self.config is not None or self.current_test_suite is not None or self.errors is not None or self.logger is not None or self.risks is not None or self.run_results is not None or self.sorted_results_by_time_frame is not None or self.sorted_results_through_all_time_frame is not None or self.strategy_class is not None or self.tentacles_setup_config is not None or self.trading_mode is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_StrategyOptimizer, (type(self), 0x2d2eb06, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_StrategyOptimizer, (type(self), 0x2d2eb06, None), state
 *     else:
 *         return __pyx_unpickle_StrategyOptimizer, (type(self), 0x2d2eb06, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_StrategyOptimizer__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_StrategyOptimizer); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_47377158);
    __Pyx_GIVEREF(__pyx_int_47377158);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_47377158);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_StrategyOptimizer, (type(self), 0x2d2eb06, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_StrategyOptimizer__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_29__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_29__setstate_cython__ = {"__setstate_cython__", (PyCFunction)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_29__setstate_cython__, METH_O, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_29__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_28__setstate_cython__(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_28__setstate_cython__(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_StrategyOptimizer, (type(self), 0x2d2eb06, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_StrategyOptimizer__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer___pyx_unpickle_StrategyOptimizer__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_StrategyOptimizer, (type(self), 0x2d2eb06, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_StrategyOptimizer__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_StrategyOptimizer(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_1__pyx_unpickle_StrategyOptimizer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_1__pyx_unpickle_StrategyOptimizer = {"__pyx_unpickle_StrategyOptimizer", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_1__pyx_unpickle_StrategyOptimizer, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_1__pyx_unpickle_StrategyOptimizer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_StrategyOptimizer (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_StrategyOptimizer", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_StrategyOptimizer", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_StrategyOptimizer") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_StrategyOptimizer", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.__pyx_unpickle_StrategyOptimizer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer___pyx_unpickle_StrategyOptimizer(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7octobot_18strategy_optimizer_18strategy_optimizer___pyx_unpickle_StrategyOptimizer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__pyx_unpickle_StrategyOptimizer", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x2d2eb06:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2d2eb06 = (all_TAs, all_time_frames, config, current_test_suite, errors, is_computing, is_properly_initialized, logger, risks, run_id, run_results, sorted_results_by_time_frame, sorted_results_through_all_time_frame, strategy_class, tentacles_setup_config, total_nb_runs, trading_mode))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x2d2eb06) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x2d2eb06:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2d2eb06 = (all_TAs, all_time_frames, config, current_test_suite, errors, is_computing, is_properly_initialized, logger, risks, run_id, run_results, sorted_results_by_time_frame, sorted_results_through_all_time_frame, strategy_class, tentacles_setup_config, total_nb_runs, trading_mode))" % __pyx_checksum)
 *     __pyx_result = StrategyOptimizer.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x2d2eb06:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2d2eb06 = (all_TAs, all_time_frames, config, current_test_suite, errors, is_computing, is_properly_initialized, logger, risks, run_id, run_results, sorted_results_by_time_frame, sorted_results_through_all_time_frame, strategy_class, tentacles_setup_config, total_nb_runs, trading_mode))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = StrategyOptimizer.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x2d, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x2d2eb06:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2d2eb06 = (all_TAs, all_time_frames, config, current_test_suite, errors, is_computing, is_properly_initialized, logger, risks, run_id, run_results, sorted_results_by_time_frame, sorted_results_through_all_time_frame, strategy_class, tentacles_setup_config, total_nb_runs, trading_mode))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2d2eb06 = (all_TAs, all_time_frames, config, current_test_suite, errors, is_computing, is_properly_initialized, logger, risks, run_id, run_results, sorted_results_by_time_frame, sorted_results_through_all_time_frame, strategy_class, tentacles_setup_config, total_nb_runs, trading_mode))" % __pyx_checksum)
 *     __pyx_result = StrategyOptimizer.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_StrategyOptimizer__set_state(<StrategyOptimizer> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2d2eb06 = (all_TAs, all_time_frames, config, current_test_suite, errors, is_computing, is_properly_initialized, logger, risks, run_id, run_results, sorted_results_by_time_frame, sorted_results_through_all_time_frame, strategy_class, tentacles_setup_config, total_nb_runs, trading_mode))" % __pyx_checksum)
 *     __pyx_result = StrategyOptimizer.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_StrategyOptimizer__set_state(<StrategyOptimizer> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = StrategyOptimizer.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_StrategyOptimizer__set_state(<StrategyOptimizer> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_StrategyOptimizer__set_state(StrategyOptimizer __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer___pyx_unpickle_StrategyOptimizer__set_state(((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x2d2eb06 = (all_TAs, all_time_frames, config, current_test_suite, errors, is_computing, is_properly_initialized, logger, risks, run_id, run_results, sorted_results_by_time_frame, sorted_results_through_all_time_frame, strategy_class, tentacles_setup_config, total_nb_runs, trading_mode))" % __pyx_checksum)
 *     __pyx_result = StrategyOptimizer.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_StrategyOptimizer__set_state(<StrategyOptimizer> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_StrategyOptimizer__set_state(<StrategyOptimizer> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_StrategyOptimizer__set_state(StrategyOptimizer __pyx_result, tuple __pyx_state):
 *     __pyx_result.all_TAs = __pyx_state[0]; __pyx_result.all_time_frames = __pyx_state[1]; __pyx_result.config = __pyx_state[2]; __pyx_result.current_test_suite = __pyx_state[3]; __pyx_result.errors = __pyx_state[4]; __pyx_result.is_computing = __pyx_state[5]; __pyx_result.is_properly_initialized = __pyx_state[6]; __pyx_result.logger = __pyx_state[7]; __pyx_result.risks = __pyx_state[8]; __pyx_result.run_id = __pyx_state[9]; __pyx_result.run_results = __pyx_state[10]; __pyx_result.sorted_results_by_time_frame = __pyx_state[11]; __pyx_result.sorted_results_through_all_time_frame = __pyx_state[12]; __pyx_result.strategy_class = __pyx_state[13]; __pyx_result.tentacles_setup_config = __pyx_state[14]; __pyx_result.total_nb_runs = __pyx_state[15]; __pyx_result.trading_mode = __pyx_state[16]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_StrategyOptimizer(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.__pyx_unpickle_StrategyOptimizer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_StrategyOptimizer__set_state(<StrategyOptimizer> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_StrategyOptimizer__set_state(StrategyOptimizer __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.all_TAs = __pyx_state[0]; __pyx_result.all_time_frames = __pyx_state[1]; __pyx_result.config = __pyx_state[2]; __pyx_result.current_test_suite = __pyx_state[3]; __pyx_result.errors = __pyx_state[4]; __pyx_result.is_computing = __pyx_state[5]; __pyx_result.is_properly_initialized = __pyx_state[6]; __pyx_result.logger = __pyx_state[7]; __pyx_result.risks = __pyx_state[8]; __pyx_result.run_id = __pyx_state[9]; __pyx_result.run_results = __pyx_state[10]; __pyx_result.sorted_results_by_time_frame = __pyx_state[11]; __pyx_result.sorted_results_through_all_time_frame = __pyx_state[12]; __pyx_result.strategy_class = __pyx_state[13]; __pyx_result.tentacles_setup_config = __pyx_state[14]; __pyx_result.total_nb_runs = __pyx_state[15]; __pyx_result.trading_mode = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer___pyx_unpickle_StrategyOptimizer__set_state(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_StrategyOptimizer__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_StrategyOptimizer__set_state(StrategyOptimizer __pyx_result, tuple __pyx_state):
 *     __pyx_result.all_TAs = __pyx_state[0]; __pyx_result.all_time_frames = __pyx_state[1]; __pyx_result.config = __pyx_state[2]; __pyx_result.current_test_suite = __pyx_state[3]; __pyx_result.errors = __pyx_state[4]; __pyx_result.is_computing = __pyx_state[5]; __pyx_result.is_properly_initialized = __pyx_state[6]; __pyx_result.logger = __pyx_state[7]; __pyx_result.risks = __pyx_state[8]; __pyx_result.run_id = __pyx_state[9]; __pyx_result.run_results = __pyx_state[10]; __pyx_result.sorted_results_by_time_frame = __pyx_state[11]; __pyx_result.sorted_results_through_all_time_frame = __pyx_state[12]; __pyx_result.strategy_class = __pyx_state[13]; __pyx_result.tentacles_setup_config = __pyx_state[14]; __pyx_result.total_nb_runs = __pyx_state[15]; __pyx_result.trading_mode = __pyx_state[16]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->all_TAs);
  __Pyx_DECREF(__pyx_v___pyx_result->all_TAs);
  __pyx_v___pyx_result->all_TAs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->all_time_frames);
  __Pyx_DECREF(__pyx_v___pyx_result->all_time_frames);
  __pyx_v___pyx_result->all_time_frames = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->config);
  __Pyx_DECREF(__pyx_v___pyx_result->config);
  __pyx_v___pyx_result->config = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->current_test_suite);
  __Pyx_DECREF(__pyx_v___pyx_result->current_test_suite);
  __pyx_v___pyx_result->current_test_suite = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PySet_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "set", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->errors);
  __Pyx_DECREF(__pyx_v___pyx_result->errors);
  __pyx_v___pyx_result->errors = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->is_computing = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->is_properly_initialized = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->logger);
  __Pyx_DECREF(__pyx_v___pyx_result->logger);
  __pyx_v___pyx_result->logger = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->risks);
  __Pyx_DECREF(__pyx_v___pyx_result->risks);
  __pyx_v___pyx_result->risks = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->run_id = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->run_results);
  __Pyx_DECREF(__pyx_v___pyx_result->run_results);
  __pyx_v___pyx_result->run_results = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->sorted_results_by_time_frame);
  __Pyx_DECREF(__pyx_v___pyx_result->sorted_results_by_time_frame);
  __pyx_v___pyx_result->sorted_results_by_time_frame = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->sorted_results_through_all_time_frame);
  __Pyx_DECREF(__pyx_v___pyx_result->sorted_results_through_all_time_frame);
  __pyx_v___pyx_result->sorted_results_through_all_time_frame = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->strategy_class);
  __Pyx_DECREF(__pyx_v___pyx_result->strategy_class);
  __pyx_v___pyx_result->strategy_class = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->tentacles_setup_config);
  __Pyx_DECREF(__pyx_v___pyx_result->tentacles_setup_config);
  __pyx_v___pyx_result->tentacles_setup_config = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->total_nb_runs = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->trading_mode);
  __Pyx_DECREF(__pyx_v___pyx_result->trading_mode);
  __pyx_v___pyx_result->trading_mode = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_StrategyOptimizer__set_state(StrategyOptimizer __pyx_result, tuple __pyx_state):
 *     __pyx_result.all_TAs = __pyx_state[0]; __pyx_result.all_time_frames = __pyx_state[1]; __pyx_result.config = __pyx_state[2]; __pyx_result.current_test_suite = __pyx_state[3]; __pyx_result.errors = __pyx_state[4]; __pyx_result.is_computing = __pyx_state[5]; __pyx_result.is_properly_initialized = __pyx_state[6]; __pyx_result.logger = __pyx_state[7]; __pyx_result.risks = __pyx_state[8]; __pyx_result.run_id = __pyx_state[9]; __pyx_result.run_results = __pyx_state[10]; __pyx_result.sorted_results_by_time_frame = __pyx_state[11]; __pyx_result.sorted_results_through_all_time_frame = __pyx_state[12]; __pyx_result.strategy_class = __pyx_state[13]; __pyx_result.tentacles_setup_config = __pyx_state[14]; __pyx_result.total_nb_runs = __pyx_state[15]; __pyx_result.trading_mode = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 17) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.all_TAs = __pyx_state[0]; __pyx_result.all_time_frames = __pyx_state[1]; __pyx_result.config = __pyx_state[2]; __pyx_result.current_test_suite = __pyx_state[3]; __pyx_result.errors = __pyx_state[4]; __pyx_result.is_computing = __pyx_state[5]; __pyx_result.is_properly_initialized = __pyx_state[6]; __pyx_result.logger = __pyx_state[7]; __pyx_result.risks = __pyx_state[8]; __pyx_result.run_id = __pyx_state[9]; __pyx_result.run_results = __pyx_state[10]; __pyx_result.sorted_results_by_time_frame = __pyx_state[11]; __pyx_result.sorted_results_through_all_time_frame = __pyx_state[12]; __pyx_result.strategy_class = __pyx_state[13]; __pyx_result.tentacles_setup_config = __pyx_state[14]; __pyx_result.total_nb_runs = __pyx_state[15]; __pyx_result.trading_mode = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[17])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_StrategyOptimizer__set_state(StrategyOptimizer __pyx_result, tuple __pyx_state):
 *     __pyx_result.all_TAs = __pyx_state[0]; __pyx_result.all_time_frames = __pyx_state[1]; __pyx_result.config = __pyx_state[2]; __pyx_result.current_test_suite = __pyx_state[3]; __pyx_result.errors = __pyx_state[4]; __pyx_result.is_computing = __pyx_state[5]; __pyx_result.is_properly_initialized = __pyx_state[6]; __pyx_result.logger = __pyx_state[7]; __pyx_result.risks = __pyx_state[8]; __pyx_result.run_id = __pyx_state[9]; __pyx_result.run_results = __pyx_state[10]; __pyx_result.sorted_results_by_time_frame = __pyx_state[11]; __pyx_result.sorted_results_through_all_time_frame = __pyx_state[12]; __pyx_result.strategy_class = __pyx_state[13]; __pyx_result.tentacles_setup_config = __pyx_state[14]; __pyx_result.total_nb_runs = __pyx_state[15]; __pyx_result.trading_mode = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_StrategyOptimizer__set_state(<StrategyOptimizer> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_StrategyOptimizer__set_state(StrategyOptimizer __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.all_TAs = __pyx_state[0]; __pyx_result.all_time_frames = __pyx_state[1]; __pyx_result.config = __pyx_state[2]; __pyx_result.current_test_suite = __pyx_state[3]; __pyx_result.errors = __pyx_state[4]; __pyx_result.is_computing = __pyx_state[5]; __pyx_result.is_properly_initialized = __pyx_state[6]; __pyx_result.logger = __pyx_state[7]; __pyx_result.risks = __pyx_state[8]; __pyx_result.run_id = __pyx_state[9]; __pyx_result.run_results = __pyx_state[10]; __pyx_result.sorted_results_by_time_frame = __pyx_state[11]; __pyx_result.sorted_results_through_all_time_frame = __pyx_state[12]; __pyx_result.strategy_class = __pyx_state[13]; __pyx_result.tentacles_setup_config = __pyx_state[14]; __pyx_result.total_nb_runs = __pyx_state[15]; __pyx_result.trading_mode = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("octobot.strategy_optimizer.strategy_optimizer.__pyx_unpickle_StrategyOptimizer__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer __pyx_vtable_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer;

static PyObject *__pyx_tp_new_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)o);
  p->__pyx_vtab = __pyx_vtabptr_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer;
  p->logger = Py_None; Py_INCREF(Py_None);
  p->config = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->tentacles_setup_config = Py_None; Py_INCREF(Py_None);
  p->sorted_results_by_time_frame = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->sorted_results_through_all_time_frame = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->current_test_suite = Py_None; Py_INCREF(Py_None);
  p->errors = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->trading_mode = Py_None; Py_INCREF(Py_None);
  p->strategy_class = Py_None; Py_INCREF(Py_None);
  p->all_time_frames = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->all_TAs = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->risks = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->run_results = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer(PyObject *o) {
  struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *p = (struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->logger);
  Py_CLEAR(p->config);
  Py_CLEAR(p->tentacles_setup_config);
  Py_CLEAR(p->sorted_results_by_time_frame);
  Py_CLEAR(p->sorted_results_through_all_time_frame);
  Py_CLEAR(p->current_test_suite);
  Py_CLEAR(p->errors);
  Py_CLEAR(p->trading_mode);
  Py_CLEAR(p->strategy_class);
  Py_CLEAR(p->all_time_frames);
  Py_CLEAR(p->all_TAs);
  Py_CLEAR(p->risks);
  Py_CLEAR(p->run_results);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *p = (struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)o;
  if (p->logger) {
    e = (*v)(p->logger, a); if (e) return e;
  }
  if (p->config) {
    e = (*v)(p->config, a); if (e) return e;
  }
  if (p->tentacles_setup_config) {
    e = (*v)(p->tentacles_setup_config, a); if (e) return e;
  }
  if (p->sorted_results_by_time_frame) {
    e = (*v)(p->sorted_results_by_time_frame, a); if (e) return e;
  }
  if (p->sorted_results_through_all_time_frame) {
    e = (*v)(p->sorted_results_through_all_time_frame, a); if (e) return e;
  }
  if (p->current_test_suite) {
    e = (*v)(p->current_test_suite, a); if (e) return e;
  }
  if (p->errors) {
    e = (*v)(p->errors, a); if (e) return e;
  }
  if (p->trading_mode) {
    e = (*v)(p->trading_mode, a); if (e) return e;
  }
  if (p->strategy_class) {
    e = (*v)(p->strategy_class, a); if (e) return e;
  }
  if (p->all_time_frames) {
    e = (*v)(p->all_time_frames, a); if (e) return e;
  }
  if (p->all_TAs) {
    e = (*v)(p->all_TAs, a); if (e) return e;
  }
  if (p->risks) {
    e = (*v)(p->risks, a); if (e) return e;
  }
  if (p->run_results) {
    e = (*v)(p->run_results, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *p = (struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *)o;
  tmp = ((PyObject*)p->logger);
  p->logger = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->config);
  p->config = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->tentacles_setup_config);
  p->tentacles_setup_config = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->sorted_results_by_time_frame);
  p->sorted_results_by_time_frame = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->sorted_results_through_all_time_frame);
  p->sorted_results_through_all_time_frame = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->current_test_suite);
  p->current_test_suite = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->errors);
  p->errors = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->trading_mode);
  p->trading_mode = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->strategy_class);
  p->strategy_class = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->all_time_frames);
  p->all_time_frames = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->all_TAs);
  p->all_TAs = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->risks);
  p->risks = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->run_results);
  p->run_results = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_is_properly_initialized(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23is_properly_initialized_1__get__(o);
}

static int __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_is_properly_initialized(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23is_properly_initialized_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_trading_mode(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode_1__get__(o);
}

static int __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_trading_mode(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode_3__set__(o, v);
  }
  else {
    return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12trading_mode_5__del__(o);
  }
}

static PyObject *__pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_strategy_class(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class_1__get__(o);
}

static int __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_strategy_class(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class_3__set__(o, v);
  }
  else {
    return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_14strategy_class_5__del__(o);
  }
}

static PyObject *__pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_all_time_frames(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames_1__get__(o);
}

static int __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_all_time_frames(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames_3__set__(o, v);
  }
  else {
    return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15all_time_frames_5__del__(o);
  }
}

static PyObject *__pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_all_TAs(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs_1__get__(o);
}

static int __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_all_TAs(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs_3__set__(o, v);
  }
  else {
    return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7all_TAs_5__del__(o);
  }
}

static PyObject *__pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_risks(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks_1__get__(o);
}

static int __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_risks(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks_3__set__(o, v);
  }
  else {
    return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5risks_5__del__(o);
  }
}

static PyObject *__pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_is_computing(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12is_computing_1__get__(o);
}

static int __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_is_computing(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_12is_computing_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_run_results(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results_1__get__(o);
}

static int __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_run_results(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results_3__set__(o, v);
  }
  else {
    return __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11run_results_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer[] = {
  {"get_name", (PyCFunction)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_17get_name, METH_NOARGS, 0},
  {"_get_activated_element", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_19_get_activated_element, METH_VARARGS|METH_KEYWORDS, 0},
  {"_get_filtered_results", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_21_get_filtered_results, METH_VARARGS|METH_KEYWORDS, 0},
  {"_get_sorted_results", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23_get_sorted_results, METH_VARARGS|METH_KEYWORDS, 0},
  {"_is_relevant_evaluation_config", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_25_is_relevant_evaluation_config, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_27__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_29__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer[] = {
  {(char *)"is_properly_initialized", __pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_is_properly_initialized, __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_is_properly_initialized, (char *)0, 0},
  {(char *)"trading_mode", __pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_trading_mode, __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_trading_mode, (char *)0, 0},
  {(char *)"strategy_class", __pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_strategy_class, __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_strategy_class, (char *)0, 0},
  {(char *)"all_time_frames", __pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_all_time_frames, __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_all_time_frames, (char *)0, 0},
  {(char *)"all_TAs", __pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_all_TAs, __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_all_TAs, (char *)0, 0},
  {(char *)"risks", __pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_risks, __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_risks, (char *)0, 0},
  {(char *)"is_computing", __pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_is_computing, __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_is_computing, (char *)0, 0},
  {(char *)"run_results", __pyx_getprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_run_results, __pyx_setprop_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_run_results, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer = {
  PyVarObject_HEAD_INIT(0, 0)
  "octobot.strategy_optimizer.strategy_optimizer.StrategyOptimizer", /*tp_name*/
  sizeof(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    StrategyOptimizer is a tool that performs backtesting with different configurations\n    ", /*tp_doc*/
  __pyx_tp_traverse_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer, /*tp_traverse*/
  __pyx_tp_clear_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_strategy_optimizer(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_strategy_optimizer},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "strategy_optimizer",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_u_, __pyx_k_, sizeof(__pyx_k_), 0, 1, 0, 0},
  {&__pyx_n_s_AbstractTradingMode, __pyx_k_AbstractTradingMode, sizeof(__pyx_k_AbstractTradingMode), 0, 0, 1, 1},
  {&__pyx_n_s_CONFIG, __pyx_k_CONFIG, sizeof(__pyx_k_CONFIG), 0, 0, 1, 1},
  {&__pyx_n_s_CONFIG_FORCED_TIME_FRAME, __pyx_k_CONFIG_FORCED_TIME_FRAME, sizeof(__pyx_k_CONFIG_FORCED_TIME_FRAME), 0, 0, 1, 1},
  {&__pyx_n_s_CONFIG_TRADER_RISK, __pyx_k_CONFIG_TRADER_RISK, sizeof(__pyx_k_CONFIG_TRADER_RISK), 0, 0, 1, 1},
  {&__pyx_n_s_CONFIG_TRADING, __pyx_k_CONFIG_TRADING, sizeof(__pyx_k_CONFIG_TRADING), 0, 0, 1, 1},
  {&__pyx_n_s_ERROR, __pyx_k_ERROR, sizeof(__pyx_k_ERROR), 0, 0, 1, 1},
  {&__pyx_kp_u_Full_execution_sorted_results_Mi, __pyx_k_Full_execution_sorted_results_Mi, sizeof(__pyx_k_Full_execution_sorted_results_Mi), 0, 1, 0, 0},
  {&__pyx_kp_u_Impossible_to_find_a_strategy_ma, __pyx_k_Impossible_to_find_a_strategy_ma, sizeof(__pyx_k_Impossible_to_find_a_strategy_ma), 0, 1, 0, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x2d, __pyx_k_Incompatible_checksums_s_vs_0x2d, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x2d), 0, 0, 1, 0},
  {&__pyx_kp_u_Logging_level_restored, __pyx_k_Logging_level_restored, sizeof(__pyx_k_Logging_level_restored), 0, 1, 0, 0},
  {&__pyx_n_s_Mode, __pyx_k_Mode, sizeof(__pyx_k_Mode), 0, 0, 1, 1},
  {&__pyx_n_s_OPTIMIZER_FORCE_ASYNCIO_DEBUG_OP, __pyx_k_OPTIMIZER_FORCE_ASYNCIO_DEBUG_OP, sizeof(__pyx_k_OPTIMIZER_FORCE_ASYNCIO_DEBUG_OP), 0, 0, 1, 1},
  {&__pyx_kp_u_Overall_best_configuration_for, __pyx_k_Overall_best_configuration_for, sizeof(__pyx_k_Overall_best_configuration_for), 0, 1, 0, 0},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_RANK, __pyx_k_RANK, sizeof(__pyx_k_RANK), 0, 0, 1, 1},
  {&__pyx_kp_u_Result, __pyx_k_Result, sizeof(__pyx_k_Result), 0, 1, 0, 0},
  {&__pyx_kp_u_Run_with_evaluators, __pyx_k_Run_with_evaluators, sizeof(__pyx_k_Run_with_evaluators), 0, 1, 0, 0},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_kp_u_Setting_logging_level_to_logging, __pyx_k_Setting_logging_level_to_logging, sizeof(__pyx_k_Setting_logging_level_to_logging), 0, 1, 0, 0},
  {&__pyx_n_s_Strategies, __pyx_k_Strategies, sizeof(__pyx_k_Strategies), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyEvaluator, __pyx_k_StrategyEvaluator, sizeof(__pyx_k_StrategyEvaluator), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer, __pyx_k_StrategyOptimizer, sizeof(__pyx_k_StrategyOptimizer), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer___reduce_cytho, __pyx_k_StrategyOptimizer___reduce_cytho, sizeof(__pyx_k_StrategyOptimizer___reduce_cytho), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer___setstate_cyt, __pyx_k_StrategyOptimizer___setstate_cyt, sizeof(__pyx_k_StrategyOptimizer___setstate_cyt), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer__find_optimal, __pyx_k_StrategyOptimizer__find_optimal, sizeof(__pyx_k_StrategyOptimizer__find_optimal), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer__get_activated, __pyx_k_StrategyOptimizer__get_activated, sizeof(__pyx_k_StrategyOptimizer__get_activated), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer__get_filtered, __pyx_k_StrategyOptimizer__get_filtered, sizeof(__pyx_k_StrategyOptimizer__get_filtered), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer__get_sorted_re, __pyx_k_StrategyOptimizer__get_sorted_re, sizeof(__pyx_k_StrategyOptimizer__get_sorted_re), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer__is_relevant_e, __pyx_k_StrategyOptimizer__is_relevant_e, sizeof(__pyx_k_StrategyOptimizer__is_relevant_e), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer_find_optimal_c, __pyx_k_StrategyOptimizer_find_optimal_c, sizeof(__pyx_k_StrategyOptimizer_find_optimal_c), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer_get_current_te, __pyx_k_StrategyOptimizer_get_current_te, sizeof(__pyx_k_StrategyOptimizer_get_current_te), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer_get_errors_des, __pyx_k_StrategyOptimizer_get_errors_des, sizeof(__pyx_k_StrategyOptimizer_get_errors_des), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer_get_name, __pyx_k_StrategyOptimizer_get_name, sizeof(__pyx_k_StrategyOptimizer_get_name), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer_get_overall_pr, __pyx_k_StrategyOptimizer_get_overall_pr, sizeof(__pyx_k_StrategyOptimizer_get_overall_pr), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer_get_report, __pyx_k_StrategyOptimizer_get_report, sizeof(__pyx_k_StrategyOptimizer_get_report), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer_is_in_progress, __pyx_k_StrategyOptimizer_is_in_progress, sizeof(__pyx_k_StrategyOptimizer_is_in_progress), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyOptimizer_print_report, __pyx_k_StrategyOptimizer_print_report, sizeof(__pyx_k_StrategyOptimizer_print_report), 0, 0, 1, 1},
  {&__pyx_n_s_StrategyTestSuite, __pyx_k_StrategyTestSuite, sizeof(__pyx_k_StrategyTestSuite), 0, 0, 1, 1},
  {&__pyx_n_s_TA, __pyx_k_TA, sizeof(__pyx_k_TA), 0, 0, 1, 1},
  {&__pyx_n_s_TAEvaluator, __pyx_k_TAEvaluator, sizeof(__pyx_k_TAEvaluator), 0, 0, 1, 1},
  {&__pyx_n_s_TAs, __pyx_k_TAs, sizeof(__pyx_k_TAs), 0, 0, 1, 1},
  {&__pyx_n_s_TRADES, __pyx_k_TRADES, sizeof(__pyx_k_TRADES), 0, 0, 1, 1},
  {&__pyx_n_s_TRADES_IN_RESULT, __pyx_k_TRADES_IN_RESULT, sizeof(__pyx_k_TRADES_IN_RESULT), 0, 0, 1, 1},
  {&__pyx_n_s_TestSuiteResult, __pyx_k_TestSuiteResult, sizeof(__pyx_k_TestSuiteResult), 0, 0, 1, 1},
  {&__pyx_kp_u_Top_rankings_per_time_frame, __pyx_k_Top_rankings_per_time_frame, sizeof(__pyx_k_Top_rankings_per_time_frame), 0, 1, 0, 0},
  {&__pyx_kp_u_Top_rankings_through_all_time_f, __pyx_k_Top_rankings_through_all_time_f, sizeof(__pyx_k_Top_rankings_through_all_time_f), 0, 1, 0, 0},
  {&__pyx_kp_u_Trying_to_find_an_optimized_conf, __pyx_k_Trying_to_find_an_optimized_conf, sizeof(__pyx_k_Trying_to_find_an_optimized_conf), 0, 1, 0, 0},
  {&__pyx_kp_u__4, __pyx_k__4, sizeof(__pyx_k__4), 0, 1, 0, 0},
  {&__pyx_kp_u__5, __pyx_k__5, sizeof(__pyx_k__5), 0, 1, 0, 0},
  {&__pyx_kp_u__6, __pyx_k__6, sizeof(__pyx_k__6), 0, 1, 0, 0},
  {&__pyx_kp_u__7, __pyx_k__7, sizeof(__pyx_k__7), 0, 1, 0, 0},
  {&__pyx_kp_u__8, __pyx_k__8, sizeof(__pyx_k__8), 0, 1, 0, 0},
  {&__pyx_n_s_additional_elements_count, __pyx_k_additional_elements_count, sizeof(__pyx_k_additional_elements_count), 0, 0, 1, 1},
  {&__pyx_n_s_all_elements, __pyx_k_all_elements, sizeof(__pyx_k_all_elements), 0, 0, 1, 1},
  {&__pyx_n_s_append, __pyx_k_append, sizeof(__pyx_k_append), 0, 0, 1, 1},
  {&__pyx_n_s_asyncio, __pyx_k_asyncio, sizeof(__pyx_k_asyncio), 0, 0, 1, 1},
  {&__pyx_n_s_asyncio_coroutines, __pyx_k_asyncio_coroutines, sizeof(__pyx_k_asyncio_coroutines), 0, 0, 1, 1},
  {&__pyx_n_s_asyncio_tasks, __pyx_k_asyncio_tasks, sizeof(__pyx_k_asyncio_tasks), 0, 0, 1, 1},
  {&__pyx_kp_u_average_trades_count, __pyx_k_average_trades_count, sizeof(__pyx_k_average_trades_count), 0, 1, 0, 0},
  {&__pyx_kp_u_average_trades_count_2, __pyx_k_average_trades_count_2, sizeof(__pyx_k_average_trades_count_2), 0, 1, 0, 0},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_cls, __pyx_k_cls, sizeof(__pyx_k_cls), 0, 0, 1, 1},
  {&__pyx_n_s_config, __pyx_k_config, sizeof(__pyx_k_config), 0, 0, 1, 1},
  {&__pyx_n_s_convert_result_into_dict, __pyx_k_convert_result_into_dict, sizeof(__pyx_k_convert_result_into_dict), 0, 0, 1, 1},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_n_s_create_evaluator_classes, __pyx_k_create_evaluator_classes, sizeof(__pyx_k_create_evaluator_classes), 0, 0, 1, 1},
  {&__pyx_n_s_current_elem, __pyx_k_current_elem, sizeof(__pyx_k_current_elem), 0, 0, 1, 1},
  {&__pyx_n_s_current_forced_element, __pyx_k_current_forced_element, sizeof(__pyx_k_current_forced_element), 0, 0, 1, 1},
  {&__pyx_n_s_current_progress, __pyx_k_current_progress, sizeof(__pyx_k_current_progress), 0, 0, 1, 1},
  {&__pyx_n_s_debug, __pyx_k_debug, sizeof(__pyx_k_debug), 0, 0, 1, 1},
  {&__pyx_n_s_deepcopy, __pyx_k_deepcopy, sizeof(__pyx_k_deepcopy), 0, 0, 1, 1},
  {&__pyx_n_s_default_element, __pyx_k_default_element, sizeof(__pyx_k_default_element), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_dict_2, __pyx_k_dict_2, sizeof(__pyx_k_dict_2), 0, 0, 1, 1},
  {&__pyx_n_s_dict_shaped, __pyx_k_dict_shaped, sizeof(__pyx_k_dict_shaped), 0, 0, 1, 1},
  {&__pyx_n_s_elem, __pyx_k_elem, sizeof(__pyx_k_elem), 0, 0, 1, 1},
  {&__pyx_n_s_elem_conf_history, __pyx_k_elem_conf_history, sizeof(__pyx_k_elem_conf_history), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_error, __pyx_k_error, sizeof(__pyx_k_error), 0, 0, 1, 1},
  {&__pyx_n_s_eval_conf, __pyx_k_eval_conf, sizeof(__pyx_k_eval_conf), 0, 0, 1, 1},
  {&__pyx_n_s_evaluator, __pyx_k_evaluator, sizeof(__pyx_k_evaluator), 0, 0, 1, 1},
  {&__pyx_n_s_evaluator_parent_inspection, __pyx_k_evaluator_parent_inspection, sizeof(__pyx_k_evaluator_parent_inspection), 0, 0, 1, 1},
  {&__pyx_n_s_evaluators, __pyx_k_evaluators, sizeof(__pyx_k_evaluators), 0, 0, 1, 1},
  {&__pyx_n_s_exceptions, __pyx_k_exceptions, sizeof(__pyx_k_exceptions), 0, 0, 1, 1},
  {&__pyx_n_u_f, __pyx_k_f, sizeof(__pyx_k_f), 0, 1, 0, 1},
  {&__pyx_n_s_find_optimal_configuration, __pyx_k_find_optimal_configuration, sizeof(__pyx_k_find_optimal_configuration), 0, 0, 1, 1},
  {&__pyx_kp_u_finished_computation, __pyx_k_finished_computation, sizeof(__pyx_k_finished_computation), 0, 1, 0, 0},
  {&__pyx_n_s_get_activated_element, __pyx_k_get_activated_element, sizeof(__pyx_k_get_activated_element), 0, 0, 1, 1},
  {&__pyx_n_s_get_activated_trading_mode, __pyx_k_get_activated_trading_mode, sizeof(__pyx_k_get_activated_trading_mode), 0, 0, 1, 1},
  {&__pyx_n_s_get_average_score, __pyx_k_get_average_score, sizeof(__pyx_k_get_average_score), 0, 0, 1, 1},
  {&__pyx_n_s_get_class_from_string, __pyx_k_get_class_from_string, sizeof(__pyx_k_get_class_from_string), 0, 0, 1, 1},
  {&__pyx_n_s_get_config_summary, __pyx_k_get_config_summary, sizeof(__pyx_k_get_config_summary), 0, 0, 1, 1},
  {&__pyx_n_s_get_current_test_suite_progress, __pyx_k_get_current_test_suite_progress, sizeof(__pyx_k_get_current_test_suite_progress), 0, 0, 1, 1},
  {&__pyx_n_s_get_errors_description, __pyx_k_get_errors_description, sizeof(__pyx_k_get_errors_description), 0, 0, 1, 1},
  {&__pyx_n_s_get_filtered_results, __pyx_k_get_filtered_results, sizeof(__pyx_k_get_filtered_results), 0, 0, 1, 1},
  {&__pyx_n_s_get_global_logger_level, __pyx_k_get_global_logger_level, sizeof(__pyx_k_get_global_logger_level), 0, 0, 1, 1},
  {&__pyx_n_s_get_logger, __pyx_k_get_logger, sizeof(__pyx_k_get_logger), 0, 0, 1, 1},
  {&__pyx_n_s_get_name, __pyx_k_get_name, sizeof(__pyx_k_get_name), 0, 0, 1, 1},
  {&__pyx_n_s_get_overall_progress, __pyx_k_get_overall_progress, sizeof(__pyx_k_get_overall_progress), 0, 0, 1, 1},
  {&__pyx_n_s_get_report, __pyx_k_get_report, sizeof(__pyx_k_get_report), 0, 0, 1, 1},
  {&__pyx_n_s_get_required_time_frames, __pyx_k_get_required_time_frames, sizeof(__pyx_k_get_required_time_frames), 0, 0, 1, 1},
  {&__pyx_n_s_get_result_string, __pyx_k_get_result_string, sizeof(__pyx_k_get_result_string), 0, 0, 1, 1},
  {&__pyx_n_s_get_sorted_results, __pyx_k_get_sorted_results, sizeof(__pyx_k_get_sorted_results), 0, 0, 1, 1},
  {&__pyx_n_s_get_sorted_results_locals_lambd, __pyx_k_get_sorted_results_locals_lambd, sizeof(__pyx_k_get_sorted_results_locals_lambd), 0, 0, 1, 1},
  {&__pyx_n_s_get_tentacles_activation, __pyx_k_get_tentacles_activation, sizeof(__pyx_k_get_tentacles_activation), 0, 0, 1, 1},
  {&__pyx_n_s_get_test_suite_result, __pyx_k_get_test_suite_result, sizeof(__pyx_k_get_test_suite_result), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_kp_u_in_installed_strategies_Please, __pyx_k_in_installed_strategies_Please, sizeof(__pyx_k_in_installed_strategies_Please), 0, 1, 0, 0},
  {&__pyx_n_s_info, __pyx_k_info, sizeof(__pyx_k_info), 0, 0, 1, 1},
  {&__pyx_n_s_init_trading_mode_config, __pyx_k_init_trading_mode_config, sizeof(__pyx_k_init_trading_mode_config), 0, 0, 1, 1},
  {&__pyx_n_s_initialize_with_strategy, __pyx_k_initialize_with_strategy, sizeof(__pyx_k_initialize_with_strategy), 0, 0, 1, 1},
  {&__pyx_n_s_inspect, __pyx_k_inspect, sizeof(__pyx_k_inspect), 0, 0, 1, 1},
  {&__pyx_kp_u_is_already_computing_processed, __pyx_k_is_already_computing_processed, sizeof(__pyx_k_is_already_computing_processed), 0, 1, 0, 0},
  {&__pyx_n_s_is_in_progress, __pyx_k_is_in_progress, sizeof(__pyx_k_is_in_progress), 0, 0, 1, 1},
  {&__pyx_n_s_is_relevant_evaluation_config, __pyx_k_is_relevant_evaluation_config, sizeof(__pyx_k_is_relevant_evaluation_config), 0, 0, 1, 1},
  {&__pyx_n_s_items, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_logging, __pyx_k_logging, sizeof(__pyx_k_logging), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_math, __pyx_k_math, sizeof(__pyx_k_math), 0, 0, 1, 1},
  {&__pyx_n_s_mean, __pyx_k_mean, sizeof(__pyx_k_mean), 0, 0, 1, 1},
  {&__pyx_n_s_min_time_frame, __pyx_k_min_time_frame, sizeof(__pyx_k_min_time_frame), 0, 0, 1, 1},
  {&__pyx_kp_u_minimum_time_frame_ranking, __pyx_k_minimum_time_frame_ranking, sizeof(__pyx_k_minimum_time_frame_ranking), 0, 1, 0, 0},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_nb_elements_to_consider, __pyx_k_nb_elements_to_consider, sizeof(__pyx_k_nb_elements_to_consider), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_octobot_commons_data_util, __pyx_k_octobot_commons_data_util, sizeof(__pyx_k_octobot_commons_data_util), 0, 0, 1, 1},
  {&__pyx_n_s_octobot_commons_logging_logging, __pyx_k_octobot_commons_logging_logging, sizeof(__pyx_k_octobot_commons_logging_logging), 0, 0, 1, 1},
  {&__pyx_n_s_octobot_commons_tentacles_manage, __pyx_k_octobot_commons_tentacles_manage, sizeof(__pyx_k_octobot_commons_tentacles_manage), 0, 0, 1, 1},
  {&__pyx_n_s_octobot_constants, __pyx_k_octobot_constants, sizeof(__pyx_k_octobot_constants), 0, 0, 1, 1},
  {&__pyx_n_s_octobot_evaluators_api_evaluator, __pyx_k_octobot_evaluators_api_evaluator, sizeof(__pyx_k_octobot_evaluators_api_evaluator), 0, 0, 1, 1},
  {&__pyx_n_s_octobot_evaluators_constants, __pyx_k_octobot_evaluators_constants, sizeof(__pyx_k_octobot_evaluators_constants), 0, 0, 1, 1},
  {&__pyx_n_s_octobot_evaluators_evaluator_TA, __pyx_k_octobot_evaluators_evaluator_TA, sizeof(__pyx_k_octobot_evaluators_evaluator_TA), 0, 0, 1, 1},
  {&__pyx_n_s_octobot_evaluators_evaluator_str, __pyx_k_octobot_evaluators_evaluator_str, sizeof(__pyx_k_octobot_evaluators_evaluator_str), 0, 0, 1, 1},
  {&__pyx_kp_s_octobot_strategy_optimizer_strat, __pyx_k_octobot_strategy_optimizer_strat, sizeof(__pyx_k_octobot_strategy_optimizer_strat), 0, 0, 1, 0},
  {&__pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_k_octobot_strategy_optimizer_strat_2, sizeof(__pyx_k_octobot_strategy_optimizer_strat_2), 0, 0, 1, 1},
  {&__pyx_n_s_octobot_strategy_optimizer_strat_3, __pyx_k_octobot_strategy_optimizer_strat_3, sizeof(__pyx_k_octobot_strategy_optimizer_strat_3), 0, 0, 1, 1},
  {&__pyx_n_s_octobot_strategy_optimizer_test, __pyx_k_octobot_strategy_optimizer_test, sizeof(__pyx_k_octobot_strategy_optimizer_test), 0, 0, 1, 1},
  {&__pyx_n_s_octobot_tentacles_manager_api_co, __pyx_k_octobot_tentacles_manager_api_co, sizeof(__pyx_k_octobot_tentacles_manager_api_co), 0, 0, 1, 1},
  {&__pyx_n_s_octobot_trading_api_modes, __pyx_k_octobot_trading_api_modes, sizeof(__pyx_k_octobot_trading_api_modes), 0, 0, 1, 1},
  {&__pyx_n_s_octobot_trading_constants, __pyx_k_octobot_trading_constants, sizeof(__pyx_k_octobot_trading_constants), 0, 0, 1, 1},
  {&__pyx_n_s_octobot_trading_modes, __pyx_k_octobot_trading_modes, sizeof(__pyx_k_octobot_trading_modes), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_pop, __pyx_k_pop, sizeof(__pyx_k_pop), 0, 0, 1, 1},
  {&__pyx_n_s_pow, __pyx_k_pow, sizeof(__pyx_k_pow), 0, 0, 1, 1},
  {&__pyx_n_s_print, __pyx_k_print, sizeof(__pyx_k_print), 0, 0, 1, 1},
  {&__pyx_n_s_print_report, __pyx_k_print_report, sizeof(__pyx_k_print_report), 0, 0, 1, 1},
  {&__pyx_kp_u_processed, __pyx_k_processed, sizeof(__pyx_k_processed), 0, 1, 0, 0},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_StrategyOptimizer, __pyx_k_pyx_unpickle_StrategyOptimizer, sizeof(__pyx_k_pyx_unpickle_StrategyOptimizer), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_result, __pyx_k_result, sizeof(__pyx_k_result), 0, 0, 1, 1},
  {&__pyx_n_s_results, __pyx_k_results, sizeof(__pyx_k_results), 0, 0, 1, 1},
  {&__pyx_n_s_reverse, __pyx_k_reverse, sizeof(__pyx_k_reverse), 0, 0, 1, 1},
  {&__pyx_kp_u_risk, __pyx_k_risk, sizeof(__pyx_k_risk), 0, 1, 0, 0},
  {&__pyx_n_s_risk_2, __pyx_k_risk_2, sizeof(__pyx_k_risk_2), 0, 0, 1, 1},
  {&__pyx_kp_u_risk_s, __pyx_k_risk_s, sizeof(__pyx_k_risk_s), 0, 1, 0, 0},
  {&__pyx_n_s_risks, __pyx_k_risks, sizeof(__pyx_k_risks), 0, 0, 1, 1},
  {&__pyx_n_s_round, __pyx_k_round, sizeof(__pyx_k_round), 0, 0, 1, 1},
  {&__pyx_n_s_run, __pyx_k_run, sizeof(__pyx_k_run), 0, 0, 1, 1},
  {&__pyx_n_s_run_test_suite, __pyx_k_run_test_suite, sizeof(__pyx_k_run_test_suite), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_n_s_set_global_logger_level, __pyx_k_set_global_logger_level, sizeof(__pyx_k_set_global_logger_level), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_sorted, __pyx_k_sorted, sizeof(__pyx_k_sorted), 0, 0, 1, 1},
  {&__pyx_n_s_sorting_key, __pyx_k_sorting_key, sizeof(__pyx_k_sorting_key), 0, 0, 1, 1},
  {&__pyx_n_s_state, __pyx_k_state, sizeof(__pyx_k_state), 0, 0, 1, 1},
  {&__pyx_n_s_staticmethod, __pyx_k_staticmethod, sizeof(__pyx_k_staticmethod), 0, 0, 1, 1},
  {&__pyx_n_s_strategy_name, __pyx_k_strategy_name, sizeof(__pyx_k_strategy_name), 0, 0, 1, 1},
  {&__pyx_kp_u_strategy_using, __pyx_k_strategy_using, sizeof(__pyx_k_strategy_using), 0, 1, 0, 0},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_kp_u_technical_evaluator_s, __pyx_k_technical_evaluator_s, sizeof(__pyx_k_technical_evaluator_s), 0, 1, 0, 0},
  {&__pyx_n_s_tentacles_Evaluator, __pyx_k_tentacles_Evaluator, sizeof(__pyx_k_tentacles_Evaluator), 0, 0, 1, 1},
  {&__pyx_n_s_tentacles_Trading, __pyx_k_tentacles_Trading, sizeof(__pyx_k_tentacles_Trading), 0, 0, 1, 1},
  {&__pyx_n_s_tentacles_setup_config, __pyx_k_tentacles_setup_config, sizeof(__pyx_k_tentacles_setup_config), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_time_frame, __pyx_k_time_frame, sizeof(__pyx_k_time_frame), 0, 0, 1, 1},
  {&__pyx_kp_u_time_frame_rank_sum, __pyx_k_time_frame_rank_sum, sizeof(__pyx_k_time_frame_rank_sum), 0, 1, 0, 0},
  {&__pyx_kp_u_time_frames, __pyx_k_time_frames, sizeof(__pyx_k_time_frames), 0, 1, 0, 0},
  {&__pyx_n_s_time_frames_2, __pyx_k_time_frames_2, sizeof(__pyx_k_time_frames_2), 0, 0, 1, 1},
  {&__pyx_kp_u_time_frames_and, __pyx_k_time_frames_and, sizeof(__pyx_k_time_frames_and), 0, 1, 0, 0},
  {&__pyx_n_s_to_use_conf, __pyx_k_to_use_conf, sizeof(__pyx_k_to_use_conf), 0, 0, 1, 1},
  {&__pyx_n_s_trades_counts, __pyx_k_trades_counts, sizeof(__pyx_k_trades_counts), 0, 0, 1, 1},
  {&__pyx_kp_u_trading_mode, __pyx_k_trading_mode, sizeof(__pyx_k_trading_mode), 0, 1, 0, 0},
  {&__pyx_kp_u_trading_mode_2, __pyx_k_trading_mode_2, sizeof(__pyx_k_trading_mode_2), 0, 1, 0, 0},
  {&__pyx_n_s_trading_mode_parent_inspection, __pyx_k_trading_mode_parent_inspection, sizeof(__pyx_k_trading_mode_parent_inspection), 0, 0, 1, 1},
  {&__pyx_n_s_union, __pyx_k_union, sizeof(__pyx_k_union), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_update_activation_configuration, __pyx_k_update_activation_configuration, sizeof(__pyx_k_update_activation_configuration), 0, 0, 1, 1},
  {&__pyx_n_s_use_setstate, __pyx_k_use_setstate, sizeof(__pyx_k_use_setstate), 0, 0, 1, 1},
  {&__pyx_kp_u_using, __pyx_k_using, sizeof(__pyx_k_using), 0, 1, 0, 0},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_values, __pyx_k_values, sizeof(__pyx_k_values), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_staticmethod = __Pyx_GetBuiltinName(__pyx_n_s_staticmethod); if (!__pyx_builtin_staticmethod) __PYX_ERR(0, 267, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 120, __pyx_L1_error)
  __pyx_builtin_print = __Pyx_GetBuiltinName(__pyx_n_s_print); if (!__pyx_builtin_print) __PYX_ERR(0, 148, __pyx_L1_error)
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) __PYX_ERR(0, 164, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(0, 199, __pyx_L1_error)
  __pyx_builtin_sorted = __Pyx_GetBuiltinName(__pyx_n_s_sorted); if (!__pyx_builtin_sorted) __PYX_ERR(0, 213, __pyx_L1_error)
  __pyx_builtin_round = __Pyx_GetBuiltinName(__pyx_n_s_round); if (!__pyx_builtin_round) __PYX_ERR(0, 251, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":210
 * 
 *         # inner function to replace uncythonizable lambda expression
 *         def _sorting_key(elem):             # <<<<<<<<<<<<<<
 *             return elem[RANK]
 * 
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_n_s_elem); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);
  __pyx_codeobj__3 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__2, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_octobot_strategy_optimizer_strat, __pyx_n_s_sorting_key, 210, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__3)) __PYX_ERR(0, 210, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":81
 *             self.is_properly_initialized = True
 * 
 *     def find_optimal_configuration(self, TAs=None, time_frames=None, risks=None):             # <<<<<<<<<<<<<<
 *         if not self.is_computing:
 * 
 */
  __pyx_tuple__9 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_TAs, __pyx_n_s_time_frames_2, __pyx_n_s_risks); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __pyx_codeobj__10 = (PyObject*)__Pyx_PyCode_New(4, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__9, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_octobot_strategy_optimizer_strat, __pyx_n_s_find_optimal_configuration, 81, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__10)) __PYX_ERR(0, 81, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":215
 *         self.sorted_results_through_all_time_frame = sorted(result_list, key=_sorting_key)
 * 
 *     def print_report(self):             # <<<<<<<<<<<<<<
 *         self.logger.info("Full execution sorted results: Minimum time frames are defining the range of the run "
 *                          "since it finishes at the end of the first data, aka minimum time frame. Therefore all "
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);
  __pyx_codeobj__12 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_octobot_strategy_optimizer_strat, __pyx_n_s_print_report, 215, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__12)) __PYX_ERR(0, 215, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":237
 *                          f"average trades count: {best_result[TRADES_IN_RESULT]:f}")
 * 
 *     def get_overall_progress(self):             # <<<<<<<<<<<<<<
 *         return int((self.run_id-1) / self.total_nb_runs * 100) if self.total_nb_runs else 0
 * 
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__13, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_octobot_strategy_optimizer_strat, __pyx_n_s_get_overall_progress, 237, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) __PYX_ERR(0, 237, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":240
 *         return int((self.run_id-1) / self.total_nb_runs * 100) if self.total_nb_runs else 0
 * 
 *     def is_in_progress(self):             # <<<<<<<<<<<<<<
 *         return self.get_overall_progress() != 100
 * 
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_codeobj__16 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__15, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_octobot_strategy_optimizer_strat, __pyx_n_s_is_in_progress, 240, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__16)) __PYX_ERR(0, 240, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":243
 *         return self.get_overall_progress() != 100
 * 
 *     def get_current_test_suite_progress(self):             # <<<<<<<<<<<<<<
 *         return self.current_test_suite.current_progress if self.current_test_suite else 0
 * 
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_octobot_strategy_optimizer_strat, __pyx_n_s_get_current_test_suite_progress, 243, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) __PYX_ERR(0, 243, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":246
 *         return self.current_test_suite.current_progress if self.current_test_suite else 0
 * 
 *     def get_report(self):             # <<<<<<<<<<<<<<
 *         # index, evaluators, risk, score, trades
 *         if self.sorted_results_through_all_time_frame:
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);
  __pyx_codeobj__20 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__19, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_octobot_strategy_optimizer_strat, __pyx_n_s_get_report, 246, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__20)) __PYX_ERR(0, 246, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":257
 *         return results
 * 
 *     def get_errors_description(self):             # <<<<<<<<<<<<<<
 *         if self.errors:
 *             return f"{', '.join(self.errors)[0:350]} ..."
 */
  __pyx_tuple__21 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);
  __pyx_codeobj__22 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_octobot_strategy_optimizer_strat, __pyx_n_s_get_errors_description, 257, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__22)) __PYX_ERR(0, 257, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":264
 * 
 *     @classmethod
 *     def get_name(cls):             # <<<<<<<<<<<<<<
 *         return cls.__name__
 * 
 */
  __pyx_tuple__23 = PyTuple_Pack(1, __pyx_n_s_cls); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);
  __pyx_codeobj__24 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__23, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_octobot_strategy_optimizer_strat, __pyx_n_s_get_name, 264, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__24)) __PYX_ERR(0, 264, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":268
 * 
 *     @staticmethod
 *     def _get_activated_element(all_elements, current_forced_element, nb_elements_to_consider,             # <<<<<<<<<<<<<<
 *                                elem_conf_history, default_element=None, dict_shaped=False):
 *         eval_conf = {current_forced_element: True}
 */
  __pyx_tuple__25 = PyTuple_Pack(13, __pyx_n_s_all_elements, __pyx_n_s_current_forced_element, __pyx_n_s_nb_elements_to_consider, __pyx_n_s_elem_conf_history, __pyx_n_s_default_element, __pyx_n_s_dict_shaped, __pyx_n_s_eval_conf, __pyx_n_s_additional_elements_count, __pyx_n_s_i, __pyx_n_s_current_elem, __pyx_n_s_to_use_conf, __pyx_n_s_key, __pyx_n_s_key); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);
  __pyx_codeobj__26 = (PyObject*)__Pyx_PyCode_New(6, 0, 13, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__25, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_octobot_strategy_optimizer_strat, __pyx_n_s_get_activated_element, 268, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__26)) __PYX_ERR(0, 268, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":297
 * 
 *     @staticmethod
 *     def _get_filtered_results(results, time_frame=None):             # <<<<<<<<<<<<<<
 *         return [result for result in results if time_frame is None or result.min_time_frame == time_frame]
 * 
 */
  __pyx_tuple__27 = PyTuple_Pack(3, __pyx_n_s_results, __pyx_n_s_time_frame, __pyx_n_s_result); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);
  __pyx_codeobj__28 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__27, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_octobot_strategy_optimizer_strat, __pyx_n_s_get_filtered_results, 297, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__28)) __PYX_ERR(0, 297, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":301
 * 
 *     @staticmethod
 *     def _get_sorted_results(results, time_frame=None):             # <<<<<<<<<<<<<<
 *         return sorted(StrategyOptimizer._get_filtered_results(results, time_frame),
 *                       key=lambda result: result.get_average_score(), reverse=True)
 */
  __pyx_tuple__29 = PyTuple_Pack(2, __pyx_n_s_results, __pyx_n_s_time_frame); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);
  __pyx_codeobj__30 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__29, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_octobot_strategy_optimizer_strat, __pyx_n_s_get_sorted_results, 301, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__30)) __PYX_ERR(0, 301, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":306
 * 
 *     @staticmethod
 *     def _is_relevant_evaluation_config(evaluator):             # <<<<<<<<<<<<<<
 *         return get_class_from_string(evaluator, TAEvaluator, TA, evaluator_parent_inspection) is not None
 * 
 */
  __pyx_tuple__31 = PyTuple_Pack(1, __pyx_n_s_evaluator); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);
  __pyx_codeobj__32 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__31, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_octobot_strategy_optimizer_strat, __pyx_n_s_is_relevant_evaluation_config, 306, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__32)) __PYX_ERR(0, 306, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_tuple__33 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_state, __pyx_n_s_dict_2, __pyx_n_s_use_setstate); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);
  __pyx_codeobj__34 = (PyObject*)__Pyx_PyCode_New(1, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__33, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_reduce_cython, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__34)) __PYX_ERR(2, 1, __pyx_L1_error)

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_StrategyOptimizer, (type(self), 0x2d2eb06, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_StrategyOptimizer__set_state(self, __pyx_state)
 */
  __pyx_tuple__35 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_pyx_state); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);
  __pyx_codeobj__36 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__35, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_setstate_cython, 16, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__36)) __PYX_ERR(2, 16, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_StrategyOptimizer(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__37 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);
  __pyx_codeobj__38 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__37, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_StrategyOptimizer, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__38)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_5 = PyInt_FromLong(5); if (unlikely(!__pyx_int_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_47377158 = PyInt_FromLong(47377158L); if (unlikely(!__pyx_int_47377158)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer = &__pyx_vtable_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer;
  __pyx_vtable_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer.find_optimal_configuration = (void (*)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, int __pyx_skip_dispatch, struct __pyx_opt_args_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_find_optimal_configuration *__pyx_optional_args))__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_find_optimal_configuration;
  __pyx_vtable_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer.print_report = (void (*)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, int __pyx_skip_dispatch))__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_print_report;
  __pyx_vtable_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer.get_overall_progress = (int (*)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, int __pyx_skip_dispatch))__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_overall_progress;
  __pyx_vtable_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer.is_in_progress = (int (*)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, int __pyx_skip_dispatch))__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_is_in_progress;
  __pyx_vtable_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer.get_current_test_suite_progress = (int (*)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, int __pyx_skip_dispatch))__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_current_test_suite_progress;
  __pyx_vtable_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer.get_report = (PyObject *(*)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, int __pyx_skip_dispatch))__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_report;
  __pyx_vtable_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer.get_errors_description = (PyObject *(*)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, int __pyx_skip_dispatch))__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_get_errors_description;
  __pyx_vtable_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer._run_test_suite = (void (*)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, PyObject *, PyObject *))__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer__run_test_suite;
  __pyx_vtable_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer._adapt_tentacles_config = (void (*)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *, PyObject *))__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer__adapt_tentacles_config;
  __pyx_vtable_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer._find_optimal_configuration_using_results = (void (*)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *))__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer__find_optimal_configuration_using_results;
  __pyx_vtable_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer._get_all_TA = (PyObject *(*)(struct __pyx_obj_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer *))__pyx_f_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer__get_all_TA;
  if (PyType_Ready(&__pyx_type_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer) < 0) __PYX_ERR(0, 47, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer.tp_dictoffset && __pyx_type_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer.tp_dict, __pyx_vtabptr_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer) < 0) __PYX_ERR(0, 47, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_StrategyOptimizer, (PyObject *)&__pyx_type_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer) < 0) __PYX_ERR(0, 47, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer) < 0) __PYX_ERR(0, 47, __pyx_L1_error)
  __pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer = &__pyx_type_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#if PY_MAJOR_VERSION < 3
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC void
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#else
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initstrategy_optimizer(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initstrategy_optimizer(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_strategy_optimizer(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_strategy_optimizer(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_strategy_optimizer(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'strategy_optimizer' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_strategy_optimizer(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("strategy_optimizer", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_octobot__strategy_optimizer__strategy_optimizer) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "octobot.strategy_optimizer.strategy_optimizer")) {
      if (unlikely(PyDict_SetItemString(modules, "octobot.strategy_optimizer.strategy_optimizer", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) goto __pyx_L1_error;
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) goto __pyx_L1_error;
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() != 0)) goto __pyx_L1_error;
  (void)__Pyx_modinit_type_import_code();
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "octobot/strategy_optimizer/strategy_optimizer.py":16
 * #  You should have received a copy of the GNU Lesser General Public
 * #  License along with this library.
 * import asyncio             # <<<<<<<<<<<<<<
 * import logging
 * import math
 */
  __pyx_t_1 = __Pyx_patch_asyncio(__Pyx_Import(__pyx_n_s_asyncio, 0, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_asyncio, __pyx_t_1) < 0) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":17
 * #  License along with this library.
 * import asyncio
 * import logging             # <<<<<<<<<<<<<<
 * import math
 * from copy import deepcopy
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_logging, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_logging, __pyx_t_1) < 0) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":18
 * import asyncio
 * import logging
 * import math             # <<<<<<<<<<<<<<
 * from copy import deepcopy
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_math, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_math, __pyx_t_1) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":19
 * import logging
 * import math
 * from copy import deepcopy             # <<<<<<<<<<<<<<
 * 
 * from octobot.constants import OPTIMIZER_FORCE_ASYNCIO_DEBUG_OPTION
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_deepcopy);
  __Pyx_GIVEREF(__pyx_n_s_deepcopy);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_deepcopy);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_copy, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_deepcopy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_deepcopy, __pyx_t_1) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":21
 * from copy import deepcopy
 * 
 * from octobot.constants import OPTIMIZER_FORCE_ASYNCIO_DEBUG_OPTION             # <<<<<<<<<<<<<<
 * from octobot.strategy_optimizer.strategy_test_suite import StrategyTestSuite
 * from octobot.strategy_optimizer.test_suite_result import TestSuiteResult
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_OPTIMIZER_FORCE_ASYNCIO_DEBUG_OP);
  __Pyx_GIVEREF(__pyx_n_s_OPTIMIZER_FORCE_ASYNCIO_DEBUG_OP);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_OPTIMIZER_FORCE_ASYNCIO_DEBUG_OP);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_octobot_constants, __pyx_t_2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_OPTIMIZER_FORCE_ASYNCIO_DEBUG_OP); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_OPTIMIZER_FORCE_ASYNCIO_DEBUG_OP, __pyx_t_2) < 0) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":22
 * 
 * from octobot.constants import OPTIMIZER_FORCE_ASYNCIO_DEBUG_OPTION
 * from octobot.strategy_optimizer.strategy_test_suite import StrategyTestSuite             # <<<<<<<<<<<<<<
 * from octobot.strategy_optimizer.test_suite_result import TestSuiteResult
 * from octobot_commons.data_util import mean
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_StrategyTestSuite);
  __Pyx_GIVEREF(__pyx_n_s_StrategyTestSuite);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_StrategyTestSuite);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_octobot_strategy_optimizer_strat_3, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_StrategyTestSuite); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_StrategyTestSuite, __pyx_t_1) < 0) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":23
 * from octobot.constants import OPTIMIZER_FORCE_ASYNCIO_DEBUG_OPTION
 * from octobot.strategy_optimizer.strategy_test_suite import StrategyTestSuite
 * from octobot.strategy_optimizer.test_suite_result import TestSuiteResult             # <<<<<<<<<<<<<<
 * from octobot_commons.data_util import mean
 * from octobot_commons.tentacles_management.class_inspector import get_class_from_string, evaluator_parent_inspection, \
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_TestSuiteResult);
  __Pyx_GIVEREF(__pyx_n_s_TestSuiteResult);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_TestSuiteResult);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_octobot_strategy_optimizer_test, __pyx_t_2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_TestSuiteResult); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TestSuiteResult, __pyx_t_2) < 0) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":24
 * from octobot.strategy_optimizer.strategy_test_suite import StrategyTestSuite
 * from octobot.strategy_optimizer.test_suite_result import TestSuiteResult
 * from octobot_commons.data_util import mean             # <<<<<<<<<<<<<<
 * from octobot_commons.tentacles_management.class_inspector import get_class_from_string, evaluator_parent_inspection, \
 *     trading_mode_parent_inspection
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_mean);
  __Pyx_GIVEREF(__pyx_n_s_mean);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_mean);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_octobot_commons_data_util, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_mean); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_mean, __pyx_t_1) < 0) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":25
 * from octobot.strategy_optimizer.test_suite_result import TestSuiteResult
 * from octobot_commons.data_util import mean
 * from octobot_commons.tentacles_management.class_inspector import get_class_from_string, evaluator_parent_inspection, \             # <<<<<<<<<<<<<<
 *     trading_mode_parent_inspection
 * from octobot_commons.logging.logging_util import get_logger
 */
  __pyx_t_2 = PyList_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_get_class_from_string);
  __Pyx_GIVEREF(__pyx_n_s_get_class_from_string);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_get_class_from_string);
  __Pyx_INCREF(__pyx_n_s_evaluator_parent_inspection);
  __Pyx_GIVEREF(__pyx_n_s_evaluator_parent_inspection);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_evaluator_parent_inspection);
  __Pyx_INCREF(__pyx_n_s_trading_mode_parent_inspection);
  __Pyx_GIVEREF(__pyx_n_s_trading_mode_parent_inspection);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_s_trading_mode_parent_inspection);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_octobot_commons_tentacles_manage, __pyx_t_2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_get_class_from_string); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_class_from_string, __pyx_t_2) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_evaluator_parent_inspection); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_evaluator_parent_inspection, __pyx_t_2) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_trading_mode_parent_inspection); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_trading_mode_parent_inspection, __pyx_t_2) < 0) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":27
 * from octobot_commons.tentacles_management.class_inspector import get_class_from_string, evaluator_parent_inspection, \
 *     trading_mode_parent_inspection
 * from octobot_commons.logging.logging_util import get_logger             # <<<<<<<<<<<<<<
 * from octobot_commons.logging.logging_util import set_global_logger_level, get_global_logger_level
 * from octobot_tentacles_manager.api.configurator import get_tentacles_activation, update_activation_configuration
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_get_logger);
  __Pyx_GIVEREF(__pyx_n_s_get_logger);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_get_logger);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_octobot_commons_logging_logging, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_get_logger); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_logger, __pyx_t_1) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":28
 *     trading_mode_parent_inspection
 * from octobot_commons.logging.logging_util import get_logger
 * from octobot_commons.logging.logging_util import set_global_logger_level, get_global_logger_level             # <<<<<<<<<<<<<<
 * from octobot_tentacles_manager.api.configurator import get_tentacles_activation, update_activation_configuration
 * from octobot_evaluators.constants import CONFIG_FORCED_TIME_FRAME
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_set_global_logger_level);
  __Pyx_GIVEREF(__pyx_n_s_set_global_logger_level);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_set_global_logger_level);
  __Pyx_INCREF(__pyx_n_s_get_global_logger_level);
  __Pyx_GIVEREF(__pyx_n_s_get_global_logger_level);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_get_global_logger_level);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_octobot_commons_logging_logging, __pyx_t_2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_set_global_logger_level); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_set_global_logger_level, __pyx_t_2) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_get_global_logger_level); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_global_logger_level, __pyx_t_2) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":29
 * from octobot_commons.logging.logging_util import get_logger
 * from octobot_commons.logging.logging_util import set_global_logger_level, get_global_logger_level
 * from octobot_tentacles_manager.api.configurator import get_tentacles_activation, update_activation_configuration             # <<<<<<<<<<<<<<
 * from octobot_evaluators.constants import CONFIG_FORCED_TIME_FRAME
 * from octobot_trading.constants import CONFIG_TRADER_RISK, CONFIG_TRADING
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_get_tentacles_activation);
  __Pyx_GIVEREF(__pyx_n_s_get_tentacles_activation);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_get_tentacles_activation);
  __Pyx_INCREF(__pyx_n_s_update_activation_configuration);
  __Pyx_GIVEREF(__pyx_n_s_update_activation_configuration);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_update_activation_configuration);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_octobot_tentacles_manager_api_co, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_get_tentacles_activation); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_tentacles_activation, __pyx_t_1) < 0) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_update_activation_configuration); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_update_activation_configuration, __pyx_t_1) < 0) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":30
 * from octobot_commons.logging.logging_util import set_global_logger_level, get_global_logger_level
 * from octobot_tentacles_manager.api.configurator import get_tentacles_activation, update_activation_configuration
 * from octobot_evaluators.constants import CONFIG_FORCED_TIME_FRAME             # <<<<<<<<<<<<<<
 * from octobot_trading.constants import CONFIG_TRADER_RISK, CONFIG_TRADING
 * from octobot_evaluators.api.evaluators import create_evaluator_classes
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_CONFIG_FORCED_TIME_FRAME);
  __Pyx_GIVEREF(__pyx_n_s_CONFIG_FORCED_TIME_FRAME);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_CONFIG_FORCED_TIME_FRAME);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_octobot_evaluators_constants, __pyx_t_2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_CONFIG_FORCED_TIME_FRAME); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONFIG_FORCED_TIME_FRAME, __pyx_t_2) < 0) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":31
 * from octobot_tentacles_manager.api.configurator import get_tentacles_activation, update_activation_configuration
 * from octobot_evaluators.constants import CONFIG_FORCED_TIME_FRAME
 * from octobot_trading.constants import CONFIG_TRADER_RISK, CONFIG_TRADING             # <<<<<<<<<<<<<<
 * from octobot_evaluators.api.evaluators import create_evaluator_classes
 * from octobot_evaluators.evaluator.strategy_evaluator import StrategyEvaluator
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_CONFIG_TRADER_RISK);
  __Pyx_GIVEREF(__pyx_n_s_CONFIG_TRADER_RISK);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_CONFIG_TRADER_RISK);
  __Pyx_INCREF(__pyx_n_s_CONFIG_TRADING);
  __Pyx_GIVEREF(__pyx_n_s_CONFIG_TRADING);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_CONFIG_TRADING);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_octobot_trading_constants, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_CONFIG_TRADER_RISK); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONFIG_TRADER_RISK, __pyx_t_1) < 0) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_CONFIG_TRADING); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONFIG_TRADING, __pyx_t_1) < 0) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":32
 * from octobot_evaluators.constants import CONFIG_FORCED_TIME_FRAME
 * from octobot_trading.constants import CONFIG_TRADER_RISK, CONFIG_TRADING
 * from octobot_evaluators.api.evaluators import create_evaluator_classes             # <<<<<<<<<<<<<<
 * from octobot_evaluators.evaluator.strategy_evaluator import StrategyEvaluator
 * from octobot_evaluators.evaluator.TA_evaluator import TAEvaluator
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_create_evaluator_classes);
  __Pyx_GIVEREF(__pyx_n_s_create_evaluator_classes);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_create_evaluator_classes);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_octobot_evaluators_api_evaluator, __pyx_t_2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_create_evaluator_classes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_evaluator_classes, __pyx_t_2) < 0) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":33
 * from octobot_trading.constants import CONFIG_TRADER_RISK, CONFIG_TRADING
 * from octobot_evaluators.api.evaluators import create_evaluator_classes
 * from octobot_evaluators.evaluator.strategy_evaluator import StrategyEvaluator             # <<<<<<<<<<<<<<
 * from octobot_evaluators.evaluator.TA_evaluator import TAEvaluator
 * from octobot_trading.api.modes import init_trading_mode_config, get_activated_trading_mode
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_StrategyEvaluator);
  __Pyx_GIVEREF(__pyx_n_s_StrategyEvaluator);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_StrategyEvaluator);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_octobot_evaluators_evaluator_str, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_StrategyEvaluator); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_StrategyEvaluator, __pyx_t_1) < 0) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":34
 * from octobot_evaluators.api.evaluators import create_evaluator_classes
 * from octobot_evaluators.evaluator.strategy_evaluator import StrategyEvaluator
 * from octobot_evaluators.evaluator.TA_evaluator import TAEvaluator             # <<<<<<<<<<<<<<
 * from octobot_trading.api.modes import init_trading_mode_config, get_activated_trading_mode
 * from octobot_trading.modes import AbstractTradingMode
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_TAEvaluator);
  __Pyx_GIVEREF(__pyx_n_s_TAEvaluator);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_TAEvaluator);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_octobot_evaluators_evaluator_TA, __pyx_t_2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_TAEvaluator); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TAEvaluator, __pyx_t_2) < 0) __PYX_ERR(0, 34, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":35
 * from octobot_evaluators.evaluator.strategy_evaluator import StrategyEvaluator
 * from octobot_evaluators.evaluator.TA_evaluator import TAEvaluator
 * from octobot_trading.api.modes import init_trading_mode_config, get_activated_trading_mode             # <<<<<<<<<<<<<<
 * from octobot_trading.modes import AbstractTradingMode
 * from tentacles.Trading import Mode
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_init_trading_mode_config);
  __Pyx_GIVEREF(__pyx_n_s_init_trading_mode_config);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_init_trading_mode_config);
  __Pyx_INCREF(__pyx_n_s_get_activated_trading_mode);
  __Pyx_GIVEREF(__pyx_n_s_get_activated_trading_mode);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_get_activated_trading_mode);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_octobot_trading_api_modes, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_init_trading_mode_config); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_init_trading_mode_config, __pyx_t_1) < 0) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_get_activated_trading_mode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_activated_trading_mode, __pyx_t_1) < 0) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":36
 * from octobot_evaluators.evaluator.TA_evaluator import TAEvaluator
 * from octobot_trading.api.modes import init_trading_mode_config, get_activated_trading_mode
 * from octobot_trading.modes import AbstractTradingMode             # <<<<<<<<<<<<<<
 * from tentacles.Trading import Mode
 * from tentacles.Evaluator import Strategies
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_AbstractTradingMode);
  __Pyx_GIVEREF(__pyx_n_s_AbstractTradingMode);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_AbstractTradingMode);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_octobot_trading_modes, __pyx_t_2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_AbstractTradingMode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_AbstractTradingMode, __pyx_t_2) < 0) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":37
 * from octobot_trading.api.modes import init_trading_mode_config, get_activated_trading_mode
 * from octobot_trading.modes import AbstractTradingMode
 * from tentacles.Trading import Mode             # <<<<<<<<<<<<<<
 * from tentacles.Evaluator import Strategies
 * from tentacles.Evaluator import TA
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_Mode);
  __Pyx_GIVEREF(__pyx_n_s_Mode);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_Mode);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_tentacles_Trading, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Mode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Mode, __pyx_t_1) < 0) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":38
 * from octobot_trading.modes import AbstractTradingMode
 * from tentacles.Trading import Mode
 * from tentacles.Evaluator import Strategies             # <<<<<<<<<<<<<<
 * from tentacles.Evaluator import TA
 * 
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_Strategies);
  __Pyx_GIVEREF(__pyx_n_s_Strategies);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_Strategies);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_tentacles_Evaluator, __pyx_t_2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_Strategies); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Strategies, __pyx_t_2) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":39
 * from tentacles.Trading import Mode
 * from tentacles.Evaluator import Strategies
 * from tentacles.Evaluator import TA             # <<<<<<<<<<<<<<
 * 
 * CONFIG = 0
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_TA);
  __Pyx_GIVEREF(__pyx_n_s_TA);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_TA);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_tentacles_Evaluator, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_TA); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TA, __pyx_t_1) < 0) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":41
 * from tentacles.Evaluator import TA
 * 
 * CONFIG = 0             # <<<<<<<<<<<<<<
 * RANK = 1
 * TRADES = 0
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CONFIG, __pyx_int_0) < 0) __PYX_ERR(0, 41, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":42
 * 
 * CONFIG = 0
 * RANK = 1             # <<<<<<<<<<<<<<
 * TRADES = 0
 * TRADES_IN_RESULT = 2
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RANK, __pyx_int_1) < 0) __PYX_ERR(0, 42, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":43
 * CONFIG = 0
 * RANK = 1
 * TRADES = 0             # <<<<<<<<<<<<<<
 * TRADES_IN_RESULT = 2
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TRADES, __pyx_int_0) < 0) __PYX_ERR(0, 43, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":44
 * RANK = 1
 * TRADES = 0
 * TRADES_IN_RESULT = 2             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_TRADES_IN_RESULT, __pyx_int_2) < 0) __PYX_ERR(0, 44, __pyx_L1_error)

  /* "octobot/strategy_optimizer/strategy_optimizer.py":81
 *             self.is_properly_initialized = True
 * 
 *     def find_optimal_configuration(self, TAs=None, time_frames=None, risks=None):             # <<<<<<<<<<<<<<
 *         if not self.is_computing:
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_3find_optimal_configuration, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_StrategyOptimizer_find_optimal_c, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__10)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_find_optimal_configuration, __pyx_t_2) < 0) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":215
 *         self.sorted_results_through_all_time_frame = sorted(result_list, key=_sorting_key)
 * 
 *     def print_report(self):             # <<<<<<<<<<<<<<
 *         self.logger.info("Full execution sorted results: Minimum time frames are defining the range of the run "
 *                          "since it finishes at the end of the first data, aka minimum time frame. Therefore all "
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_5print_report, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_StrategyOptimizer_print_report, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__12)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_print_report, __pyx_t_2) < 0) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":237
 *                          f"average trades count: {best_result[TRADES_IN_RESULT]:f}")
 * 
 *     def get_overall_progress(self):             # <<<<<<<<<<<<<<
 *         return int((self.run_id-1) / self.total_nb_runs * 100) if self.total_nb_runs else 0
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_7get_overall_progress, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_StrategyOptimizer_get_overall_pr, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__14)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_get_overall_progress, __pyx_t_2) < 0) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":240
 *         return int((self.run_id-1) / self.total_nb_runs * 100) if self.total_nb_runs else 0
 * 
 *     def is_in_progress(self):             # <<<<<<<<<<<<<<
 *         return self.get_overall_progress() != 100
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_9is_in_progress, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_StrategyOptimizer_is_in_progress, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__16)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_is_in_progress, __pyx_t_2) < 0) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":243
 *         return self.get_overall_progress() != 100
 * 
 *     def get_current_test_suite_progress(self):             # <<<<<<<<<<<<<<
 *         return self.current_test_suite.current_progress if self.current_test_suite else 0
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_11get_current_test_suite_progress, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_StrategyOptimizer_get_current_te, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__18)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_get_current_test_suite_progress, __pyx_t_2) < 0) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":246
 *         return self.current_test_suite.current_progress if self.current_test_suite else 0
 * 
 *     def get_report(self):             # <<<<<<<<<<<<<<
 *         # index, evaluators, risk, score, trades
 *         if self.sorted_results_through_all_time_frame:
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_13get_report, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_StrategyOptimizer_get_report, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__20)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_get_report, __pyx_t_2) < 0) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":257
 *         return results
 * 
 *     def get_errors_description(self):             # <<<<<<<<<<<<<<
 *         if self.errors:
 *             return f"{', '.join(self.errors)[0:350]} ..."
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_15get_errors_description, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_StrategyOptimizer_get_errors_des, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__22)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_get_errors_description, __pyx_t_2) < 0) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":264
 * 
 *     @classmethod
 *     def get_name(cls):             # <<<<<<<<<<<<<<
 *         return cls.__name__
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_17get_name, __Pyx_CYFUNCTION_CLASSMETHOD | __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_StrategyOptimizer_get_name, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__24)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_get_name, __pyx_t_2) < 0) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":263
 *             return None
 * 
 *     @classmethod             # <<<<<<<<<<<<<<
 *     def get_name(cls):
 *         return cls.__name__
 */
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer, __pyx_n_s_get_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_Method_ClassMethod(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_get_name, __pyx_t_1) < 0) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":268
 * 
 *     @staticmethod
 *     def _get_activated_element(all_elements, current_forced_element, nb_elements_to_consider,             # <<<<<<<<<<<<<<
 *                                elem_conf_history, default_element=None, dict_shaped=False):
 *         eval_conf = {current_forced_element: True}
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_19_get_activated_element, __Pyx_CYFUNCTION_STATICMETHOD | __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_StrategyOptimizer__get_activated, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__26)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_get_activated_element, __pyx_t_1) < 0) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":267
 *         return cls.__name__
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def _get_activated_element(all_elements, current_forced_element, nb_elements_to_consider,
 *                                elem_conf_history, default_element=None, dict_shaped=False):
 */
  __Pyx_GetNameInClass(__pyx_t_1, (PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer, __pyx_n_s_get_activated_element); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_get_activated_element, __pyx_t_2) < 0) __PYX_ERR(0, 268, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":297
 * 
 *     @staticmethod
 *     def _get_filtered_results(results, time_frame=None):             # <<<<<<<<<<<<<<
 *         return [result for result in results if time_frame is None or result.min_time_frame == time_frame]
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_21_get_filtered_results, __Pyx_CYFUNCTION_STATICMETHOD | __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_StrategyOptimizer__get_filtered, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__28)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_get_filtered_results, __pyx_t_2) < 0) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":296
 *         return None
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def _get_filtered_results(results, time_frame=None):
 *         return [result for result in results if time_frame is None or result.min_time_frame == time_frame]
 */
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer, __pyx_n_s_get_filtered_results); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 296, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_get_filtered_results, __pyx_t_1) < 0) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":301
 * 
 *     @staticmethod
 *     def _get_sorted_results(results, time_frame=None):             # <<<<<<<<<<<<<<
 *         return sorted(StrategyOptimizer._get_filtered_results(results, time_frame),
 *                       key=lambda result: result.get_average_score(), reverse=True)
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_23_get_sorted_results, __Pyx_CYFUNCTION_STATICMETHOD | __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_StrategyOptimizer__get_sorted_re, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__30)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_get_sorted_results, __pyx_t_1) < 0) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":300
 *         return [result for result in results if time_frame is None or result.min_time_frame == time_frame]
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def _get_sorted_results(results, time_frame=None):
 *         return sorted(StrategyOptimizer._get_filtered_results(results, time_frame),
 */
  __Pyx_GetNameInClass(__pyx_t_1, (PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer, __pyx_n_s_get_sorted_results); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_get_sorted_results, __pyx_t_2) < 0) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":306
 * 
 *     @staticmethod
 *     def _is_relevant_evaluation_config(evaluator):             # <<<<<<<<<<<<<<
 *         return get_class_from_string(evaluator, TAEvaluator, TA, evaluator_parent_inspection) is not None
 * 
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_25_is_relevant_evaluation_config, __Pyx_CYFUNCTION_STATICMETHOD | __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_StrategyOptimizer__is_relevant_e, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__32)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_is_relevant_evaluation_config, __pyx_t_2) < 0) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "octobot/strategy_optimizer/strategy_optimizer.py":305
 *                       key=lambda result: result.get_average_score(), reverse=True)
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def _is_relevant_evaluation_config(evaluator):
 *         return get_class_from_string(evaluator, TAEvaluator, TA, evaluator_parent_inspection) is not None
 */
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer, __pyx_n_s_is_relevant_evaluation_config); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_is_relevant_evaluation_config, __pyx_t_1) < 0) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_27__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_StrategyOptimizer___reduce_cytho, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__34)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_reduce_cython, __pyx_t_1) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_StrategyOptimizer, (type(self), 0x2d2eb06, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_StrategyOptimizer__set_state(self, __pyx_state)
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_17StrategyOptimizer_29__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_n_s_StrategyOptimizer___setstate_cyt, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__36)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer->tp_dict, __pyx_n_s_setstate_cython, __pyx_t_1) < 0) __PYX_ERR(2, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_7octobot_18strategy_optimizer_18strategy_optimizer_StrategyOptimizer);

  /* "(tree fragment)":1
 * def __pyx_unpickle_StrategyOptimizer(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_7octobot_18strategy_optimizer_18strategy_optimizer_1__pyx_unpickle_StrategyOptimizer, 0, __pyx_n_s_pyx_unpickle_StrategyOptimizer, NULL, __pyx_n_s_octobot_strategy_optimizer_strat_2, __pyx_d, ((PyObject *)__pyx_codeobj__38)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_StrategyOptimizer, __pyx_t_1) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "octobot/strategy_optimizer/strategy_optimizer.py":1
 * #  Drakkar-Software OctoBot             # <<<<<<<<<<<<<<
 * #  Copyright (c) Drakkar-Software, All rights reserved.
 * #
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init octobot.strategy_optimizer.strategy_optimizer", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init octobot.strategy_optimizer.strategy_optimizer");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      CYTHON_UNUSED Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind;
    Py_ssize_t i, char_pos;
    void *result_udata;
#if CYTHON_PEP393_ENABLED
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    result_udata = PyUnicode_DATA(result_uval);
#else
    result_uval = PyUnicode_FromUnicode(NULL, result_ulength);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = sizeof(Py_UNICODE);
    result_udata = PyUnicode_AS_UNICODE(result_uval);
#endif
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = PyTuple_GET_ITEM(value_tuple, i);
        if (unlikely(__Pyx_PyUnicode_READY(uval)))
            goto bad;
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        if (unlikely(!ulength))
            continue;
        if (unlikely(char_pos + ulength < 0))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (!CYTHON_PEP393_ENABLED || ukind == result_ukind) {
            memcpy((char *)result_udata + char_pos * result_ukind, udata, (size_t) (ulength * result_ukind));
        } else {
            #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030300F0 || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    result_ulength++;
    value_count++;
    return PyUnicode_Join(__pyx_empty_unicode, value_tuple);
#endif
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a - b);
            if (likely((x^a) >= 0 || (x^~b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
            }
        }
                x = a - b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla - llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("subtract", return NULL)
            result = ((double)a) - (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceSubtract : PyNumber_Subtract)(op1, op2);
}
#endif

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* CIntToDigits */
static const char DIGIT_PAIRS_10[2*10*10+1] = {
    "00010203040506070809"
    "10111213141516171819"
    "20212223242526272829"
    "30313233343536373839"
    "40414243444546474849"
    "50515253545556575859"
    "60616263646566676869"
    "70717273747576777879"
    "80818283848586878889"
    "90919293949596979899"
};
static const char DIGIT_PAIRS_8[2*8*8+1] = {
    "0001020304050607"
    "1011121314151617"
    "2021222324252627"
    "3031323334353637"
    "4041424344454647"
    "5051525354555657"
    "6061626364656667"
    "7071727374757677"
};
static const char DIGITS_HEX[2*16+1] = {
    "0123456789abcdef"
    "0123456789ABCDEF"
};

/* BuildPyUnicode */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, char* chars, int clength,
                                                int prepend_sign, char padding_char) {
    PyObject *uval;
    Py_ssize_t uoffset = ulength - clength;
#if CYTHON_USE_UNICODE_INTERNALS
    Py_ssize_t i;
#if CYTHON_PEP393_ENABLED
    void *udata;
    uval = PyUnicode_New(ulength, 127);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_DATA(uval);
#else
    Py_UNICODE *udata;
    uval = PyUnicode_FromUnicode(NULL, ulength);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_AS_UNICODE(uval);
#endif
    if (uoffset > 0) {
        i = 0;
        if (prepend_sign) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, 0, '-');
            i++;
        }
        for (; i < uoffset; i++) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, i, padding_char);
        }
    }
    for (i=0; i < clength; i++) {
        __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, uoffset+i, chars[i]);
    }
#else
    {
        PyObject *sign = NULL, *padding = NULL;
        uval = NULL;
        if (uoffset > 0) {
            prepend_sign = !!prepend_sign;
            if (uoffset > prepend_sign) {
                padding = PyUnicode_FromOrdinal(padding_char);
                if (likely(padding) && uoffset > prepend_sign + 1) {
                    PyObject *tmp;
                    PyObject *repeat = PyInt_FromSize_t(uoffset - prepend_sign);
                    if (unlikely(!repeat)) goto done_or_error;
                    tmp = PyNumber_Multiply(padding, repeat);
                    Py_DECREF(repeat);
                    Py_DECREF(padding);
                    padding = tmp;
                }
                if (unlikely(!padding)) goto done_or_error;
            }
            if (prepend_sign) {
                sign = PyUnicode_FromOrdinal('-');
                if (unlikely(!sign)) goto done_or_error;
            }
        }
        uval = PyUnicode_DecodeASCII(chars, clength, NULL);
        if (likely(uval) && padding) {
            PyObject *tmp = PyNumber_Add(padding, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
        if (likely(uval) && sign) {
            PyObject *tmp = PyNumber_Add(sign, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
done_or_error:
        Py_XDECREF(padding);
        Py_XDECREF(sign);
    }
#endif
    return uval;
}

/* CIntToPyUnicode */
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned short    uint16_t;
        #else
           typedef unsigned __int16  uint16_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define GCC_DIAGNOSTIC
#endif
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(int)*3+2];
    char *dpos, *end = digits + sizeof(int)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    int remaining;
#ifdef GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (int) (remaining / (8*8));
            dpos -= 2;
            *(uint16_t*)dpos = ((const uint16_t*)DIGIT_PAIRS_8)[digit_pos];
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (int) (remaining / (10*10));
            dpos -= 2;
            *(uint16_t*)dpos = ((const uint16_t*)DIGIT_PAIRS_10)[digit_pos];
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (int) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    if (last_one_off) {
        assert(*dpos == '0');
        dpos++;
    }
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (descr != NULL) {
        *method = descr;
        return 0;
    }
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(name));
#endif
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError */
static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else if (PyTuple_GET_SIZE(t) < index) {
      __Pyx_RaiseNeedMoreValuesError(PyTuple_GET_SIZE(t));
    } else {
      __Pyx_RaiseTooManyValuesError(index);
    }
}

/* UnpackTuple2 */
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_COMPILING_IN_PYPY
    value1 = PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_COMPILING_IN_PYPY
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = Py_TYPE(iter)->tp_iternext;
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_COMPILING_IN_PYPY
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#elif PY_MAJOR_VERSION >= 3
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        PyObject *key, *value;
        if (unlikely(orig_length != PyDict_Size(iter_obj))) {
            PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
            return -1;
        }
        if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
            return 0;
        }
        if (pitem) {
            PyObject* tuple = PyTuple_New(2);
            if (unlikely(!tuple)) {
                return -1;
            }
            Py_INCREF(key);
            Py_INCREF(value);
            PyTuple_SET_ITEM(tuple, 0, key);
            PyTuple_SET_ITEM(tuple, 1, value);
            *pitem = tuple;
        } else {
            if (pkey) {
                Py_INCREF(key);
                *pkey = key;
            }
            if (pvalue) {
                Py_INCREF(value);
                *pvalue = value;
            }
        }
        return 1;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyTuple_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyList_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* FetchCommonType */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunctionShared */
#include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromString(m->func.m_ml->ml_name);
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    if (unlikely(op == NULL))
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* PyObjectFormat */
#if CYTHON_USE_UNICODE_WRITER
static PyObject* __Pyx_PyObject_Format(PyObject* obj, PyObject* format_spec) {
    int ret;
    _PyUnicodeWriter writer;
    if (likely(PyFloat_CheckExact(obj))) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x03040000
        _PyUnicodeWriter_Init(&writer, 0);
#else
        _PyUnicodeWriter_Init(&writer);
#endif
        ret = _PyFloat_FormatAdvancedWriter(
            &writer,
            obj,
            format_spec, 0, PyUnicode_GET_LENGTH(format_spec));
    } else if (likely(PyLong_CheckExact(obj))) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x03040000
        _PyUnicodeWriter_Init(&writer, 0);
#else
        _PyUnicodeWriter_Init(&writer);
#endif
        ret = _PyLong_FormatAdvancedWriter(
            &writer,
            obj,
            format_spec, 0, PyUnicode_GET_LENGTH(format_spec));
    } else {
        return PyObject_Format(obj, format_spec);
    }
    if (unlikely(ret == -1)) {
        _PyUnicodeWriter_Dealloc(&writer);
        return NULL;
    }
    return _PyUnicodeWriter_Finish(&writer);
}
#endif

/* PyUnicode_Substring */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Substring(
            PyObject* text, Py_ssize_t start, Py_ssize_t stop) {
    Py_ssize_t length;
    if (unlikely(__Pyx_PyUnicode_READY(text) == -1)) return NULL;
    length = __Pyx_PyUnicode_GET_LENGTH(text);
    if (start < 0) {
        start += length;
        if (start < 0)
            start = 0;
    }
    if (stop < 0)
        stop += length;
    else if (stop > length)
        stop = length;
    if (stop <= start)
        return PyUnicode_FromUnicode(NULL, 0);
#if CYTHON_PEP393_ENABLED
    return PyUnicode_FromKindAndData(PyUnicode_KIND(text),
        PyUnicode_1BYTE_DATA(text) + start*PyUnicode_KIND(text), stop-start);
#else
    return PyUnicode_FromUnicode(PyUnicode_AS_UNICODE(text)+start, stop-start);
#endif
}

/* UnpackUnboundCMethod */
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    }
#endif
    return 0;
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    if (likely(cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            #if PY_VERSION_HEX >= 0x030700A0
            return (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)cfunc->func)(self, args, 2);
            #else
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, 2, NULL);
            #endif
        }
        #if PY_VERSION_HEX >= 0x030700A0
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, 2, NULL);
        #endif
    }
    return __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(3);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 1, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 2, arg2);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(3, self, arg1, arg2);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* CallUnboundCMethod1 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    if (likely(cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return (*(cfunc->func))(self, arg);
        } else if (PY_VERSION_HEX >= 0x030600B1 && flag == METH_FASTCALL) {
            if (PY_VERSION_HEX >= 0x030700A0) {
                return (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)cfunc->func)(self, &arg, 1);
            } else {
                return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
            }
        } else if (PY_VERSION_HEX >= 0x030700A0 && flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)cfunc->func)(self, &arg, 1, NULL);
        }
    }
    return __Pyx__CallUnboundCMethod1(cfunc, self, arg);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)(void*)(PyCFunction)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 1, arg);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(2, self, arg);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* py_dict_pop */
static CYTHON_INLINE PyObject *__Pyx_PyDict_Pop(PyObject *d, PyObject *key, PyObject *default_value) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030600B3
    if ((1)) {
        return _PyDict_Pop(d, key, default_value);
    } else
#endif
    if (default_value) {
        return __Pyx_CallUnboundCMethod2(&__pyx_umethod_PyDict_Type_pop, d, key, default_value);
    } else {
        return __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_pop, d, key);
    }
}

/* PyObjectCallMethod1 */
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
}

/* append */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x) {
    if (likely(PyList_CheckExact(L))) {
        if (unlikely(__Pyx_PyList_Append(L, x) < 0)) return -1;
    } else {
        PyObject* retval = __Pyx_PyObject_CallMethod1(L, __pyx_n_s_append, x);
        if (unlikely(!retval))
            return -1;
        Py_DECREF(retval);
    }
    return 0;
}

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* PatchModuleWithCoroutine */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchInspect */
static PyObject* __Pyx_patch_inspect(PyObject* module) {
#if defined(__Pyx_Generator_USED) && (!defined(CYTHON_PATCH_INSPECT) || CYTHON_PATCH_INSPECT)
    static int inspect_patched = 0;
    if (unlikely((!inspect_patched) && module)) {
        module = __Pyx_Coroutine_patch_module(
            module, ""
"old_types = getattr(_module.isgenerator, '_cython_generator_types', None)\n"
"if old_types is None or not isinstance(old_types, set):\n"
"    old_types = set()\n"
"    def cy_wrap(orig_func, type=type, cython_generator_types=old_types):\n"
"        def cy_isgenerator(obj): return type(obj) in cython_generator_types or orig_func(obj)\n"
"        cy_isgenerator._cython_generator_types = cython_generator_types\n"
"        return cy_isgenerator\n"
"    _module.isgenerator = cy_wrap(_module.isgenerator)\n"
"old_types.add(_cython_generator_type)\n"
        );
        inspect_patched = 1;
    }
#else
    if ((0)) return __Pyx_Coroutine_patch_module(module, NULL);
#endif
    return module;
}

/* PatchAsyncIO */
static PyObject* __Pyx_patch_asyncio(PyObject* module) {
#if PY_VERSION_HEX < 0x030500B2 &&\
        (defined(__Pyx_Coroutine_USED) || defined(__Pyx_Generator_USED)) &&\
        (!defined(CYTHON_PATCH_ASYNCIO) || CYTHON_PATCH_ASYNCIO)
    PyObject *patch_module = NULL;
    static int asyncio_patched = 0;
    if (unlikely((!asyncio_patched) && module)) {
        PyObject *package;
        package = __Pyx_Import(__pyx_n_s_asyncio_coroutines, NULL, 0);
        if (package) {
            patch_module = __Pyx_Coroutine_patch_module(
                PyObject_GetAttrString(package, "coroutines"), ""
"try:\n"
"    coro_types = _module._COROUTINE_TYPES\n"
"except AttributeError: pass\n"
"else:\n"
"    if _cython_coroutine_type is not None and _cython_coroutine_type not in coro_types:\n"
"        coro_types = tuple(coro_types) + (_cython_coroutine_type,)\n"
"    if _cython_generator_type is not None and _cython_generator_type not in coro_types:\n"
"        coro_types = tuple(coro_types) + (_cython_generator_type,)\n"
"_module._COROUTINE_TYPES = coro_types\n"
            );
        } else {
            PyErr_Clear();
            package = __Pyx_Import(__pyx_n_s_asyncio_tasks, NULL, 0);
            if (unlikely(!package)) goto asyncio_done;
            patch_module = __Pyx_Coroutine_patch_module(
                PyObject_GetAttrString(package, "tasks"), ""
"if hasattr(_module, 'iscoroutine'):\n"
"    old_types = getattr(_module.iscoroutine, '_cython_coroutine_types', None)\n"
"    if old_types is None or not isinstance(old_types, set):\n"
"        old_types = set()\n"
"        def cy_wrap(orig_func, type=type, cython_coroutine_types=old_types):\n"
"            def cy_iscoroutine(obj): return type(obj) in cython_coroutine_types or orig_func(obj)\n"
"            cy_iscoroutine._cython_coroutine_types = cython_coroutine_types\n"
"            return cy_iscoroutine\n"
"        _module.iscoroutine = cy_wrap(_module.iscoroutine)\n"
"    if _cython_coroutine_type is not None:\n"
"        old_types.add(_cython_coroutine_type)\n"
"    if _cython_generator_type is not None:\n"
"        old_types.add(_cython_generator_type)\n"
            );
        }
        Py_DECREF(package);
        if (unlikely(!patch_module)) goto ignore;
asyncio_done:
        PyErr_Clear();
        asyncio_patched = 1;
#ifdef __Pyx_Generator_USED
        {
            PyObject *inspect_module;
            if (patch_module) {
                inspect_module = PyObject_GetAttr(patch_module, __pyx_n_s_inspect);
                Py_DECREF(patch_module);
            } else {
                inspect_module = __Pyx_Import(__pyx_n_s_inspect, NULL, 0);
            }
            if (unlikely(!inspect_module)) goto ignore;
            inspect_module = __Pyx_patch_inspect(inspect_module);
            if (unlikely(!inspect_module)) {
                Py_DECREF(module);
                module = NULL;
            }
            Py_XDECREF(inspect_module);
        }
#else
        if ((0)) return __Pyx_patch_inspect(module);
#endif
    }
    return module;
ignore:
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch asyncio package with custom generator type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    if ((0)) return __Pyx_patch_inspect(__Pyx_Coroutine_patch_module(module, NULL));
#endif
    return module;
}

/* ClassMethod */
static PyObject* __Pyx_Method_ClassMethod(PyObject *method) {
#if CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM <= 0x05080000
    if (PyObject_TypeCheck(method, &PyWrapperDescr_Type)) {
        return PyClassMethod_New(method);
    }
#else
#if CYTHON_COMPILING_IN_PYSTON || CYTHON_COMPILING_IN_PYPY
    if (PyMethodDescr_Check(method))
#else
    static PyTypeObject *methoddescr_type = NULL;
    if (methoddescr_type == NULL) {
       PyObject *meth = PyObject_GetAttrString((PyObject*)&PyList_Type, "append");
       if (!meth) return NULL;
       methoddescr_type = Py_TYPE(meth);
       Py_DECREF(meth);
    }
    if (__Pyx_TypeCheck(method, methoddescr_type))
#endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject *)method;
        #if PY_VERSION_HEX < 0x03020000
        PyTypeObject *d_type = descr->d_type;
        #else
        PyTypeObject *d_type = descr->d_common.d_type;
        #endif
        return PyDescr_NewClassMethod(d_type, descr->d_method);
    }
#endif
    else if (PyMethod_Check(method)) {
        return PyClassMethod_New(PyMethod_GET_FUNCTION(method));
    }
    else if (PyCFunction_Check(method)) {
        return PyClassMethod_New(method);
    }
#ifdef __Pyx_CyFunction_USED
    else if (__Pyx_CyFunction_Check(method)) {
        return PyClassMethod_New(method);
    }
#endif
    PyErr_SetString(PyExc_TypeError,
                   "Class-level classmethod() can only be called on "
                   "a method_descriptor or instance method.");
    return NULL;
}

/* GetNameInClass */
static PyObject *__Pyx_GetGlobalNameAfterAttributeLookup(PyObject *name) {
    PyObject *result;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    __Pyx_GetModuleGlobalNameUncached(result, name);
    return result;
}
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name) {
    PyObject *result;
    result = __Pyx_PyObject_GetAttrStr(nmspace, name);
    if (!result) {
        result = __Pyx_GetGlobalNameAfterAttributeLookup(name);
    }
    return result;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
